//
// Active contour example
//
// Bala Amavasai (bala@amavasai.org)
// Sun Apr 29 22:57:26 BST 2001
//

#include <iostream>
#include "mm_active_contour.h"
#include "mm_image.h"
#include "mm_image_fileinput.h"
#include "mm_image_qtoutput.h"

using namespace std; 
using namespace mimas;

int main(int argc, char **argv)
{
   int retVal = 0;
   try {
	mm_activeContour<int> ac;
	mm_image<int> inputimage, initialimage;

   QApplication app( argc, argv );
   mm_image_qtoutput< unsigned char > display( app );

   MMERROR( argc >= 6, mm_exception, ,
            "Syntax : " << argv[0] << " <pgm file> <circleCentre x> "
            "<circleCentre y> <circleRadius> <numPoints> "
            "[<gray level [0, 255]>] [SMOOTH; 1 = yes, 0 = no] "
            "[DEBUG; 1 = yes, 0 = no]" );

	int xcenter = atoi(argv[2]);
	int ycenter = atoi(argv[3]);
	int circleRadius = atoi(argv[4]);
	int numPoints = atoi(argv[5]);
	if (argc >= 7 && atoi(argv[6]) >= 0 && atoi(argv[6]) <= 255)
     ac.showPolygonOnImage_graylevel = atoi(argv[6]);
	if (argc >= 8 && atoi(argv[7]) == 1) ac.smooth = true;
#ifndef NDEBUG
	if (argc >= 9 && atoi(argv[8]) == 1) ac.debug = true;
#endif

	// read in input image
	ifstream infile1( argv[1], ios::binary );
	infile1 >> inputimage;
	ac.setPolyCircle(xcenter, ycenter, circleRadius, numPoints);

	// show initial contour on image
	ifstream infile2( argv[1], ios::binary );
	infile2 >> initialimage;
	ac.showPolygonOnImage(initialimage);
   display << initialimage << mimas::pause();

	// adapt contour - also see adaptContourList function
	ac.adaptContour(inputimage);

	// show output
	ac.showPolygonOnImage(inputimage);
   display << inputimage << mimas::pause();

	// show final coordinate set
	ac.showPolyCoords();
   } catch ( exception &e ) {
     cerr << e.what() << endl;
     retVal = 1;
   };
   return retVal;
}
