#include <aalib.h>
#include <stdlib.h>
#include <fstream>
#include <iomanip>
#include <iostream>
#include <sstream>
#include "mm_image.h"
#include "mm_image_funcs.h"
#include "mm_image_v4linput.h"

// Ascii-camera using aalib - Jan Wedekind - 1 Mar 2005 (C)

using namespace mimas;
using namespace std;

mm_image< unsigned char > sample( const mm_image< unsigned char > &image, int width, int height )
{
  mm_image< unsigned char > retVal;
  retVal.init( width, height );
  unsigned char *p = retVal.rawData().data();
  for ( int y=0; y<height; y++ ) {
    int yy = y * image.getHeight() / height;
    const unsigned char *q = image.rawData().data() + yy * image.getWidth();
    for ( int x=0; x<width; x++ ) {
      int xx = x * image.getWidth() / width;
      *p++ = q[ xx ];
    };
  };
  return retVal;
}

// Global environment (for output message redirection etc) -----------------

int main( int argc, char *argv[] )
{
  int retVal = 0;
  aa_context *context = NULL;
  int keyboard = 0;

  try {

    cerr << "Space-key takes a screenshot. ESC-key terminates program." << endl;

    mm_image_v4linput< mm_rgba< unsigned char > > webcam( "/dev/video" );
#ifndef NDEBUG
    cerr << "Grabbing images of size " << webcam.getWidth() << "x" << webcam.getHeight() << endl;
#endif
    
    struct aa_hardware_params params;
    memcpy( &params, &aa_defparams, sizeof(struct aa_hardware_params) );
    
    params.supported = AA_NORMAL_MASK;
    params.width = 80;
    params.height = 40;

    struct aa_renderparams renderParams;
    memcpy( &renderParams, &aa_defrenderparams, sizeof(struct aa_renderparams) );

    MMERROR( aa_parseoptions( &params, &renderParams, &argc, argv ), mm_exception, ,
	     "Error parsing command-line." );
    MMERROR( argc <= 1, mm_exception, ,
	     "Leftover command-line argument '" << argv[1] << '\'' );

    MMERROR( context = aa_autoinit( &params ), mm_exception, ,
	     "Couldn't initialise aalib" );

    MMERROR( keyboard = aa_autoinitkbd( context, 0 ), mm_exception, ,
	     "Failed initialising aalib keyboard driver" );

    bool loop = true;
    int screenshot = 0;

    while ( loop ) {

      // Grab image
      mm_image< mm_rgba< unsigned char > > colorImage;
      webcam >> colorImage;
      mm_image< unsigned char > image( sample( colorImage,
                                               2 * params.width,
                                               2 * params.height ) );
      equalise( image );
      memcpy( aa_image( context ), image.rawData().data(), 4 * params.width * params.height );
      aa_render( context, &renderParams, 0, 0, params.width, params.height );
      aa_flush( context );

      int key;
      switch ( key = aa_getevent( context, 0 ) ) {
      case AA_NONE:
	break;
      case AA_ESC:
	loop = false;
	break;
      case ' ': {
	ostringstream fileName; fileName << "screenshot" << setfill( '0' ) << setw( 4 ) << screenshot++ << ".txt";
	ofstream file( fileName.str().c_str() );
	const char *p = (const char *)aa_text( context );
	for ( int y=0; y<params.height; y++ ) {
	  file.write( p, params.width );
	  file << endl;
	  p += params.width;
	};
      };
#ifndef NDEBUG
      default:
	cerr << "key " << key << " was pressed." << endl;
#endif
      };

    };

    assert( keyboard != 0 );
    assert( context != NULL );
    aa_uninitkbd( context );
    aa_close( context );

  } catch ( exception &e ) {

    if ( keyboard ) {
      assert( context != NULL );
      aa_uninitkbd( context );
    };
    if ( context != NULL ) aa_close( context );

    cerr << e.what() << endl
	 << "Usage: " << argv[0] << " [options]" << endl
	 << "Options:" << endl
	 << aa_help;
    
  };

  return retVal;

}
