// example applications to use the base classes / miniman pairwise libs
// stuart meikle Thu Mar  2 10:38:28 2000
//
#include <boost/numeric/ublas/vector.hpp>
#include <cassert>
#include <qpainter.h>
#include "autoPghWidget.h"
#include "mm_linalg.h"

using namespace boost;
using namespace std;
using namespace mimas;

typedef numeric::ublas::vector< double > Vector;

AutoPghWidget::AutoPghWidget( QWidget *parent, const char *name, WFlags f ):
  QWidget( parent, name, f ), house( new mimas::mm_pgh_2dobject ),
  geom( new mm_line_ptr_list )
{
  /*
    cout << endl;
    cout << "Viarga v20000708 Copyright (C) Stuart Meikle 2000" << endl;
    cout << "--------------------------------------------------------------------" << endl;
    cout << "This program is free software; you can redistribute it and/or modify" << endl;
    cout << "it under the terms of the GNU General Public License as published by" << endl;
    cout << "the Free Software Foundation (www.fsf.org)." << endl;
    cout << endl;
    cout << "original inspiration from Tinatools (www.niac.man.ac.uk/Tina)," << endl;
    cout << "this code written by Stuart Meikle." << endl << endl;
   */
  //Now go!
  //(1) create an object -> a square in this case
  //straight line segments would be loaded from ccd extracted lines
  //ordinarily
  geom->push_back
    ( mm_line_ptr( new mm_line(  50,   0, 100,  50 ) ) );
  geom->push_back
    ( mm_line_ptr( new mm_line( 100,  50, 100, 100 ) ) );
  geom->push_back
    ( mm_line_ptr( new mm_line( 100, 100,   0, 100 ) ) );
  geom->push_back
    ( mm_line_ptr( new mm_line(   0, 100,   0,  50 ) ) );
  geom->push_back
    ( mm_line_ptr( new mm_line(   0,  50,  50,   0 ) ) );
    
  house->setName("house");
  house->createFromLines( *geom );

  // longer lifetime of geom required?
#ifndef NDEBUG
  cerr << "Created the house object" << endl;
#endif
    
  camera_scene.addModel( house );
  srand48( time( NULL ));

  startTimer( 1000 );
  setBackgroundColor( white );		// white background
}

void AutoPghWidget::timerEvent( QTimerEvent * )
{
  shared_ptr< mm_line_ptr_list >
    geomnew( new mm_line_ptr_list );

  // Add random lines as noise.    
  for(int i=0;i<0;i++)
    {
      double	cx,cy,s;
      double	a = drand48() * 2.0 * M_PI;
	    
      cx = drand48() *640;
      cy = drand48() *480;
      s  = drand48() * 25;
	    
      geomnew->push_back( mm_line_ptr
                          ( new mm_line( cx + s*cos(a), cy + s*sin(a), 
                                         cx - s*cos(a), cy - s*sin(a) ) ) );
    }
        
  // Add several real houses at some locations
  double	cx,cy;
  for(int i=0;i<1;i++)
    {

      Vector s(2), e(2), shift(2);
      mm_angle	a;
	    
      cx = drand48() * 640;
      cy = drand48() * 480;
      shift( 0 ) = cx; shift( 1 ) = cy;
      a  = mm_angle(drand48() * M_PI *2.0);
	    	    
      s( 0 ) =  50; s( 1 ) =   0;
      e( 0 ) = 100; e( 1 ) =  50;
      s  = rotate( s, a );
      e  = rotate( e, a );
      s = s + shift;
      e = e + shift;
      geomnew->push_back( mm_line_ptr( new mm_line( s, e ) ) );

      
      s( 0 ) = 100; s( 1 ) =  50;
      e( 0 ) = 100; e( 1 ) = 100;
      s  = rotate( s, a );
      e  = rotate( e, a );
      s = s + shift;
      e = e + shift;
      geomnew->push_back( mm_line_ptr( new mm_line( s, e ) ) );

      s( 0 ) = 100; s( 1 ) = 100;
      e( 0 ) =   0; e( 1 ) = 100;
      s  = rotate( s, a );
      e  = rotate( e, a );
      s = s + shift;
      e = e + shift;
      geomnew->push_back( mm_line_ptr( new mm_line( s, e ) ) );

      s( 0 ) =   0; s( 1 ) = 100;
      e( 0 ) =   0; e( 1 ) =  50;
      s  = rotate( s, a );
      e  = rotate( e, a );
      s = s + shift;
      e = e + shift;
      geomnew->push_back( mm_line_ptr( new mm_line( s, e ) ) );

      s( 0 ) =   0; s( 1 ) =  50;
      e( 0 ) =  50; e( 1 ) =   0;
      s  = rotate( s, a );
      e  = rotate( e, a );
      s = s + shift;
      e = e + shift;
      geomnew->push_back( mm_line_ptr( new mm_line( s, e ) ) );

      // end of 1 house
    }

  time_t		starttime,endtime, histtime;
  starttime = time(NULL);
  camera_scene.updateSceneLines( *geomnew, 640,480 );
    
  histtime  = time(NULL);
    
  //export the scene lines to somewhere the GUI can see it!
  geom = geomnew;

#ifndef NDEBUG
  cerr << "Looking for the house ..." << endl;
#endif
  houses = shared_ptr< mm_object_instance_list >
    ( camera_scene.findModel( house ) );
    
  endtime   = time(NULL);
    
  //now ping the gui to update itself
    
  //camera_scene.findModel( &square );
  update();// my_user_interface.refreshGUI();
	
    
  cout << "total search time = " << endtime-starttime << "seconds " << endl;
  cout << "( " << histtime - starttime << " seconds to generate hists for the 104 scene lines " << endl;
  cout << "  and " << endtime - histtime << " seconds to compare and locate the objects. " << endl;
  cout << "------------------------------------------------------------------" << endl << endl;
  
  // geom = geomnew;
}

void AutoPghWidget::paintEvent( QPaintEvent * )
{
  QPainter paint( this );

  //draw the current input scene 
  assert( geom );

  paint.setPen( Qt::black );

  for( mm_line_ptr_list::iterator i=geom->begin();
       i!=geom->end(); i++ )
    {
      mm_line_ptr ln = (*i);
      const Vector *start, *end;
		
      start = &ln->getStart();
      end   = &ln->getEnd();
		
      paint.drawLine( (int)(*start)(0), (int)(*start)(1),
                      (int)(*end)(0), (int)(*end)(1) );
    }	

  if (houses)
    {
      //I can't be bothered to draw the actual shapes... I'll just mark the position
      //and orientation for now
      for( mm_coordinate_xform_ptr_list::iterator it =
             houses->getList().begin(); it != houses->getList().end();
           it++ ) {
        mm_coordinate_xform_ptr x = *it;
        
        double	x1,x2,y1,y2;
        Vector d( x->getDisplacement() );
        double	a = (double)x->getRotation();
        
        paint.setPen(Qt::red);
        x1 = d(0) + cos( a ) * 10;
        y1 = d(1) + sin( a ) * 10;
        x2 = d(0) - cos( a ) * 10;
        y2 = d(1) - sin( a ) * 10;
        paint.drawLine( (int) x1, (int) y1, (int)x2, (int) y2 );
        paint.setPen(Qt::blue);
        paint.drawLine( (int) x1, (int) y1, (int)x1, (int) y1 );
      }
    }
}
