#include <iostream>
#include <fstream>
#include <string>
#include <mm_image.h>
#include <mm_image_fileinput.h>
#include <mm_image_qtoutput.h>
#include <mm_image_funcs.h>
#include <mm_nn_assoc.h>

using namespace boost::numeric;
using namespace std;
using namespace mimas;

int main( int argc, char *argv[] )
{
  int retVal = 0;
  try {
   QApplication app( argc, argv );
	mm_nn_assoc nn;
	mm_image<int> im;
	int numwidth, numheight;
   mm_image_qtoutput< unsigned char > display( app );


	cout << endl << endl;
	cout << "************************************************************************" << endl;
	cout << "  This program demonstrates Hebb heteroassociative training and recall" << endl;
	cout << "************************************************************************" << endl;


	// Width and height of each character
	numwidth=11;
	numheight=15;

   ublas::matrix< double >
     x(10,numwidth*numheight), // sample numbers are are 15x7 pixels
     y(10,10); // target pattern 10 pattern combinations

	// read in patters and store it as input(x) and output(y) pairs
	for (int num=0; num<10; num++)
	{
		char fn[80];
		sprintf(fn,"numbers/%d.pgm",num);
      ifstream file( fn, ios::binary );
      file >> im;

		int count=0;
		for (int j=0; j<im.getHeight(); j++)
			for (int i=0; i<im.getWidth(); i++)
			{
				x(num,count)=(double)(im.getPixel(i,j)>127?1:0);
				count++;
			}
		y(num,num)=1.0;
		
	}
	
	
	// train pattern
	cout << endl << "Training patterns now.......";
	nn.train(x,y);
	cout << "training completed" << endl << endl;

	// recognise on new image
	
   ublas::vector< double > pat( numheight*numwidth );
	
	// read in new image
	ifstream file( "numbers/numbers-noise.pgm", ios::binary );
	file >> im;
	//readFromFile(im,"numbers/numbers.pgm");
	cout << "Note down number that appear in noisy image display and close display to continue" << endl << endl;
	display << im << mimas::pause();
	cout << endl << "I'm now going to attempt to recognise these numbers. they are :         ";

	int l=0; // increment this by numheight if there is more than one row
	for (int k=0; k<im.getWidth(); k+=numwidth)
	{

		// store 15x7 pattern into 7x15 matrix
		int count=0;
		for (int j=l; j<l+numheight; j++)
			for (int i=k; i<k+numwidth; i++)
			{
				pat(count)=(double)(im.getPixel(i,j)>127?1:0);
				count++;
			}
		// recall the pattern and store it in y
      ublas::vector< double > y( nn.recall(pat) );

		// y is a 1x10 matrix. The highest value indicates a
		// match since everywhere else is zero. Can improve this
		// by doing some filtering but I can't really be bothered
		double maxval=0.0;
		int outnum=0;
		for (int index=0; index<(signed)y.size(); index++)
		{
			if (y(index)>maxval)
			{
				maxval=y(index);
				outnum=index;
			}
		}
		// here is the recognised number
		cout << outnum ;
	}

	cout << endl << endl << "Was I successful? I'm a bad mutha ain't I?" << endl ;
	cout << "But seriously heteroassociative ANNS are very rigid and not very good at interpolating solutions" << endl << endl;
  } catch ( exception &e ) {
    cerr << e.what() << endl;
    retVal = 1;
  };	

  return retVal;
}
