//
// stumeikle
// canny (with postprocessing) example
//


#include <iostream>
#include <fstream>
#include <vector>
#include "mm_image.h"
#include "mm_image_fileinput.h"
#include "mm_image_qtoutput.h"
#include "mm_image_funcs.h"
#include "mm_property_image.h"
#include "mm_canny_tool.h"
#include "mm_corner_tool.h"

using namespace std;
using namespace mimas;

int main(int argc, char **argv)
{
  int retVal = 0;
  try {
    QApplication app( argc, argv );
    mm_image<int> 		input;
    mm_canny_tool<int>		canny;
    mm_corner_tool<int>		corner;
    mm_corner_image		cim;
    mm_property_image		pim;
    mm_edge_string_ptr_list	edge_strings;
    mm_image_qtoutput< mm_rgba< unsigned char > > display( app );

    ifstream file( argv[1], ios::binary );
    MMERROR( file >> input, mm_exception, , "Error loading file \""
             << argv[1] << "\"." );

    //extract features
    corner.filterImage(input, cim);
    edge_strings=canny.findEdgeStrings( input );
    // pim.setOwnership(false);
    pim.init( input.getWidth(), input.getHeight());
    pim.addStrings( edge_strings );

    //then create an image to contain them for visualisation
    //be very careful about who owns the data!
    pim+=cim;
    
    displayWithOverlay( display, input, pim ); display << mimas::pause();
    
    // delete the strings
    // mm_edge_string_ptr_list::iterator	sit;
    // for(sit=edge_strings.begin();sit!=edge_strings.end();++sit)
    // delete (*sit);
  } catch ( exception &e ) {
    cerr << e.what() << endl;
    retVal = 1;
  };
  return retVal;
}


