//
// $Header: /cvs/mimas/examples/cornerwise/main.cc,v 1.14 2005/06/03 13:49:52 engjw Exp $


#include <iostream>
#include <fstream>
#include <vector>
#include <sstream>

#include "mm_canny_tool.h"
#include "mm_cgh.h"
#include "mm_cgh_scene.h"
#include "mm_cgh_wrapper.h"
#include "mm_corner.h"
#include "mm_corner_tool.h"
#include "mm_edge_string_op.h"
#include "mm_filter.h"
#include "mm_image.h"
#include "mm_image_fileinput.h"
#include "mm_image_funcs.h"
#include "mm_mesa_onscreen.h"
#include "mm_prop.h"
#include "mm_x11_window.h"


using namespace std;
using namespace mimas;

class corner_window
{
 public:
  corner_window( mm_x11_display_ptr display, int _width, int _height,
                 const mm_image< int > &_image,
                 vector< mm_cgh_wrapper_ptr > _list ):
    window( new mm_x11_window( display, _width, _height, &callback, this ) ),
    image( _image ), list(_list) {}
 protected:
  static void callback( void *arg ) { ((corner_window *)arg)->paint(); }
  void paint(void) {

    mm_mesa_onscreen mesa( window );
    glPixelStorei( GL_UNPACK_ALIGNMENT, 1 );
    glRasterPos2i( -1, 1 );
    glPixelZoom( 1, -1 );
    glDrawPixels( window->get_width(), window->get_height(),
                  GL_BGRA, GL_UNSIGNED_BYTE, image.rawData().data() );

    mesa.ortho2D();
    mesa.setColor( mm_rgba< unsigned char >( 0, 255, 0 ) );

    int c = 0;
    for ( vector< mm_cgh_wrapper_ptr >::const_iterator i = 
            list.begin(); i!= list.end(); i++, c++ ) {

      int
        x = (int)(*i)->position(0),
        y = (int)(*i)->position(1);
      
      glBegin( GL_LINES ); {
        glVertex2i( x - 1, y - 1 );
        glVertex2i( x + 1, y + 1 );
        glVertex2i( x + 1, y - 1 );
        glVertex2i( x - 1, y + 1 );
      }; glEnd();
      
      ostringstream stream; stream << c;
      glPushMatrix(); {
        glTranslated( x + 4, y + 14, 0 );
        glScaled( 0.08, -0.08, 1.0 );
        mesa.text( stream.str() );
      }; glPopMatrix();

    };
    
    mesa.finish();

  };

  mm_x11_window_ptr window;
  mm_image< mm_rgba< unsigned char > > image;
  vector< mm_cgh_wrapper_ptr > list;

};

int main(int argc, char **argv)
{
  int retVal = 0;
  try {
	mm_image<int> 		im1,im2;
	mm_cgh_scene<int>		view1,view2;

   mm_x11_display_ptr display( new mm_x11_display );

	ifstream file1( "bpa1.jpg", ios::binary );
	file1 >> im1;
	cout << "Creating CGHs from first image..." << endl;
   
	view1.createFromImage( im1, 0.1 );
	//    view1.dumpAll( "cgh_" );

	ifstream file2( "bpa2.jpg", ios::binary );
	file2 >> im2;
	cout << "Creating CGHs from second image..." << endl;
	view2.createFromImage( im2, 0.1 );

	//match the histograms
	mm_cgh_wrapper_ptr_list	matchlist;
	cout << "Cross comparing features ..." << endl;

	matchlist= view1.crossCompare( view2 );
   
   vector< mm_cgh_wrapper_ptr > list1, list2;

	for( mm_cgh_wrapper_ptr_list::iterator cghit = matchlist.begin();
        cghit != matchlist.end(); cghit++ ) {
     list1.push_back( (*cghit)             );
     list2.push_back( (*cghit)->best_match );
   };

   corner_window
     win1( display, im1.getWidth(), im1.getHeight(), im1, list1 ),
     win2( display, im2.getWidth(), im2.getHeight(), im2, list2 );

	cout << "It finds your spot, irrespective of scale bpa!!" << endl;
	cout << "(image 2 is 50% bigger than image1)" << endl;

   mm_x11_window::process_events( true );

  } catch ( exception &e ) {
    cerr << e.what() << endl;
    retVal = 1;
  };
  return retVal;
}


