// Digital video input
// (C) Jan Wedekind, MMVL 2005
// GPL
#include <iostream>
#include "mm_exception.h"
#include "mm_image_funcs.h"
#include "mm_image_firewireinput.h"
#include "mm_image_mesaoutput.h"

// #define SAVEPICS
#ifdef SAVEPICS
#include <iomanip>
#include <fstream>
#include <sstream>
#include "mm_image_fileoutput.h"
#endif

using namespace std;
using namespace mimas;

int main( int argc, char *argv[] )
{
  int
    retVal = 0;

  try {

    MMERROR( argc == 2, mm_exception, ,
             "Syntax: " << argv[0] << " <number of frames>");

    int nFrames = atoi( argv[1] );

    mm_x11_display_ptr display( new mm_x11_display );

    mm_image_mesaoutput< mm_rgba< unsigned char > > window( display );

    mm_image_firewireinput< mm_rgba< unsigned char > >
      input( MODE_640x480_YUV422, FRAMERATE_15, 0, "/dev/video1394/0" );

    // Set shutter to maximum timing.
    dc1394_feature_info shutter = input.get_feature( FEATURE_SHUTTER );
    input.set_feature_value( FEATURE_SHUTTER,
                             shutter.max );
    
    // Set gain to 50%.
    dc1394_feature_info gain = input.get_feature( FEATURE_GAIN );
    input.set_feature_value( FEATURE_GAIN,
                             ( gain.min + gain.max ) / 2 );

    mm_image< mm_rgba< unsigned char > > image;

    mm_timer t;

    for ( int i=0; i<nFrames; i++ ) {

      input >> image;
      window << image << mimas::delay( 1000 / 15 );
#ifdef SAVEPICS
      ostringstream filename;
      filename << "test" << setfill( '0' ) << setw( 6 ) << i << ".ppm";
      ofstream f( filename.str().c_str(), ios::binary );
      f << setformat( "PPM" ) << image;
#endif

    };

    cout << ( nFrames / t.elapsed() ) << " frames/second." << endl;

  } catch ( exception &e ) {

    // Display error-message.
    cerr << e.what() << endl;

    // Indicate error in return value.
    retVal = 1;

  };
  return retVal;
}
