#include <fstream>
#include "mm_image_fileinput.h"
#include "mm_linalg.h"
#include "mm_mesa_onscreen.h"
#include "mm_pol_model.h"
#include "mm_pol_scene.h"
#include "mm_x11_window.h"

using namespace boost;
using namespace mimas;
using namespace std;

class tux_window
{
 public:
  tux_window( mm_x11_display_ptr display, const mm_image< int > &_image,
              mm_pol_match_ptr_list _matches, mm_line_ptr_list _ml ):
    image( _image ), matches(_matches), ml(_ml),
    window( new mm_x11_window( display, _image.getWidth(), _image.getHeight(),
                               &callback, this ) ) {}
  void wait(void) { window->process_events( true ); }
 protected:
  static void callback( void *arg ) { ((tux_window *)arg)->paint(); }
  void paint(void) {
    mm_mesa_onscreen mesa( window );
    mm_image< mm_rgba< unsigned char > > colourImage( image );
    glPixelStorei( GL_UNPACK_ALIGNMENT, 1 );
    glRasterPos2i( -1, 1 );
    glPixelZoom( 1, -1 );
    glDrawPixels( window->get_width(), window->get_height(),
                  GL_BGRA, GL_UNSIGNED_BYTE, colourImage.rawData().data() );

    mesa.ortho2D();

    mesa.setColor( mm_rgba< unsigned char >( 255, 0, 0 ) );

    glBegin( GL_LINES );

    for( mm_pol_match_ptr_list::const_iterator it=matches.begin();
         it!=matches.end(); it++ ) {

      mm_pol_match_ptr pm=(*it);
	    
      mm_pol_match::Vector d( pm->getPosition() );
      mm_angle	a = (double)pm->getOrientation();
      double		s = pm->getScale();

      //try and draw them properly now
      for( mm_line_ptr_list::const_iterator lineit=ml.begin();
           lineit!=ml.end(); lineit++ ) {

        mm_line_ptr l =(*lineit);
        mm_line::Vector start(2), end(2);

        // For some reason 'rotate( l->getStart() * s, a )' won't compile in
        // release-mode.
        start = l->getStart() * s;
        end   = l->getEnd()   * s;

        start = rotate( start, -a ) + d;
        end   = rotate( end  , -a ) + d;
		  
        glVertex2i( (int)start(0), (int)start(1) );
        glVertex2i( (int)end(0)  , (int)end(1)   );
        
      }
    };

    glEnd();
    
    mesa.finish();
  };
  mm_image< int > image;
  mm_pol_match_ptr_list matches;
  mm_line_ptr_list ml;
  mm_x11_window_ptr window;
};

int main( int argc, char *argv[] )
{
  int retVal = 0;
  try {

    MMERROR( argc == 2, mm_exception, ,
             "Usage: " << argv[0] << " imagefile" << endl
             << "       searches for penguin_model in imagefile" );

    // Open X11 display.
    mm_x11_display_ptr display( new mm_x11_display );

    cout << "Creating all new model..." << endl;

    mm_pol_model p;
    
    //switch on scale invariance AT THE MODEL...
    p.setScaleInvariance( true );
    p.createFromImageFile( "../images/stills/penguin_model.ppm" );

    mm_line_ptr_list ml( p.getModelLines() );

    cout << "...Finished." << endl;

    mm_pol_scene s;
  
    ifstream imfile( argv[1], ios::binary );
    mm_image< int > im;
    imfile >> im;

    s.createFromImage(im);
    cout << "Locating model in scene..." << endl;
    mm_pol_match_ptr_list matches( s.getMatches(p) );

    cout << "Finished searching.. did i find it?" << endl;

    // Create window.
    tux_window window( display, im, matches, ml );

    // Wait for space-key to be pressed.
    window.wait();

  } catch ( exception &e ) {
    cerr << e.what() << endl;
    retVal = 1;
  };
  return retVal;
}
