#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/io.hpp>
#include <fstream>
#include <iostream>
#include "mm_gnuplot.h"
#include "mm_nn_mlp_tool.h"

using namespace boost::numeric;
using namespace std;
using namespace mimas;

int main(int argc, char **argv)
{
	mm_nn_mlp_tool bp;
	ublas::matrix< double > sse, ff;
	mm_gnuplot ffgraph, ssegraph;


	cout << endl << endl << "Using mm_nn_mlp_tool for regression" << endl;
	cout << "-----------------------------------" << endl;
	cout << "The objective of this exercise is to use a neural network" << endl;
	cout << "to fit a cosine wave. The training data is found in file cos.dat." << endl;

	// load training data
   ublas::matrix< double > trainpat;
   ifstream file( "cos.dat" );
   file >> trainpat;
   file.close();

	// initialise a 1-20-5-1 network
	bp.init(1, 20, 5, 1, NULL);

	bp.save( "test.txt" );
	bp.load( "test.txt" );

	// set training limit to 5000 epochs
	bp.setIterations(500);

	// set the transfer function for output layer to be linear
	bp.setTransferFuncLayer(&mm_nn_mlp_tool::transferFuncLinear,4);
	bp.setTransferFuncDerivLayer(&mm_nn_mlp_tool::transferFuncDerivLinear,4);

	// show configuration of network just for debugging purposes
	bp.showConfig();
	cout << "Hidden units transfer function is tanh" << endl;
	cout << "Output units transfer function is linear" << endl;


	cout << "Training over 500 iterations using QuickProp....";
	// train the network using Qprop
	sse=bp.trainQprop(trainpat,0.05,0.5);
	//sse=bp.trainOnline(trainpat);
	//sse=bp.trainBatch(trainpat,0.3,0.05,0.0);
	//sse=bp.trainRprop(trainpat);
	cout << "completed!" << endl << endl;

	// plot the sum-squared-error
	ssegraph.command("set log y\n set grid \n"); // use logscale on y axis an switch grid on
	ssegraph.command("set xlabel 'iteration' \n set ylabel 'sum-squared-error' \n ");
	ssegraph.plotxy( sse,0,1,NULL);

	cout << "Displaying error graph. ";
	cout << "Press <enter> to continue" << endl;
	cin.get();

	//feedforward inputs using column 1 of trainpat
	ublas::matrix< double > testpat(trainpat.size1(),1);
	typedef ublas::matrix_column< ublas::matrix< double > > matrix_column;
	matrix_column( testpat, 0 ) = matrix_column( trainpat, 0 );

	ff=bp.feedForward(testpat);


	// copy and paste original data and neural network estimates into new matrix
	// arrange data so that contents of ffplot data is as follows
	//  col 0 - the input training data
	//  col 1 - the output training data
	//  col 2 - the estimated output from the neural network
	ublas::matrix< double > ffplotdata( trainpat.size1(), 3 ); // three columns
        matrix_column( ffplotdata, 0 ) = matrix_column( trainpat, 0 );
        matrix_column( ffplotdata, 1 ) = matrix_column( trainpat, 1 );
        matrix_column( ffplotdata, 2 ) = matrix_column( ff, 0 );
	
	ffgraph.command("set grid \n set yrange[-1:1] \n"); // plot with grid and set limits on y axis
	ffgraph.command("set xlabel 'input' \n set ylabel 'output' \n ");
	ffgraph.setLineTitle("training","estimated",NULL);
	ffgraph.plotxy( ffplotdata,0,1,2,NULL); // plot the data

	cout << "Displaying training data and neural network fit. ";
	cout << "Press <enter> to continue" << endl;
	cin.get();

	// save the weights of the neural net
	// bp.save("weights.nn");	

	return 0;
}
