#include <fstream>
#include <iostream>
#include <vector>
#include <sstream>
#include <string>
#include "mm_image.h"
#include "mm_image_fileinput.h"
#include "mm_image_qtoutput.h"
#include "mm_opticflow_diff_tool.h"
#include "mm_image_draw.h"

using namespace std;
using namespace mimas;

/**
 * Differential optic flow - see page 196-197 of Trucco and Verri
 * @author Bala Amavasai (bala@amavasai.org)
 * @date Wed Apr 23 16:46:27 BST 2003
 * $Header: /cvs/mimas/examples/opticflow/main.cc,v 1.9 2005/07/28 22:01:08 engmb Exp $
 */

int main( int argc, char *argv[] )
{
  int retVal = 0;
  try {
   QApplication app( argc, argv );
   mm_image_qtoutput< unsigned char > display( app );
	mm_opticflow_diff_tool of;
	mm_image<double> flow;
	mm_image<double> xflow,yflow;
	vector < mm_image<double> > imagein(5);

	// Read in five images
	// note that we are computing the flow based on all five of these
	// images!
   for ( int i=0; i<5; i++ ) {
     ostringstream fileName;
     fileName << "../images/sequences/rubik/rubic." << i << ".pgm\0";
     ifstream file( fileName.str().c_str(), ios::binary );
     file >> imagein[i];
   };

	// Find optic flow field
	of.findFlow(imagein);

	// get flows in x and y directions and store in xflow and yflow respectively
	of.getXflow(xflow);
	of.getYflow(yflow);


	// plot the flows on the centre (third) image and display
	// note the 10x multiplier is just to scale to flow so that it
	// can be seen!
	flow.init(imagein[2].getWidth(),imagein[2].getHeight());
   flow.fill(255);

	// set background to a bright version of the third image
	for (int j=0; j<flow.getHeight(); j++)
		for (int i=0; i<flow.getWidth(); i++)
			flow.setPixel(i,j,(int)(192.0*(double)imagein[2].getPixel(i,j)/255.0)+63);
			
	for (int j=0; j<flow.getHeight(); j++)
	  for (int i=0; i<flow.getWidth(); i++)
	    if ( (xflow.getPixel(i,j)!=0.0) && (yflow.getPixel(i,j)!=0.0) )
	      {
		drawThickLine(flow, i, j, i, j, 0., 3); // plot a thick dot
		drawLine(flow, i,j,i+(int)(10*xflow.getPixel(i,j)),j+(int)(10*yflow.getPixel(i,j)),0.); // draw the line
	      }

	display << flow << mimas::pause();
  } catch ( exception &e ) {
    cerr << e.what() << endl;
    retVal = 1;
  };
  return retVal;
}

