#ifndef MMVL_CAMERA_H_INCLUDED
#define MMVL_CAMERA_H_INCLUDED

#include <iostream>

using std::cout;
using std::cerr;
using std::endl;

#include <boost/numeric/ublas/vector.hpp>
#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/io.hpp>

typedef boost::numeric::ublas::vector<double> vec;
typedef boost::numeric::ublas::matrix<double> mat;

/**
 *
 *Project a point to the image. Camera is oriented towards z positive
 *x axis and y axis as in the image ->left-handed frame
 *
 *Distorsion of the image is not used for the moment
 *
 * Time-stamp: <2005-02-21 15:45:14 engmb>
 * Copyright (C) 2004 by Manuel Boissenin
 *
 * @author Manuel Boissenin
 * @date 14/12/2004
 *
 ***********************************************************************/

class Camera{
  mat calibrMat;
public:

  //  Camera(double fx, double fy, double ox, double oy)
  Camera(const vec &calib)
    :calibrMat(3,3)
  {
    //    assert(calibration_param.size() == 4);
    
    calibrMat(0,0) = calib(0);//fx;
    calibrMat(1,1) = calib(1);//fy;
    calibrMat(0,2) = calib(2);//ox;
    calibrMat(1,2) = calib(3);//oy;
    calibrMat(2,2) = 1;
  }

  vec project(const vec &point)
  {
    assert(point.size() == 3);

    vec projectedPoint(2);
    vec temp = prod(calibrMat, point);
    projectedPoint(0) =  temp(0)/temp(2);
    projectedPoint (1) = temp(1)/temp(2);   
	
    return projectedPoint;
  }
};

#endif
