#include "Camera.h"
#include "Transformation.h"
#include "RANSACPoseEstimation.h"
#include <iostream>
#include "mm_image.h"
#include <vector>

using namespace std;
using namespace mimas;
using namespace boost;

/**
@author: Manuel Boissenin
@date: 28 november 2004
*/

#include <algorithm>

#include <boost/numeric/ublas/vector.hpp>
#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/io.hpp>

typedef boost::numeric::ublas::vector<double> vec;
typedef boost::numeric::ublas::matrix<double> mat;

//This code is used to test the 3D pose estimation.

int main(int argc, char **argv)
{
  try{
    //create model points

    vector<vec> generatedPoints;
    vector<vec>::iterator it3;
    
    mat rotation(3,3);
    
    vec unity(3);
    unity(0) = 1;
    unity(1) = 0;
    unity(2) = 0;

    for(int i = 0; i< 10; i++){
      vec temp = unity * (1+ rand()/(RAND_MAX+1.0)); //norm between 1 and 2
      
      //We turn around y with an angle of [0, PI] and around x [0,2PI]
      
      double theta = 3.14115 * rand()/(RAND_MAX+1.0);
      double phi = 3.14115 *  2 * rand()/(RAND_MAX+1.0);
      
      rotation(0,0) = cos(theta) ;
      rotation(0,1) = 0;
      rotation(0,2) = sin(theta);
      rotation(1,0) = 0 - sin(theta)*sin(phi);
      rotation(1,1) = cos(phi);
      rotation(1,2) = sin(phi)*cos(theta);
      rotation(2,0) = 0 - sin(theta)*cos(phi);
      rotation(2,1) = 0 - sin(phi);
      rotation(2,2) = cos(phi)*cos(theta);
      
      temp = prod(rotation, temp);

      generatedPoints.push_back(temp);
    }

    cerr<<"generated model points: "<<endl;
    copy(generatedPoints.begin(), generatedPoints.end(), ostream_iterator<vec>(cerr,"\n"));

    //Now move the model point in their position in front of the camera.
    //put in a class

    vec transformationP(6);

    transformationP(0) = atof(argv[1]);
    transformationP(1) = atof(argv[2]);
    transformationP(2) = atof(argv[3]);
    transformationP(3) = atof(argv[4]);
    transformationP(4) = atof(argv[5]);
    transformationP(5) = atof(argv[6]);
    
    Transformation transTool;
    transTool.set(transformationP);

    vector<vec> movedPoints;

    for(it3 = generatedPoints.begin(); it3 != generatedPoints.end(); it3++)
      {
	movedPoints.push_back(transTool.move(*it3));
      }

    cerr<<"model points after transformation (rotation and translation): "<<endl;
    copy(movedPoints.begin(), movedPoints.end(), ostream_iterator<vec>(cerr,"\n"));    

    //project model on the image according to camera calibration parameters.
    // put in a class

    vec calibrationP(4);
    
    calibrationP(0) = atof(argv[7]);
    calibrationP(1) = atof(argv[8]);
    calibrationP(2) = atof(argv[9]);
    calibrationP(3) = atof(argv[10]);

    Camera camera(calibrationP);
    
    vector<vec> projectedPoints;

    for( it3 = movedPoints.begin(); it3 != movedPoints.end(); it3++)
	projectedPoints.push_back(camera.project(*it3));

    cerr<<"projection of the model on the image: "<<endl;
    copy(projectedPoints.begin(), projectedPoints.end(), ostream_iterator<vec>(cerr,"\n"));

    //test the pose estimation algorithm

    transformationP(2) += 10;
    transformationP(1) += 3;
    transformationP(0) += 5;
    transformationP(4) += .3;
    
    RANSACPoseEstimation ransac(generatedPoints,projectedPoints,calibrationP,transformationP);

    ransac.estimatePose(12,3);
   
  }catch (exception &e)
    {
      e.what();
      exit(0); //check if number appropriate FAILURE
    }

}

