#include <iostream>
#include <cmath>
#include <fstream>
#include <valarray>
#include "mm_multi_array_op.h"
#include "mm_image_fileinput.h"
#include "mm_image_qtoutput.h"
#include "mm_image.h"

#include "registration.h"

using namespace boost;
using namespace std;
using namespace mimas;

/* the executable in 'registration' takes the two test images
   in parameters and the size of the square area to be used for
   registration.
   Dennis Tessier Sep 2004 */
int main(int argc, char **argv)
{
  int retVal = 0;
  try {
    
    MMERROR( argc == 4, mm_exception, ,
             "Syntax: " << argv[0] << " image1 image2 size" );

    QApplication app( argc, argv );
    mm_image_qtoutput< unsigned char > display( app );
    
    mm_image< int > inimage1, inimage2;

    ifstream
      infile1( argv[1], ios::binary ),
      infile2( argv[2], ios::binary );
  
    MMERROR( infile1 >> inimage1, mm_exception, ,
	     "Error loading image file '" << argv[1] << '\'' );
    MMERROR( infile2 >> inimage2, mm_exception, ,
	     "Error loading image file '" << argv[2] << '\'' );

    int a = atoi(argv[3]);
    MMERROR( a > 0, mm_exception, ,
	     "3rd parameter must be an integer greater than zero." );

    typedef multi_array< double, 2 > array_type;
    typedef array_type::index_range range;
    mm_image< int > im1, im2;
    im1.init( a, a ), im2.init( a, a );
    im1.rawData() = inimage1.rawData()[ indices[ range( 0, a ) ]
      	                                       [ range( 0, a ) ] ];
    im2.rawData() = inimage2.rawData()[ indices[ range( 0, a ) ]
                                               [ range( 0, a ) ] ];

    display << registration( im1, im2 ) << mimas::pause();

  } catch ( exception &e ) {

    cerr << e.what() << endl;
    retVal = 1;

  };

  return retVal;
}
