//
// Colour segmentation using hashmaps
// @author Bala Amavasai (bala@amavasai.org)
// @date Wed Mar 16 18:17:19 GMT 2005
// $Header: /cvs/mimas/examples/segmentHashmap/main.cc,v 1.2 2005/03/17 12:30:52 bpa Exp $
//

#include <fstream>
#include <iostream>
#include <qapplication.h>
#include "mm_segment_hashmap_tool.h"
#include "mm_image_fileinput.h"
#include "mm_image_fileoutput.h"
#include "mm_image_qtoutput.h"
#include "mm_rgba.h"

using namespace mimas;
using namespace std;

int main(int argc, char **argv)
{
	int retVal = 0;
	try {
		QApplication app( argc, argv );

		MMERROR( argc >= 4, mm_exception, ,
				"Usage:  segmentHashmap <model-colour-image> <scene-colour-image> <threshold>" << endl << "        or please see the runMe file for details of how to run this app" );

		cout << endl;
		cout << "Matching colours found in model image to scene image" << endl; 
		cout << endl;

		mm_segment_hashmap_tool seg;
		
		// load in the image that contains sample colours
		mm_image< mm_rgba< unsigned char > > im;
		ifstream imFile( argv[1], ios::binary ); imFile >> im;

		// load the image to be segmented
		mm_image< mm_rgba< unsigned char > > sim;
		ifstream simFile( argv[2], ios::binary ); simFile >> sim;

		// store the colours from the first image
		seg.storeColours(im);
		
		// segment the image
		mm_image< mm_rgba< unsigned char > > simout;
		seg.segment(sim, simout, atoi(argv[3]));

		mm_image_qtoutput< mm_rgba< unsigned char > > display( app );
		display << simout << mimas::pause();
		
	} catch ( exception &e ) {
		cerr << e.what() << endl;
		retVal = 1;
	};
	return retVal;
}
