// example applications to use the base classes / miniman pairwise libs
// stuart meikle Thu Mar  2 10:38:28 2000
//
#include <boost/numeric/ublas/vector.hpp>
#include <cassert>
#include <qapplication.h>
#include <qpainter.h>
#include "mm_canny_tool.h"
#include <fstream>
#include "mm_image.h"
#include "mm_image_fileinput.h"
#include "mm_slap_tool.h"
#include "slapToolWidget.h"

using namespace boost;
using namespace std;
using namespace mimas;

typedef numeric::ublas::vector< double > Vector;

SlapToolWidget::SlapToolWidget( QWidget *parent, const char *name, WFlags f ):
  QWidget( parent, name, f )
{
  mm_image<int> input;
  mm_canny_tool<int> canny;
  mm_edgestring_image eim;

  MMERROR( qApp->argc() == 2, mm_exception, ,
           "Usage: " << qApp->argv()[0] << " imagefile" );

  ifstream file( qApp->argv()[1], ios::binary );
  mm_image< mm_rgba< unsigned char > > temp;
  file >> input;

  //(1) first run canny to extract a set of mm_string_el strings
  mm_edge_string_ptr_list edge_strings =
    canny.findEdgeStrings( input );
  
  //(2) then create an image to contain them for visualisation
  // eim.setOwnership(false);
  eim.init( input.getWidth(), input.getHeight());
  eim.addStrings( edge_strings );

  // Removed, because it'll crash the program later.
  // display(eim);

  //(2.5) convert the edge strings to straight line segments
  //      slap_tool = Straight Line AProximation tool.
  //
  mm_slap_tool slapper(M_PI/10);
  
  l_lines=slapper.findStraightLines( edge_strings );
    
#ifndef NDEBUG
  cerr << "I'm still alive!" << endl;
#endif


  //(3) display the lines ...
  setBackgroundColor( white );		// white background

  //(4) delete the strings
  // mm_edge_string_ptr_list::iterator	sit;
  // for(sit=edge_strings.begin();sit!=edge_strings.end();++sit)
  // delete (*sit);
}

void SlapToolWidget::paintEvent( QPaintEvent * )
{
  QPainter paint( this );

  //draw the current input scene 
  //draw it!
  paint.setPen( Qt::black );
  for(mm_line_ptr_list::iterator i=l_lines.begin();
      i!=l_lines.end(); i++)
    {
      mm_line_ptr ln = (*i);
      const Vector *start, *end;
		
      start = &ln->getStart();
      end   = &ln->getEnd();
		
      paint.setPen( Qt::black );
      paint.drawLine( (int)(*start)(0), (int)(*start)(1), 
                      (int)(*end)(0),(int)(*end)(1) );
    }	
}
