// example applications to use the base classes / miniman pairwise libs
// stuart meikle Thu Mar  2 10:38:28 2000
//
#include <cassert>
#include <qimage.h>
#include <qpainter.h>
#include "webcamTrackWidget.h"
#include "mm_image_draw.h"

using namespace boost;
using namespace std;
using namespace mimas;

WebcamTrackWidget::WebcamTrackWidget( QWidget *parent, const char *name,
                                      WFlags f ):
  QWidget( parent, name, f ), webcam( "/dev/video" ), counter(100)
{
   startTimer( 50 );
   setBackgroundColor( white );		// white background
}

void WebcamTrackWidget::timerEvent( QTimerEvent * )
{
  const int objectarea=200;
  if ( counter > 0 ) {
    cerr << "Hold object on top left 20x20 pixels. Capturing colours in "
         << counter << " counts" << endl;
    webcam >> im;
    counter--;
  } else if ( counter == 0 ) {
    cerr << "Capturing colours...." ;

    mm_image< mm_rgba< unsigned char > > imagetemplate;
    imagetemplate.init(20,20);

    for (int j=0; j<imagetemplate.getHeight(); j++)
		for (int i=0; i<imagetemplate.getHeight(); i++)
        {
          imagetemplate.pixel( i, j ) = im.pixel( i, j );
        }
    hashmap.storeColours(imagetemplate);
	
    cerr << "finished" << endl << endl;
    cerr << "Now move object around screen. If the area of the object is >=" << objectarea<< " pixels," << endl; 
    cerr << "I'll draw a rectangle around it" << endl;
    cerr << "If you get a core dump then you haven't held the object in place properly ;)" << endl;
    counter--;
  } else {
      mimas::mm_image< mm_rgba< unsigned char > > imtemp;
		webcam >> im;

		hashmap.segment( im, imtemp, 31 );


		int threshold=127;
		mm_Cartesian<int> mincoord, maxcoord;
		int areacount;

		for (int j=0; j<imtemp.getHeight(); j++)
        for (int i=0; i<imtemp.getWidth(); i++)
          {
				if ((double)imtemp.getPixel(i,j)>threshold)
              {
                mincoord.x=imtemp.getWidth();
                mincoord.y=imtemp.getHeight();
                maxcoord.x=0;
                maxcoord.y=0;
                areacount=0;
                findEnclosedArea(i, j, imtemp, threshold, mincoord, maxcoord, areacount);

                if (areacount>200)
                  {
                    drawThickLine(im, mincoord.x, mincoord.y, maxcoord.x, mincoord.y, mm_rgba< unsigned char >( 255, 255, 255 ), 2);
                    drawThickLine(im, mincoord.x, mincoord.y, mincoord.x, maxcoord.y, mm_rgba< unsigned char >( 255, 255, 255 ), 2);
                    drawThickLine(im, maxcoord.x, mincoord.y, maxcoord.x, maxcoord.y, mm_rgba< unsigned char >( 255, 255, 255 ), 2);
                    drawThickLine(im, mincoord.x, maxcoord.y, maxcoord.x, maxcoord.y, mm_rgba< unsigned char >( 255, 255, 255 ), 2);
                  }

              }

          }
  };
  repaint( false );
}

void WebcamTrackWidget::paintEvent( QPaintEvent * )
{
  QPainter paint( this );

  //draw the current input scene
  //draw the bit map
  QImage img(im.getWidth(),im.getHeight(),32);

  for(int y=0;y<im.getHeight();y++ )
    for(int x=0;x<im.getWidth();x++ )
      {
        mm_rgba< unsigned char > *pixel = &im.getPixel( x, y );
        int	pr = pixel->getRed();
        int	pg = pixel->getGreen();
        int	pb = pixel->getBlue();
        img.setPixel( x,y, qRgb(pr,pg,pb));
      }

  QPixmap pix;
  pix.convertFromImage(img);

  bitBlt( this , 0,0, &pix, 0,0, im.getWidth(), im.getHeight(),
          CopyROP, false);
}

void WebcamTrackWidget::findEnclosedArea( int x, int y,
                                          mm_image< mm_rgba< unsigned char > > &refimg,
                                          int threshold,
                                          mm_Cartesian<int> &min,
                                          mm_Cartesian<int> &max,
                                          int &areacount )
{
  // note - need only to grow down since scanning from top->bottom, left->right
  //

  MMERROR( threshold != 0, mm_exception, ,
           "threshold cannot be zero!" );

  if ((x<0) || (y<0) || (x>refimg.getWidth()) || (y>refimg.getHeight()))
    return;

  if ((double)refimg.getPixel(x,y)>=threshold)
    {
		refimg.setPixel(x,y,mm_rgba< unsigned char >( 0, 0, 0 ));
		areacount++;

		if (x>max.x) max.x=x;
		if (y>max.y) max.y=y;
		if (x<min.x) min.x=x;
		if (y<min.y) min.y=y;

		findEnclosedArea(x-1, y, refimg, threshold, min, max, areacount);
		findEnclosedArea(x+1, y, refimg, threshold, min, max, areacount);
		findEnclosedArea(x, y+1, refimg, threshold, min, max, areacount);
		//findEnclosedArea(x, y-1, refimg, threshold, min, max); // not needed since the scanline is from top->bottom
    }
}
