#include <cassert>
#include <fstream>
#include <iomanip>
#include <mm_image_v4linput.h>
#include <mm_image_fileoutput.h>
#include <mm_image_mesaoutput.h>
#include <vector>

// #define SAVEPICS

using namespace boost;
using namespace mimas;
using namespace std;

typedef mm_image_v4linput< mm_rgba< unsigned char > > videosource;
typedef shared_ptr< videosource > videosource_ptr;
typedef mm_image_mesaoutput< mm_rgba< unsigned char > > mesasink;
typedef shared_ptr< mesasink > mesasink_ptr;

int main( int argc, char *argv[] )
{
  int retVal = 0;
  int c = 0;
  try {
    MMERROR( argc > 1 && argc % 2 == 1, mm_exception, ,
             "Syntax: " << argv[0] << " device channel [device channel ...]" );
    mm_x11_display_ptr display( new mm_x11_display );
    vector< videosource_ptr > sources;
    vector< mesasink_ptr > sinks;
    for ( int i=1; i<argc; i += 2 ) {
      videosource_ptr source( new videosource( argv[i], atoi(argv[i+1]) ) );
      source->setSensivity( 32768, 32768, 32768, 32768 );
      sources.push_back( source );
      sinks.push_back
              ( mesasink_ptr( new mesasink( display,
                                            source->getWidth(),
                                            source->getHeight() ) ) );
    };
    
    while ( true ) {
      
      assert( sinks.size() == sources.size() );
      for ( int i=0; i<(signed)sources.size(); i++ ) {
         mm_image< mm_rgba< unsigned char > > image;
	 *sources[i] >> image;
	 *sinks[i] << image;
#ifdef SAVEPICS
         ostringstream x;
         x << setw( 6 ) << setfill( '0' ) << c++ << ".ppm";
         ofstream f( x.str().c_str(), ios::binary );
         f << image;
#endif
      };
    };
    
  } catch ( exception &e ) {
    cerr << e.what() << endl;
    retVal = 1;
  };
  return retVal;
}

