//
// Example application using wiggle filter
// stumeikle Wed May 22 18:56:12 2002
//
//


#include <iostream>
#include <vector>
#include <fstream>
#include "mm_image.h"
#include "mm_image_fileinput.h"
#include "mm_image_qtoutput.h"
#include "mm_image_funcs.h"
#include "mm_canny_tool.h"
#include "mm_filter.h"
#include "mm_corner_tool.h"
#include "mm_prop.h"
#include "mm_edge_string_op.h"

using namespace mimas;
using namespace std;

int main(int argc, char **argv)
{
  int retVal = 0;
  try {
    QApplication app( argc, argv );
    mm_image_qtoutput< mm_rgba< unsigned char > > display( app );
    mm_image<int> 		someimage;
    mm_canny_tool<int>		canny;
    mm_edge_string_ptr_list	edge_strings;

    ifstream file( argv[1], ios::binary );
    file >> someimage;
    
    //(1) first run canny to extract a set of mm_string_el strings
    edge_strings=canny.findEdgeStrings( someimage );
  
    //(2) then create an image to contain them for visualisation
    mm_property_image	pim;
    // pim.setOwnership(false);
    pim.init( someimage.getWidth(), someimage.getHeight());
    pim.addStrings( edge_strings );    
    display << pim << mimas::pause();
    
    //(3) now filter the wiggles / texture / noise
    pim.clear();
    pim.init( someimage.getWidth(), someimage.getHeight());
    mm_edge_string_op::filterWiggles( edge_strings, 0.2, 10, 35.0 );
    pim.addStrings( edge_strings );
    display << pim << mimas::pause();

    //(4) delete the strings
    // mm_edge_string_ptr_list::iterator sit;
    // for(sit=edge_strings.begin();sit!=edge_strings.end();++sit)
    // delete (*sit);
  } catch ( exception &e ) {
    cerr << e.what() << endl;
    retVal = 1;
  };
  return retVal;

}


