
// WARNING! DO NOT USE THIS CLASS
// This class has been replaced with the much better mm_canny_tool class
// mm_canny will be deprecated in the next release

#ifndef MM_CANNY_H
#define MM_CANNY_H

#include <iostream> 
#include <fstream>
#include <cmath>
#include "mm_image.h"
#include "mm_cartesian.h"
#include "mm_image_draw.h"

namespace mimas {
	
// $Header: /cvs/mimas/include/mm_canny.h,v 1.16 2005/07/28 21:53:31 engmb Exp $

/* Scale floating point magnitudes and angles to 8 bits */
#define ORI_SCALE 40.0
#define MAG_SCALE 20.0
/* Biggest possible filter mask */
#define MAX_MASK_SIZE 20

template<typename T>
class mm_canny : public mm_object
            //perhaps public virtual drawable one day
{
protected:
    int WIDTH;
    float norm (float x, float y);
    float gauss(float x, float sigma);
    float dGauss (float x, float sigma);
    float meanGauss (float x, float sigma);
    void seperable_convolution (float *gau, int width);
    void dxy_seperable_convolution (float *gau, int width, int which);
    void nonmax_suppress (void);
    void estimate_thresh (void);
    float distancePointToPoint(int ax, int ay, int bx, int by);
    float distancePointToLine(int ax, int ay, int bx, int by, int px, int py);
    void hysteresis (void);
    int trace (int i, int j);
    void initialiseSharedImages(mm_image<T> &inputimage);


    mm_image<int> ori, mag, im;
    mm_image<int> traced;
    mm_image<float> smx, smy, dx, dy;


public:
    float ratio; /* Fraction of pixels that should be above the HIGH threshold */
    float sd; /* Gaussian standard deviation */
    int highthres, lowthres; /* high and low thresholds */
    bool autothreshold; /* autothresholding */

    void setCannyPixHighThresRatio(double tempval);
    double getCannyPixHighThresRatio(void);
    void setCannyStdDev(double tempval);
    double getCannyStdDev(void);
    void setCannyHighThres(int tempval);
    int getCannyHighThres(void);
    void setCannyLowThres(int tempval);
    int getCannyLowThres(void);
    void setCannyAutoThres(bool tempval);
    bool getCannyAutoThres(void);
    void canny(mm_image<T> &inputImage, mm_image<T> &outimage);
    void canny(mm_image<T> &image);


    mm_canny()
    {
        ratio=0.1;
        sd=1.0;
        highthres=127;
        lowthres=63;
        autothreshold=true;
    }

    ~mm_canny()
    {
        smx.clear();
        smy.clear();
        dx.clear();
        dx.clear();
        ori.clear();
        mag.clear();
        im.clear();

    }
};


template<typename T>
void mm_canny<T>::setCannyPixHighThresRatio(double tempval)
{
    ratio=tempval;
}

template<typename T>
double mm_canny<T>::getCannyPixHighThresRatio(void)
{
    return ratio;
}

template<typename T>
void mm_canny<T>::setCannyStdDev(double tempval)
{
    sd=tempval;
}

template<typename T>
double mm_canny<T>::getCannyStdDev(void)
{
    return sd;
}

template<typename T>
void mm_canny<T>::setCannyHighThres(int tempval)
{
    highthres=tempval;
}

template<typename T>
int mm_canny<T>::getCannyHighThres(void)
{
    return highthres;
}

template<typename T>
void mm_canny<T>::setCannyLowThres(int tempval)
{
    lowthres=tempval;
}

template<typename T>
int mm_canny<T>::getCannyLowThres(void)
{
    return lowthres;
}


template<typename T>
void mm_canny<T>::setCannyAutoThres(bool tempval)
{
    autothreshold=tempval;
}

template<typename T>
bool mm_canny<T>::getCannyAutoThres(void)
{
    return autothreshold;
}




template<typename T>
void mm_canny<T>::initialiseSharedImages(mm_image<T> &inputImage)
{
    ori.init(inputImage.getWidth(),inputImage.getHeight());

    mag.init(inputImage.getWidth(),inputImage.getHeight());

    im = mm_image< int >( inputImage );

    smx.init(inputImage.getWidth(),inputImage.getHeight());

    smy.init(inputImage.getWidth(),inputImage.getHeight());

    dx.init(inputImage.getWidth(),inputImage.getHeight());

    dy.init(inputImage.getWidth(),inputImage.getHeight());

    traced.init(inputImage.getWidth(),inputImage.getHeight());
}


template<typename T>
float mm_canny<T>::norm (float x, float y)
{
    return (float) sqrt ( (double)(x*x + y*y) );
}

template<typename T>
void mm_canny<T>::canny(mm_image<T> &inputImage, mm_image<T> &outimage)
{
    int width=0;
    int i,j,n;
    float gau[MAX_MASK_SIZE], dgau[MAX_MASK_SIZE], z;
    /* float gau[20], dgau[20], z; */

    initialiseSharedImages(inputImage);

    /* Create a Gaussian and a derivative of Gaussian filter mask */
    for(i=0; i<MAX_MASK_SIZE; i++)
    {
        gau[i] = meanGauss ((float)i, sd);
        if (gau[i] < 0.005)
        {
            width = i;
            break;
        }
        dgau[i] = dGauss ((float)i, sd);
    }

    n = width+width + 1;
    WIDTH = width/2;
    //cout << "Smoothing with a Gaussian (width = %d) ..." << std::endl;

    /* Convolution of source image with a Gaussian in X and Y directions  */
    seperable_convolution (gau, width);

    /* Now convolve smoothed data with a derivative */
    //cout << "Convolution with the derivative of a Gaussian..." << std::endl;
    dxy_seperable_convolution (dgau, width, 1); /* x direction - works on (mm_image)dx */
    //smx.clear();

    dxy_seperable_convolution (dgau, width, 0); /* y direction - works on (mm_image)dy */
    //smy.clear();

    /* Create an image of the norm of dx,dy */
    for (i=0; i<im.getHeight(); i++)
        for (j=0; j<im.getWidth(); j++)
        {
            z = norm (dx.pixel(j,i), dy.pixel(j,i));
            mag.pixel(j,i)=(int)(z*MAG_SCALE);
        }

    /* Non-maximum suppression - edge pixels should be a local max */

    nonmax_suppress();
    hysteresis();

    outimage.init(im.getWidth(),im.getHeight());
    for (i=1; i<im.getHeight()-1; i++)
        for (j=1; j<im.getWidth()-1; j++)
          outimage.pixel(j,i)=im.pixel(j,i);
    
    // set boundaries to zero due to an artefact of this implement of Canny
    drawLine(outimage,0,0,outimage.getWidth()-1,0,T(0));
    drawLine(outimage, outimage.getWidth()-1,0,outimage.getWidth()-1,outimage.getHeight()-1,T(0));
    drawLine(outimage, 0,outimage.getHeight()-1, outimage.getWidth()-1,
	   outimage.getHeight()-1,T(0));
    drawLine(outimage,0,0,0,outimage.getHeight()-1,T(0));
    

    //dx.clear();
    //dy.clear();
}

template<typename T>
void mm_canny<T>::canny(mm_image<T> &image)
{
    canny(image,image);
}
	
 /*      Gaussian        */
template<typename T>
float mm_canny<T>::gauss(float x, float sigma)
{
    float xx;

    if (sigma == 0) return 0.0;
    xx = (float)exp((double) ((-x*x)/(2*sigma*sigma)));
    return xx;
}

template<typename T>
float mm_canny<T>::meanGauss (float x, float sigma)
{
    float z;

    z = (gauss(x,sigma)+gauss(x+0.5,sigma)+gauss(x-0.5,sigma))/3.0;
    z = z/(M_PI*2.0*sigma*sigma);
    return z;
}

/*      First derivative of Gaussian    */
template<typename T>
float mm_canny<T>::dGauss (float x, float sigma)
{
    return -x/(sigma*sigma) * gauss(x, sigma);
}



/*      HYSTERESIS thersholding of edge pixels. Starting at pixels with a
	value greater than the HIGH threshold, trace a connected sequence
	of pixels that have a value greater than the LOW threhsold.        */

template<typename T>
void mm_canny<T>::hysteresis (void)
{
    int i,j;

    //cout << "Beginning hysteresis thresholding..." << std::endl;
    for (i=0; i<im.getHeight(); i++)
        for (j=0; j<im.getWidth(); j++)
          im.pixel(j,i)=0;


    if ((autothreshold) || (highthres<lowthres))
    {
        estimate_thresh ();
        //cout << "Estimated hysteresis thresholds (from image): highthres=" << highthres << " lowthres=" << lowthres << std::endl;
    }


    /* For each edge with a magnitude above the high threshold, begin
      tracing edge pixels that are above the low threshold.                */

    for (i=0; i<im.getHeight(); i++)
        for (j=0; j<im.getWidth(); j++)
            if (mag.pixel(j,i) >= highthres)
                trace (i, j);
    return; /*************************************************/
}


template<typename T>
float mm_canny<T>::distancePointToPoint(int ax, int ay, int bx, int by)
{
    return(sqrt( ((bx-ax)*(bx-ax)) + ((by-ay)*(by-ay)) ));
}

/* Formula from comp.graphics.algorithms FAQ */
template<typename T>
float mm_canny<T>::distancePointToLine(int ax, int ay, int bx, int by, int px, int py)
{
    return(fabs( ((ay-py)*(bx-ax) - (ax-px)*(by-ay)) / distancePointToPoint (ax,ay,bx,by)  ));
}


/*******************************/

/*      TRACE - recursively trace edge pixels that have a threshold > the low
	edge threshold, continuing from the pixel at (i,j).                     */



template<typename T>
int mm_canny<T>::trace (int i, int j)
{
    int n,m;
    char flag = 0;

    if (im.getPixel(j,i) == 0)
    {
        im.setPixel(j,i,255);
        flag=0;
        for (n= -1; n<=1; n++)
        {
            for(m= -1; m<=1; m++)
            {
                if (n==0 && m==0) continue;
                if (mag.getPixel(j+m, i+n) >=lowthres)
                {
                    if (((i+n)>=0) && ((i+n)<mag.getHeight()) &&
                            ((j+m)>=0) && ((j+m)<mag.getWidth()))
                        if (trace(i+n, j+m ))
                        {
                            flag=1;
                            break;
                        }
                }
            }
            if (flag) break;
        }
        return(1);
    }
    return(0);
}






template<typename T>
void mm_canny<T>::seperable_convolution (float *gau, int width)
{
    int i,j,k, I1, I2, nr, nc;
    float x, y;

    nr = im.getHeight();;
    nc = im.getWidth();

    for (i=0; i<nr; i++)
        for (j=0; j<nc; j++)
        {
            x = gau[0] * im.getPixel(j,i); y = gau[0] * im.getPixel(j,i);
            for (k=1; k<width; k++)
            {
                I1 = (i+k)%nr; I2 = (i-k+nr)%nr;
                y += gau[k]*im.getPixel(j,I1) + gau[k]*im.getPixel(j,I2);
                I1 = (j+k)%nc; I2 = (j-k+nc)%nc;
                x += gau[k]*im.getPixel(I1,i) + gau[k]*im.getPixel(I2,i);
            }
            smx.setPixel(j,i,x); smy.setPixel(j,i,y);
        }
}

template<typename T>
void mm_canny<T>::dxy_seperable_convolution (float *gau, int width, int which)
{
    int i,j,k, I1, I2;
    int nr,nc;
    float x;

    nr = im.getHeight();;
    nc = im.getWidth();

    for (i=0; i<nr; i++)
        for (j=0; j<nc; j++)
        {
            x = 0.0;
            for (k=1; k<width; k++)
            {
                if (which == 0)
                {
                    I1 = (i+k)%nr; I2 = (i-k+nr)%nr;
                    x += -gau[k]*smy.getPixel(j,I1) + gau[k]*smy.getPixel(j,I2);
                }
                else
                {
                    I1 = (j+k)%nc; I2 = (j-k+nc)%nc;
                    x += -gau[k]*smx.getPixel(I1,i) + gau[k]*smx.getPixel(I2,i);
                }
            }
            if (which==1)
                dx.setPixel(j,i,x);
            if (which==0)
                dy.setPixel(j,i,x);
        }
}

template<typename T>
void mm_canny<T>::nonmax_suppress (void)
{
    int i,j;
    float xx, yy, g2, g1, g3, g4, g, xc, yc;

    for (i=1; i<mag.getHeight()-1; i++)
    {
        for (j=1; j<mag.getWidth()-1; j++)
        {
          mag.pixel(j,i)=0;

            /* Treat the x and y derivatives as components of a vector */
            xc = dx.getPixel(j,i);
            yc = dy.getPixel(j,i);
            if (fabs(xc)<0.01 && fabs(yc)<0.01) continue;

            g  = norm (xc, yc);

            /* Follow the gradient direction, as indicated by the direction of
               the vector (xc, yc); retain pixels that are a local maximum. */

            if (fabs(yc) > fabs(xc))
            {

                /* The Y component is biggest, so gradient direction is basically UP/DOWN */
                xx = fabs(xc)/fabs(yc);
                yy = 1.0;

                g2 = norm (dx.getPixel(j,i-1), dy.getPixel(j,i-1));
                g4 = norm (dx.getPixel(j,i+1), dy.getPixel(j,i+1));
                if (xc*yc > 0.0)
                {
                    g3 = norm (dx.getPixel(j+1,i+1), dy.getPixel(j+1,i+1));
                    g1 = norm (dx.getPixel(j-1,i-1), dy.getPixel(j-1,i-1));
                }
                else
                {
                    g3 = norm (dx.getPixel(j-1,i+1), dy.getPixel(j-1,i+1));
                    g1 = norm (dx.getPixel(j+1,i-1), dy.getPixel(j+1,i-1));
                }

            } else
            {

                /* The X component is biggest, so gradient direction is basically LEFT/RIGHT */
                xx = fabs(yc)/fabs(xc);
                yy = 1.0;

                g2 = norm (dx.getPixel(j+1,i), dy.getPixel(j+1,i));
                g4 = norm (dx.getPixel(j-1,i), dy.getPixel(j-1,i));
                if (xc*yc > 0.0)
                {
                    g3 = norm (dx.getPixel(j-1,i-1), dy.getPixel(j-1,i-1));
                    g1 = norm (dx.getPixel(j+1,i+1), dy.getPixel(j+1,i+1));
                }
                else
                {
                    g1 = norm (dx.getPixel(j+1,i-1), dy.getPixel(j+1,i-1));
                    g3 = norm (dx.getPixel(j-1,i+1), dy.getPixel(j-1,i+1));
                }
            }

            /* Compute the interpolated value of the gradient magnitude */
            if ( (g > (xx*g1 + (yy-xx)*g2)) &&
                    (g > (xx*g3 + (yy-xx)*g4)) )
            {
                if (g*MAG_SCALE <= 255)
                  mag.pixel(j,i) = (int)(g*MAG_SCALE);
                else
                  mag.pixel(j,i) = 255;

                ori.setPixel(j,i,(T)(atan2 (yc, xc) * ORI_SCALE));
            } else
            {
              mag.pixel(j,i)= 0;
                ori.setPixel(j,i,0);
            }

        }
        //	cerr << i << " " << j << std::endl;

    }
}

template<typename T>
void mm_canny<T>::estimate_thresh (void)
{
    int i,j,k, hist[256], count;

    /* Build a histogram of the magnitude image. */
    for (k=0; k<256; k++) hist[k] = 0;

    for (i=WIDTH; i<mag.getHeight()-WIDTH; i++)
        for (j=WIDTH; j<mag.getWidth()-WIDTH; j++)
            hist[mag.getPixel(j,i)]++;

    /* The high threshold should be > 80 or 90% of the pixels
    	j = (int)(ratio*mag->info->nr*mag->info->nc);
    */
    j = mag.getHeight();
    if (j<mag.getWidth()) j = mag.getWidth();
    j = (int)(0.9*j);
    k = 255;

    count = hist[255];
    while (count < j)
    {
        k--;
        if (k<0) break;
        count += hist[k];
    }
    highthres = k;

    i=0;
    while (hist[i]==0) i++;

    lowthres = (int)((highthres+i)/2.0);
}

}

#endif /* MM_CANNY_H */

