#ifndef MM_CGH_H
#define MM_CGH_H

#include <boost/numeric/ublas/vector.hpp>
#include <list>
#include "mm_edge_string.h"
#include "mm_hough_transform.h"
#include "mm_object.h"

namespace mimas {
	
/** Cornerwise geometric histrograms.
    Cornerwise geometric histrograms encode edge strings. can be used for
    feature recognition purposes. developed for learning of environments ,
    they have good invariance to scale ( to viewing position ). They *can*
    have good invariance to feature rotation on the image plane but the
    current code doesn't have this.
    Capacity of the representation is about 250,000 with the default parameters.
    For more info consult this
    <A HREF="http://www.stumeikle.org/stumeikle-thesis.pdf">PhD thesis</A>.
    
    @author Stuart Meikle (stu@stumeikle.org)
    @date Fri May 24 17:01:53 2002
    $Header: /cvs/mimas/include/mm_cgh.h,v 1.9 2004/12/14 23:27:07 engjw Exp $ */
class mm_cgh : public mm_hough_transform
{
private:
    double	bin_threshold;		///< after creation the histogram is thresholded to suppress noise.
    double	max_radius;		///< used in the creation process for calculating how much each edgel should contribute
    double	min_radius;		///< used in the creation process for calculating how much each edgel should contribute
    int		size;			///< size of the histogram generated

    void	setup();

public:
    typedef std::ifstream ifstream;
    typedef std::ofstream ofstream;
    typedef boost::numeric::ublas::vector< double > Vector;
    typedef boost::numeric::ublas::vector< int > IntVector;
    mm_cgh()
    {
	//install defaults
	bin_threshold = 0.1;

	min_radius = 2.0;
	max_radius = 150.0;
	size = 72;

	setup();
    }

    void	setMaxRadius( double d ) { max_radius = d; }
    void	setBinThreshold( double d ) { bin_threshold = d; }
    void	createFromStrings( mm_edge_string_ptr_list strings,
                            const Vector &cpos ); ///< create a CGH from extracted (canny) edge strings plus the position of a point-of-interest (corner)

   /* double	dotProduct( mm_cgh& cgh2 ) ///< used for comparing two features
		{ return mm_image<double>::dotProduct( cgh2 ); } */

      // void	readFromFile( ifstream& ifs );    ///< read histogram from a file
      // void	writeToFile( ofstream& outfile); ///< write histogram to a file
};

typedef boost::shared_ptr< mm_cgh > mm_cgh_ptr;

};


#endif
