//
// corner (element) property
// stumeikle Sun May 12 20:31:50 2002
//
//

#ifndef MM_CORNER_H
#define MM_CORNER_H

#include <boost/numeric/ublas/vector.hpp>
#include <boost/shared_ptr.hpp>
#include "mm_angle.h"
#include "mm_prop.h"

namespace mimas {

/**
 *
 * Another image property.. this is a small structure used to contain a small amount
 * of data associated with a found corner, and is used to render the corner too.
 *
 * @author Stuart Meikle (stu@stumeikle.org)
 * @date Sun May 12 20:31:50 2002
 * $Header: /cvs/mimas/include/mm_corner.h,v 1.7 2004/12/14 23:27:07 engjw Exp $
 */
class mm_corner : public mm_prop
{
   public:
   typedef boost::numeric::ublas::vector< double > Vector;
    private:
   Vector centre;		///< position wrt the image plane
	double		strength;	///< contrast/strength of the corner
	mm_angle 	orientation;	///< orientation of the corner

    public:
	void 	setPosition( double x, double y) { centre(0) = x; centre(1) = y; }
	void	setPosition( const Vector &m ) 	{ centre = m; }
	void	setStrength( double s ) 	{ strength = s; }
	void	setContrast( double s ) 	{ strength = s; }
	void	setOrientation( mm_angle a )    { orientation = a; }
	const Vector &getPosition() { return centre; }
	double  getContrast()			{ return strength; }

	mm_corner(void) : mm_prop( corner ), centre( 2 ) {}

	virtual void draw( mm_image< mm_rgba< unsigned char > > &output,
                      int i, int j, bool c)
	{
	    int s;
	    if (!c)
		s = 255;
	    else
	    {
		s = (int)(strength);
		if (s>255) s=255;
	    }

	    output.setPixel( i-1,j-1, mm_rgba< unsigned char >(0,s,0) );
	    output.setPixel( i,j-1, mm_rgba< unsigned char >(0,s,0) );
	    output.setPixel( i+1,j-1, mm_rgba< unsigned char >(0,s,0) );
	    output.setPixel( i-1,j, mm_rgba< unsigned char >(0,s,0) );
	    output.setPixel( i+1,j, mm_rgba< unsigned char >(0,s,0) );
	    output.setPixel( i-1,j+1, mm_rgba< unsigned char >(0,s,0) );
	    output.setPixel( i,j+1, mm_rgba< unsigned char >(0,s,0) );
	    output.setPixel( i+1,j+1, mm_rgba< unsigned char >(0,s,0) );

	}

};

typedef boost::shared_ptr< mm_corner > mm_corner_ptr;

};

#endif
