#ifndef MM_CORRELATE_SQUARE_REGION_TCC
#define MM_CORRELATE_SQUARE_REGION_TCC

#include <algorithm>

namespace mimas {
template< typename T, class Corr >
double mm_correlate_square_region< T, Corr >::operator()( const Vector &a,
                                                          const Vector &b )
{
  int
    x1min = std::max( (int)a( 0 ) - areaDiv2, 0 ),
    y1min = std::max( (int)a( 1 ) - areaDiv2, 0 ),
    x1max = std::min( (int)a( 0 ) + areaDiv2 + 1, image1->getWidth() ),
    y1max = std::min( (int)a( 1 ) + areaDiv2 + 1, image1->getHeight() ),
    x2min = std::max( (int)b( 0 ) - areaDiv2, 0 ),
    y2min = std::max( (int)b( 1 ) - areaDiv2, 0 ),
    x2max = std::min( (int)b( 0 ) + areaDiv2 + 1, image2->getWidth() ),
    y2max = std::min( (int)b( 1 ) + areaDiv2 + 1, image2->getHeight() ),
    xmin = std::max( x1min - (int)a( 0 ), x2min - (int)b( 0 ) ),
    ymin = std::max( y1min - (int)a( 1 ), y2min - (int)b( 1 ) ),
    xmax = std::min( x1max - (int)a( 0 ), x2max - (int)b( 0 ) ),
    ymax = std::min( y1max - (int)a( 1 ), y2max - (int)b( 1 ) );

  typename Corr::first_argument_type p1;
  typename Corr::second_argument_type p2;

  for ( int y=ymin; y<ymax; y++ )
    for ( int x=xmin; x<xmax; x++ ) {
      p1.push_back( image1->getPixel( x + (int)a( 0 ),
                                      y + (int)a( 1 ) ) );
      p2.push_back( image2->getPixel( x + (int)b( 0 ),
                                      y + (int)b( 1 ) ) );
    };

  return corr( p1, p2 );
}
}

#endif
