//
// edge string funcs or edge string operators
// stumeikle Mon May  6 18:10:06 2002
//

#ifndef MM_EDGE_STRING_OP_H
#define MM_EDGE_STRING_OP_H

#include <boost/numeric/ublas/vector.hpp>
#include "mm_property_image.h"
#include "mm_edge_string.h"

namespace mimas {

//operators
class mm_edge_string_op
{
    // just a namespace really. functions which operate on
    // edge strings.    
    //20030331... almost
    static bool				fast_mode;

    static int				shortSpur( const mm_string_el &spur, const mm_string_el &last, int thres, int count);
    static mm_edge_string_ptr_list findEdgeStrings( mm_property_image& input, int, double );
    static void 			pruneSpurs( mm_string_el_ptr_list& junc_list, mm_string_el_ptr_list& term_list );
    static void				pruneTees(mm_string_el_ptr_list& junc_list, mm_string_el_ptr_list& term_list );
    static void				pruneJuncs(mm_string_el_ptr_list& junc_list, mm_string_el_ptr_list& term_list );
    static void				removeJuncs( mm_string_el_ptr_list& junc_list,
							mm_string_el_ptr_list& term_list );
    static void				bridgeSmallGaps( mm_string_el_ptr_list& term_list );

public:
    typedef boost::numeric::ublas::vector< double > Vector;
    static mm_edge_string_ptr_list extractFromImage( mm_property_image& im, int len_thres,
							  double up_edge_thres );
    static mm_edge_string_ptr_list findStrings( mm_string_el_ptr_list& term_list );
    static mm_edge_string_ptr		findEdgeString( mm_string_el_ptr el  );
    static mm_edge_string_ptr		findLoop( mm_string_el_ptr el );
    static void				thresholdStrings( mm_edge_string_ptr_list& strings,
                                             int lt, double t );
    static bool				thresholdString( const mm_edge_string& s, int len, double t );
    static void				findContrastMaxima( mm_edge_string& s, mm_property_image& cim );
    static void				filterWiggles( mm_edge_string_ptr_list& edge_strings,
					  double wiggle_thres, double len_thres, double contrast_thre );

    //fast methods for fast canny. 20030331
    static void				useFastMode() {fast_mode=true;}
    static void				useNormalMode(){fast_mode=false;}
    static mm_edge_string_ptr_list findEdgeStrings_Fast( mm_property_image& input, int len_thres, double thres );
    static mm_edge_string_ptr getString_Fast(mm_property_image& input, int len_thres, double thres,
					      	int start_x, int start_y );
    static void  			firstHalf_Fast(mm_property_image& input, double thres,
				     		int startx, int starty, mm_edge_string& es,
				     		bool* crossed_thres, long *len );
    static void  			secondHalf_Fast(mm_property_image& input, double thres,
				     		int startx, int starty, mm_edge_string& es,
				     		bool* crossed_thres, long *len );


};

};

#endif



