//
// Method to find and track pipette tip using the Haynes and Jain edge-change
// detector and a recursive search algorithm. Note that this implementation
// assumes that the pipette is approaching from the top of the screen.
// 
// In order for initial recognition to take place the pipette has to be moved
// (ideally horizontally) so that the initial estimate of the tip can be
// obtained.
//
// Bala Amavasai (bala@amavasai.org)
// $Header: /cvs/mimas/include/mm_findpipette.h,v 1.6 2004/12/12 23:31:18 engjw Exp $
//

#ifndef MM_FINDPIPETTE_H
#define MM_FINDPIPETTE_H

#include <iostream>
#include <cmath>
#include <values.h>

#include "mm_image.h"
#include "mm_cartesian.h"

namespace mimas {
	
class mm_findPipette
{
    private:
	int skipXboundaries; // left and right boundary to skip
	int skipYboundaries; // top and botton boundary to skip
	int ySearchRange;  // search from -ve in the y direction when not first frame
	int thresval;        // hj threshold value
	int linesDiffRange; // difference between longest and shortest line that is considered
	int holdPositionCondition; // hold previous position if outside this range
	bool firstrecog;    // recognition for the first time so that prevdepth can be set
	int scanliney;

	mm_Cartesian<int> prevdepth, currdepth;
	void searchDown(int x, int y, int colour, mm_image<int> &imagein, mm_Cartesian<int> &d);

    protected:

    public:
	void find(mm_image<int> &imageprev, mm_image<int> &imagecurrent, int &x, int &y);
	void setSkipXboundaries(int tempval);
	void setSkipYboundaries(int tempval);
	int getSkipYboundaries(void);
	void setYSearchRange(int tempval);
	void setFirstRunFlag(bool tempval);
	void setHJThresholdVal(int tempval);
	void setLinesDiffRange(int tempval);
	void setHoldPositionCondition(int tempval);

 	mm_findPipette()
	{
	    skipXboundaries=20;
	    skipYboundaries=10;
	    ySearchRange=20;
	    thresval=5;
	    linesDiffRange=20;
	    holdPositionCondition=30;
	    firstrecog=true;
	    scanliney=skipYboundaries;
	}

	~mm_findPipette()
	{
	}
	
};
};

#endif
