#ifndef MM_IMAGE_CONV_H
#define MM_IMAGE_CONV_H

#include "mm_image.h"
#include "mm_multi_array_conv.h"

namespace mimas {

/** Convolution of two images.
    This method provides convolution of two n-dimensional arrays. The
    resulting array will have the same size as the input-array \c x.
    Elements outside of the array-boundaries are assumed to be zero.

    The algorithm is intented to be used for convoluting an image with
    a small filter. If the filter is very big, it may be more efficient, to
    perform the convolution in fourier-space.
    @param x First array.
    @param y Second array.
    @return Result of convolution.
    @see fourierTransforms */
template< typename T >
inline mm_image< T > correlate( const mm_image< T > &x,
                                const mm_image< T > &y )
{
  return mm_image< T >( correlate( x.rawData(), y.rawData() ) );
}

#ifdef HAVE_LIBLAPACK

template< typename T >
mm_image< T > correlate_separable( const mm_image< T > &x,
                                   const mm_image< T > &f )
{
  return mm_image< T >( correlate_separable< T >( x.rawData(), f.rawData() ) );
}

#endif

}

#endif
