#ifndef MM_IMAGE_FILESINPUT_H
#define MM_IMAGE_FILESINPUT_H

#include <deque>
#include <string>
#include "mm_image_fileinput.h"

namespace mimas {

/** @addtogroup imageIO
    @{ */
/** @addtogroup imageInput
    @{ */
/** Class for reading a set of image-files.
    Example of usage:
    \code
    using namespace mimas;
    int main( int argc, char *argv[] ) {
       mm_image_filesinput< mm_rgba< unsigned char > > input;
       for ( int i=1; i<argc; i++ )
          input.addFileName( argv[i] );
       ...
       mm_image< mm_rgba< unsigned char > > image;
       while ( input >> image ) {
          ...
       };
    }
    \endcode
    @see mm_image_fileinput */
template< typename T >
class mm_image_filesinput: public mm_image_input< T >
{
 public:
  /// Constructor.
  explicit mm_image_filesinput(void) {}
  /** Add a filename to the list.
      A filename is added to the end of the list. */
  void addFileName( const std::string &fileName )
  { fileNames.push_back( fileName ); }
  /** Read the next image file.
      The first filename in the list is removed and the file with this
      name is opened. The content of the file is read into the image.
      @param image Object to store image in. */
  virtual void read( mm_image< T > &image ) throw (mm_exception);
 protected:
  /// List of filenames.
  std::deque< std::string > fileNames;
};

///@}

///@}

};

#include "mm_image_filesinput.tcc"

#endif
