#ifndef MM_IMAGE_FIREWIREINPUT_H
#define MM_IMAGE_FIREWIREINPUT_H

#include "mm_config.h"

#ifdef HAVE_LIBDC1394
#include <libraw1394/raw1394.h>
#include <libdc1394/dc1394_control.h>
#include "mm_image_input.h"

namespace mimas {

/** @addtogroup imageIO
    @{ */
/** @addtogroup imageInput
    @{ */
/** Image input from firewire digital camera (video1394).
    At the moment only grabbing of grayscale and YUV422-encoded images is
    supported.
    Grabbing has been tested with the following device:
    \li Basler A302fc

    See <A HREF="http://www.rmatsumoto.org/camera/dc1394-ohphone-old.html">Ryutaroh Matsumoto's</A>
    page for more information.

    @author Jan Wedekind (jan@wedesoft.de) 
    @date Wed Aug  4 13:53:59 GMT 2005 */
template< typename T >
class mm_image_firewireinput: public mm_image_input< T >
{
 public:
  /** Constructor.
      At the moment, there is no support to get a list of available modes or
      automatically select the best mode.
      You need to use f.e. the software `coriander', if you want to know
      the capabilities of the camera. */
  mm_image_firewireinput( int _mode,
                          int _framerate = FRAMERATE_15,
                          int _node = 0,
                          const char *_device = NULL,
                          int _channel = 0,
                          int _speed = SPEED_400 ) throw (mm_exception);
  ///
  virtual ~mm_image_firewireinput(void);
  ///
  virtual void read( mm_image<T> &image ) throw (mm_exception);
  ///
  dc1394_feature_info get_feature( int _id ) throw (mm_exception);
  ///
  void set_feature_value( int _id, unsigned int _value ) throw (mm_exception);
  ///
  unsigned int get_feature_value( int _id ) throw (mm_exception);
 protected:
  ///
  raw1394handle_t handle;
  ///
  dc1394_cameracapture camera;
  ///
  int mode;
  ///
  int node;
};

///@}

///@}

};

#include "mm_image_firewireinput.tcc"
#endif

#endif
