#ifndef MM_MESAOUTPUT_H
#define MM_MESAOUTPUT_H

#include <map>
#include "mm_image.h"
#include "mm_image_output.h"
#include "mm_timer.h"
#include "mm_x11_window.h"

namespace mimas {

/** @addtogroup imageIO
    @{ */
/** @addtogroup imageOutput
    @{ */
/** Display images using Xlib and OpenGL.
    Displaying images with this class may not work properly, if another
    part of the program is consuming X11-events.

    Objects of this class must be working on a single display.

    This class is not thread-safe.

    @see mm_mesa_base */
template< typename T >
class mm_image_mesaoutput: public mm_image_output< T >
{
 public:
  /** Open X11-window.
      @param _display X11 display.
      @param _width Initial width of drawing area.
      @param _height Initial height of drawing area. */
  mm_image_mesaoutput( mm_x11_display_ptr _display,
                       int _width = 32, int _height = 32,
                       bool depthBuffer = false );
  ///
  virtual void write( const mm_image< T > &image );
  ///
  virtual void repaint(void);
  ///
  static std::map< Window, mm_image_mesaoutput< T > * > windows;
  ///
  void callBack( XEvent &event );
  ///
  void wait( int milliSeconds );
  ///
  void delay( int milliSeconds ) { interval = milliSeconds; }
  ///
  mm_x11_window_ptr get_window(void) { return window; }
 protected:
  ///
  static void callback( void *arg );
  ///
  mm_x11_window_ptr window;
  ///
  mm_image< T > image;
  ///
  mm_timer time;
  ///
  int interval;
  ///
  int lag;
};

/// Stream-operator for displaying images.
template< typename T >
inline mm_image_mesaoutput< T > &operator<<
( mm_image_mesaoutput< T > &output, const mm_image< T > &image )
{ output.write( image ); return output; }

/// Stream-operator for waiting.
template< typename T >
inline mm_image_mesaoutput< T > &operator<<
( mm_image_mesaoutput< T > &output, _wait wait )
{ output.wait( wait.milliSeconds ); return output; }

/// Stream-operator for adjusting frame-rate.
template< typename T >
inline mm_image_mesaoutput< T > &operator<<
( mm_image_mesaoutput< T > &output, _delay delay )
{ output.delay( delay.milliSeconds ); return output; }

///@}

///@}

};

#include "mm_image_mesaoutput.tcc"

#endif
