#define GLX_GLXEXT_PROTOTYPES
#include <GL/glx.h>

namespace mimas {

template< typename T >
mm_image_mesaoutput< T >::mm_image_mesaoutput( mm_x11_display_ptr _display,
                                               int _width, int _height,
                                               bool depthBuffer ):
  window( new mm_x11_window( _display, _width, _height, &callback, this, depthBuffer ) ),
  interval( 0 ), lag( 0 )
{
}

template< typename T >
void mm_image_mesaoutput< T >::wait( int milliSeconds )
{
  mm_timer t;
  window->process_events();
  double remaining = milliSeconds * 0.001 - t.elapsed();
  if ( remaining > 0 ) usleep( (int)( remaining * 1.0E+6 ) );
}

template< typename T >
void mm_image_mesaoutput< T >::callback( void *arg )
{
  ((mm_image_mesaoutput< T > *)arg)->repaint();
}


template< typename T >
void mm_image_mesaoutput< T >::write( const mm_image< T > &_image )
{
  window->resize( _image.getWidth(), _image.getHeight() );

  if ( interval > 0 ) {
    int delay = interval - (int)( time.elapsed() * 1000 ) - lag;
    lag = delay > 0 ? 0 : -delay;
    wait( delay );
  } else {
    lag = 0;
  };

  time.reset();

  image = _image;
  repaint();
}

};
