#ifndef MM_IMAGE_MPLAYEROUTPUT_H
#define MM_IMAGE_MPLAYEROUTPUT_H

#include "mm_image_output.h"
#include "mm_rgba.h"

namespace mimas {

/** @addtogroup imageIO
    @{ */
/** @addtogroup imageOutput
    @{ */
/** Display tool that sits over Mplayer.
    @author Bala Amavasai (bala@amavasai.org)
    @date Fri May 14 19:06:10 BST 2004
    $Header: /cvs/mimas/include/mm_image_mplayeroutput.h,v 1.2 2005/06/03 13:49:53 engjw Exp $ */
template< typename T >
class mm_image_mplayeroutput: public mm_image_output< T >
{
 public:
  /** Constructor.
      @param _width Width of display.
      @param _height Height of display. */
  mm_image_mplayeroutput( int _width, int _height )
    throw (mm_exception);
  /// Destructor.
  virtual ~mm_image_mplayeroutput(void);
  ///
  virtual void write( const mm_image< T > &image );
 protected:
  /// Width of display.
  int width;
  /// Height of display.
  int height;
  ///
  std::string pipename;
  /// The pipe for images.
  int imgpipe;
  /// The pipe for opening MPlayer.
  FILE *mplayer;
};

template<>
void mm_image_mplayeroutput< mm_rgba< unsigned char > >::write
  ( const mm_image< mm_rgba< unsigned char > > &image );
template<>
void mm_image_mplayeroutput< unsigned char >::write
  ( const mm_image< unsigned char > &image );

/// Stream-operator for displaying image with mplayer.
template< typename T, typename U >
inline mm_image_mplayeroutput< T > &operator<<
  ( mm_image_mplayeroutput< T > &output,
    const mm_image< U > &image )
{
  if ( typeid( T ) == typeid( U ) )
    output.write( (mm_image< T > &)image );
  else {
    output.write( mm_image< T >( image ) );
  };
  return output;
}

///@}

///@}

};

#include "mm_image_mplayeroutput.tcc"

#endif
