#ifndef MM_IMAGE_OP_TCC
#define MM_IMAGE_OP_TCC

#include "mm_multi_array_op.h"

namespace mimas {

// Jan Wedekind <jan@wedesoft.de>
// Mon Jun 13 17:19:18 UTC 2005

// Right-hand image-scalar operation (internal and external).
#define MM_IMAGE_SCALAR_OP_NOSWAP( INAME, ENAME )                       \
  template< typename T >                                                  \
  mimas::mm_image< T > INAME( mimas::mm_image< T > &image, const T &x ) { \
    INAME( image.rawData(), x );                                          \
    return image;                                                         \
  }                                                                       \
  template< typename T >                                                  \
  mimas::mm_image< T > ENAME( const mimas::mm_image< T > &image,          \
                              const T &x ) {                              \
    return mimas::mm_image< T >( ENAME( image.rawData(), x ) );           \
  }

// Left-hand image-scalar operation (external only).
#define MM_IMAGE_SCALAR_OP_SWAP( INAME, ENAME )                    \
  MM_IMAGE_SCALAR_OP_NOSWAP( INAME, ENAME )                       \
  template< typename T >                                            \
  mimas::mm_image< T > ENAME( const T &x,                           \
                              const mimas::mm_image< T > &image ) { \
    return mimas::mm_image< T >( ENAME( x, image.rawData() ) );   \
  }

// Image-image operation (internal and external).
#define MM_IMAGE_IMAGE_OP( INAME, ENAME )                                 \
  template< typename T >                                                  \
  mimas::mm_image< T > INAME( mimas::mm_image< T > &image,                \
                              const mimas::mm_image< T > &x ) {           \
    INAME( image.rawData(), x.rawData() );                                \
    return image;                                                         \
  }                                                                       \
  template< typename T >                                                  \
  mimas::mm_image< T > ENAME( const mimas::mm_image< T > &image,          \
                              const mimas::mm_image< T > &x ) {           \
    return mimas::mm_image< T >( ENAME( image.rawData(), x.rawData() ) ); \
  }

// Image-scalar operation.
#define MM_IMAGE_SCALAR_OP( INAME, ENAME, TYPE )   \
  MM_IMAGE_SCALAR_OP_##TYPE( INAME, ENAME )

// External operations on a single image.
#define MM_IMAGE_OP_EXTERN( ENAME, OBJECT )                             \
  template< typename T >                                                \
  inline mimas::mm_image< typename OBJECT< T >::result_type > ENAME      \
    ( const mimas::mm_image< T > & x )                                  \
  {                                                                     \
    return                                                              \
      mimas::mm_image< typename OBJECT< T >::result_type >( ENAME( x.rawData() ) ); \
  }

// External operations on a single image.
#define MM_IMAGE_OP_INTERN( INAME, OBJECT )                             \
  template< typename T >                                                \
  inline mimas::mm_image< typename OBJECT< T >::result_type > & INAME   \
  ( mimas::mm_image< T > & x )                                          \
  {                                                                     \
    INAME( x.rawData() );                                               \
    return x;                                                           \
  }

#define MM_IMAGE_OP( INAME, ENAME, OBJECT )     \
  MM_IMAGE_OP_INTERN( INAME, OBJECT )           \
  MM_IMAGE_OP_EXTERN( ENAME, OBJECT )

#define MM_IMAGE_OP_EX( ENAME, OBJECT )         \
  MM_IMAGE_OP_EXTERN( ENAME, OBJECT )

// Element-wise multiplication
MM_IMAGE_IMAGE_OP( operator*=, operator* )
// Element-wise division
MM_IMAGE_IMAGE_OP( operator/=, operator/ )
// Element-wise addition
MM_IMAGE_IMAGE_OP( operator+=, operator+ )
// Element-wise subtraction
MM_IMAGE_IMAGE_OP( operator-=, operator- )
// Sum of squares
MM_IMAGE_IMAGE_OP( sumSquaresIt, sumSquares )

// Scalar multiplication
MM_IMAGE_SCALAR_OP( operator*=, operator*, SWAP )
// Scalar division
MM_IMAGE_SCALAR_OP( operator/=, operator/, SWAP )
MM_IMAGE_SCALAR_OP( operator+=, operator+, SWAP )
MM_IMAGE_SCALAR_OP( operator-=, operator-, SWAP )

// Absolute value.
MM_IMAGE_OP( absoluteIt, absolute, _abs )
// Square root.
MM_IMAGE_OP( squareRootIt, squareRoot, _sqrt )

// Threshold values.
MM_IMAGE_SCALAR_OP( thresholdIt, threshold, NOSWAP )


};

#endif
