#include <qpixmap.h>
#include "mm_rgba.h"

namespace mimas {

template< typename T >
mm_image_qtoutput< T >::mm_image_qtoutput( QApplication &_qApp ):
    qApp(&_qApp), label( new QLabel( NULL ) ),
    interval( 0 ), lag( 0 ), windowWasOpen( false )
{
  label->setBackgroundMode( Qt::NoBackground );
}

template< typename T >
void mm_image_qtoutput< T >::wait( int milliSeconds )
{
  if ( milliSeconds > 0 ) {
    mm_timing t( qApp, milliSeconds );
    pause();
  } else
    qApp->processEvents();
}

template< typename T >
void mm_image_qtoutput< T >::pause(void)
{
  qApp->setMainWidget( label.get() );
  label->show();
  
  // Wait for user to close window.
  qApp->exec();

  qApp->setMainWidget( NULL );
  windowWasOpen = false;
}

template< typename T >
void mm_image_qtoutput< T >::writePixmap( const QPixmap &pix )
{
  int delay;
  if ( interval > 0 ) {
    delay = interval - (int)( time.elapsed() * 1000 ) - lag;
    lag = delay > 0 ? 0 : -delay;
  } else {
    delay = 0;
    lag = 0;
  };
  
  bool hidden = label->isHidden();
  if ( hidden && windowWasOpen ) {
    mm_image_output< T >::state = false;
  } else {
    wait( delay );
    time.reset();
    label->setPixmap(pix);
    label->resize( pix.size() );
    if ( hidden ) label->show();
    windowWasOpen = true;
  };
  wait( 0 );
}

};
