#ifndef MM_IMAGE_V4LINPUT_H
#define MM_IMAGE_V4LINPUT_H

#include "mm_image_input.h"
#include <linux/videodev.h>
#include "mm_rgba.h"

namespace mimas {

/** @addtogroup imageIO
    @{ */
/** @addtogroup imageInput
    @{ */
/** Class for reading images from a video4linux device.
    The class has been successfully tested for colour-images (i.e.
    T = mm_rgba< unsigned char >) and the following devices:
    \li Logitech QuickCam Pro 4000
    \li BT848A video (Imagenation PXC20)
    \li BT878 (Unknown generic)

    See <A HREF="file:///usr/src/linux/Documentation/video4linux/API.html">V4L-API</A>
    for more information about video4linux.
    
    @author Bala Amavasai (bala@amavasai.org)
    @author Jan Wedekind (jan@wedesoft.de)
    @date Wed Mar  2 14:58:34 GMT 2005 */
template< typename T >
class mm_image_v4linput: public mm_image_input< T >
{
 public:
  mm_image_v4linput( const std::string &_device,
		     int channel = 0,
		     int width = -1, int height = -1,
		     int channel_norm = VIDEO_MODE_PAL ) throw (mm_exception);
  ~mm_image_v4linput(void);
  virtual void read( mm_image<T> &image ) throw (mm_exception);
  int getWidth(void) const { return win.width; }
  int getHeight(void) const { return win.height; }
  void setSensivity( __u16 brightness,
                     __u16	hue,
                     __u16	colour,
                     __u16	contrast ) throw (mm_exception);
 protected:
  int xioctl( int request, void *arg );
  void selectPalette(void) throw (mm_exception);
  std::string device;
  int fd;
  void *map;
  struct video_channel chan;
  struct video_picture pic;
  struct video_window win;
  struct video_mbuf buf;
};

///@}

///@}

};

#include "mm_image_v4linput.tcc"

#endif
