//
// line structures
//
// stuart meikle
//Thu Mar  2 10:42:32 2000
//

// future improvement (?) we could make line a subclass of vector 
// as a vector is a line which starts at 0,0 ! maybe vector should be a subclass of
// line , etc ,etc

#ifndef MM_LINE_H
#define MM_LINE_H

#include <boost/numeric/ublas/vector.hpp>
#include <boost/smart_ptr.hpp>
#include <list>
#include "mm_object.h"
#include "mm_angle.h"

namespace mimas {
/** updated by stu 20030228. removing the vectors-as-pointers. this cuts down on the
    number of new's and delete's we need to do and speed things up loads.
    Ive removed the base class too for now. there doesn't really seem to be much need
    for it, and again, it slows down construction times.

    I'll leave you to work this one out. There should be lots of overloaded
    operators for lines too, but i havent got around to it yet. */
class mm_line
{
    public:
  typedef boost::numeric::ublas::vector< double > Vector;
    private:
	// i don't think we'll need a co-ordinate system description for a line?
	// or maybe we do . i don't know. perhaps the co-ordinate system description
	// goes into mm_object base class ?

  Vector start,end;

    public:
   mm_line(void): start(2), end(2)
	{
	}

   mm_line( double sx, double sy, double ex, double ey ): start( 2 ), end( 2 )
	{
     start(0) = sx; start(1) = sy;
     end  (0) = ex; end  (1) = ey;
	}

   mm_line( const Vector &_start, const Vector &_end ):
     start( _start ), end( _end ) {}

	//WARNING ! use getVector().unit() to do a tina-like line->v command
   Vector getVector( void ) const
	{
     return end - start;
	}

   Vector findMidPoint(void) const
	{
     return 0.5 * ( start + end );
	}

	// mm_vector	rotate( mm_angle a );
	Vector getIntersection( const mm_line &l2, bool *on_this_line, bool *on_that_line ) const;
   Vector getIntersectionAngle( const mm_line &l2, mm_angle *retval) const;
	const Vector &getStart(void) const { return start; }
	const Vector &getEnd(void) const { return end; }
   Vector &getStart(void) { return start; }
	Vector &getEnd(void) { return end; }
  double length(void) const { return norm_2( end - start ); }
	mm_angle	angle( const mm_line &l ) const;
	//mm_line&	operator=( const mm_line& a );

};

///
typedef boost::shared_ptr< mm_line > mm_line_ptr;

///
typedef std::list< mm_line_ptr > mm_line_ptr_list;

///
typedef boost::shared_ptr< mm_line_ptr_list > mm_line_ptr_list_ptr;

};
#endif

