#ifndef MM_MESA_ONSCREEN_H
#define MM_MESA_ONSCREEN_H

#include "mm_mesa_base.h"
#include "mm_x11_window.h"

namespace mimas {

/** Mesa on-screen rendering.
    Support for on-screen rendering using Mesa.

    No stencil- and accumulation-buffer are allocated.
    @see mm_x11_window */
class mm_mesa_onscreen: public mm_mesa_base
{
 public:
  /** Constructor.
      All parameters must be destructed by the calling program.
      @param _window Window to open GLX context for. */
  mm_mesa_onscreen( mm_x11_window_ptr _window ) throw (mm_exception);
  /** Destructor.
      The destructor will destruct the OpenGL-context and call
      \c mm_mesa_base::finish() . */
  virtual ~mm_mesa_onscreen(void);
  /** Finish on-screen rendering.
      This method calls \c glFinish() and \c mm_mesa_base::finish() . */
  virtual void finish(void);
  ///
  virtual void ortho2D(void) const;
 protected:
  ///
  mm_x11_window_ptr window;
  /// GLX context for on-screen rendering.
  GLXContext context;
  /// State of object.
  bool finishOnScreen;
};

};

#endif
