#include <iostream>
#include <fstream>
#include <values.h>
#include <math.h>
#include "mm_object.h"
#include "mm_image.h"
#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/vector.hpp>
#include <boost/numeric/ublas/io.hpp>

#ifndef MM_MOMENTS_TOOL_H
#define MM_MOMENTS_TOOL_H

namespace mimas {
	/**
	 * Computation of image moments and Hu invariant moments
	 * @author Bala Amavasai (bala@amavasai.org)
	 * @date Tue Apr 12 16:00:05 BST 2005
	 * $Header: /cvs/mimas/include/mm_moments_tool.h,v 1.1 2005/04/12 15:00:27 bpa Exp $
	 */

	class mm_moments_tool: public mm_object
	{
		public:
			typedef boost::numeric::ublas::matrix< double > Matrix;
			typedef boost::numeric::ublas::vector< double > Vector;

		private:


		public:
			mm_moments_tool(void); ///< constructor
			boost::numeric::ublas::matrix< double > findMoments(const mm_image< unsigned char > imagein, const unsigned char bglevel=0, bool binaryImage=false); ///< find image moments and return a 5x5 matrix containing the moments. The bglevel parameter is the background grey level that is ignored when find the moment
			boost::numeric::ublas::vector< double > findHuInvariantMoments(const mm_image< unsigned char > imagein, const unsigned char bglevel=0, bool binaryImage=false); ///< find the 7 Hu invariant moments and return it as a vector. Note that the vector indices from 0..6, so Hu moment 1 is 0, Hu moment 2 is 1, etc. The bglevel parameter sets the background grey level so that these pixels are ignored in the image.


	};

	
}

#endif
