#ifndef MM_ARRAY_CONVOLUTION_H
#define MM_ARRAY_CONVOLUTION_H

// For compatibility with boost <1.31 when apply_if replaced with eval_if
#include <boost/version.hpp>
#if BOOST_VERSION<=103100
#include <boost/mpl/apply_if.hpp>
#ifndef eval_if
#define eval_if apply_if
#endif
#else
#include <boost/mpl/eval_if.hpp>
#endif

#include <boost/array.hpp>
#include <boost/multi_array.hpp>
#include <map>
#include <vector>
#include "mm_config.h"
#include "mm_linalg.h"

namespace mimas {

/** Convolution of two multi-arrays.
    This method provides convolution of two n-dimensional arrays. The
    resulting array will have the same size as the input-array \c x.
    Elements outside of the array-boundaries are assumed to be zero.

    The algorithm is intented to be used for convoluting an array with
    a small filter. If the filter is very big, it may be more efficient, to
    perform the convolution in fourier-space.
    @param x First array (the big one).
    @param y Second array (the small one).
    @return Result of convolution.
    @see fourierTransforms
    @author Jan Wedekind (jan@wedesoft.de)
    @date Fri Jul 15 21:37:32 UTC 2005 */
template< class _Array1, class _Array2 >
boost::multi_array< typename _Array1::element, _Array1::dimensionality >
  correlate( const _Array1 &x, const _Array2 &y );


template< typename T >
boost::multi_array< T, 2 > correlate_separable
  ( const boost::const_multi_array_ref< T, 2 > &x,
    const std::vector< boost::multi_array< T, 2 > > &f )
{
  return correlate( correlate( x, f[0] ), f[1] );
}

#ifdef HAVE_LIBLAPACK

template< typename T >
std::vector< boost::multi_array< T, 2 > > separate
   ( const boost::const_multi_array_ref< T, 2 > &array );

template< typename T >
boost::multi_array< T, 2 > correlate_separable
  ( const boost::const_multi_array_ref< T, 2 > &x,
    const boost::const_multi_array_ref< T, 2 > &f )
{
  boost::multi_array< T, 2 > result( boost::extents[ x.shape()[0] ]
                                                   [ x.shape()[1] ] );
  std::vector< boost::multi_array< T, 2 > > components( separate( f ) );
  return correlate_separable< T >( x, components );
}

#endif

};

#include "mm_multi_array_conv.tcc"

#endif
