#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/vector.hpp>
#include "mm_object.h"
#include "values.h"


#ifndef MM_NN_ASSOC_H
#define MM_NN_ASSOC_H

namespace mimas {
/**
 * Associative neural network using Hebb rule
 * @author Bala Amavasai (bala@amavasai.org)
 * @date Tue Jul  2 15:13:42 UTC 2002
 * $Header: /cvs/mimas/include/mm_nn_assoc.h,v 1.9 2005/02/22 14:17:34 engjw Exp $
 */
  class mm_nn_assoc: public mm_object
{
  public:
      typedef boost::numeric::ublas::matrix< double > Matrix;
      typedef boost::numeric::ublas::vector< double > Vector;

	private:
		Matrix weights;
		

   
	public:
		mm_nn_assoc(void); ///< constructor
		void train(Matrix& input_data, Matrix& target_data); ///< train data using input_data/target_data pairs. It is recommended that the values in the matrix are binarised.
      Vector recall(Vector& x); ///< given input_data return the predicted output
		void useWeights(Matrix& weight_matrix); ///< use the weights given by weight_matrix;
		Matrix getWeights(void); ///< return the weights that are stored
};


mm_nn_assoc::mm_nn_assoc()
{
}

void mm_nn_assoc::train(Matrix& x, Matrix& y)
{
	Matrix s, t, curweights;

   MMERROR( x.size1() == y.size1(), mm_exception, ,
            "mm_nn_assoc::train - input/output matrix rows not equal" );

	// initialise weight matrix
	weights.resize( x.size2(), y.size2() );
	curweights.resize( x.size2(), y.size2() );

   weights = weights + prod( trans( x ), y );

	//weights.save("hebb.weights");
}

mm_nn_assoc::Vector mm_nn_assoc::recall(Vector& x)
{
   MMERROR( x.size() == weights.size1(),
            mm_exception, ,
            "mm_nn_assoc::recall - input vector has incorrect dimensions" );

	Vector y( prod( x, weights ) );

	double ymax=-1.0;
	double ymin=MAXDOUBLE;

	for (int i=0; i<(signed)y.size(); i++)
	{
		if ((ymax) < y(i)) ymax=y(i);
		if ((ymin) > y(i)) ymin=y(i);
	}

	return y;
	
	
}


// use the weights given by weight_matrix;
void mm_nn_assoc::useWeights(Matrix& weight_matrix)
{
   //if (weights.initialised()) weights.clear();
	weights=weight_matrix;
}


// return the weights that are stored	
mm_nn_assoc::Matrix mm_nn_assoc::getWeights(void)
{
	return weights;
}
}
#endif
