//
// Principal component analysis tool
//


#ifndef MM_PCA_TOOL_H
#define MM_PCA_TOOL_H

#include <iostream>
#include <cmath>
#include "mm_object.h"

namespace mimas {
/**
 * Principal component analysis (PCA) tool class
 * @author Bala Amavasai (bala@amavasai.org)
 * @date Mon Jun 17 12:55:13 UTC 2002
 * $Header: /cvs/mimas/include/mm_pca_tool.h,v 1.8 2004/12/09 23:27:19 engjw Exp $
 */

  class mm_pca_tool: public mm_object
{
	private:
		mm_matrix_tool scores, loadings, variances;
		bool pca_performed;
	
	public:
		mm_pca_tool(void); ///< constructor
		void pca(mm_matrix_tool& X); ///< perform PCA on columns of input matrix X
		void project(mm_matrix_tool& Y, int n); ///< reduced to n component and store new representation in Y
};

mm_pca_tool::mm_pca_tool(void)
{
	pca_performed=false;
}

void mm_pca_tool::pca(mm_matrix_tool& X)
{
	mm_matrix_tool u, d, v, centeredX;
	
   MMERROR( X.initialised(), mm_exception, ,
            "mm_pca_tool::pca - input matrix not initialised" );

	centeredX=X;

	// remove column means from centeredX
	for (int j=0; j<centeredX.getCols(); j++)
	{
		double mean=0.0;

		for (int i=0; i<centeredX.getRows(); i++)
			mean+=centeredX(i,j);

		mean/=centeredX.getRows();

		for (int i=0; i<centeredX.getRows(); i++)
			centeredX(i,j)=centeredX(i,j)-mean;

	}
	
	centeredX.svd(u,d,loadings);
	
	scores=u*d;

	variances.redim(d.getCols(),1);
	for(int i=0; i<d.getCols(); i++)
		variances(i,0)=d(i,i);

	pca_performed=true;
	
}

void mm_pca_tool::project(mm_matrix_tool& Y, int n)
{
	if (!pca_performed) return;

   MMERROR( n>=0 && n<loadings.getCols(), mm_exception, ,
            "mm_pca_tool::reduce - number of components out of range" );
	
	Y.redim(scores.getRows(),loadings.getRows());

	mm_matrix_tool t,p;

	t.redim(scores.getRows(),n);

	for (int i=0; i<scores.getRows(); i++)
		for (int j=0; j<n; j++)
			t(i,j)=scores(i,j);
	
	p.redim(loadings.getRows(),n);

	for (int i=0; i<loadings.getRows(); i++)
		for (int j=0; j<n; j++)
			p(i,j)=loadings(i,j);


	p.save("p.dat");
	t.save("t.dat");
	
	p.transpose();

	Y=t*p;
	
}
}

#endif


