#ifndef MM_POL_MODEL_H
#define MM_POL_MODEL_H

#include <boost/numeric/ublas/vector.hpp>
#include <boost/shared_ptr.hpp>
#include <list>
#include <cmath>
#include "mm_line.h"
#include "mm_pol.h"

namespace mimas {
/** Part of the pairs of lines tool. 
    Forms the representation of a given model in terms of pairs of lines
    extracted from that model. 
    
    small open structure used to store important info about a given
    line. used partly to reduce the workload and speed the algo up
    
    stumeikle Mon 21 Oct 2002 19:49:13 CEST
    
    $Header: /cvs/mimas/include/mm_pol_model.h,v 1.11 2004/12/12 23:31:18 engjw Exp $ */
class mm_pol_line_info
{
    public:
  typedef boost::numeric::ublas::vector< double > Vector;
  mm_line_ptr line;	///< line from the model/scene.
	double		len;	///< length of the line
     Vector unitv;	///< unit v of the line
     Vector midpoint;///< midpoint of the line
	
       mm_pol_line_info(): unitv(2),midpoint(2)
    {
    }
};

typedef boost::shared_ptr< mm_pol_line_info > mm_pol_line_info_ptr;
typedef std::list< mm_pol_line_info_ptr > mm_pol_line_info_ptr_list;


class   mm_pol_locator;


/**
 * 
 * Pairs of lines model . shape description in terms of pairs of lines
 * found within the model view. 
 *
 */
class	mm_pol_model : public mm_pol
{
 public:
  typedef boost::numeric::ublas::vector< double > Vector;
private:
    //internal types:
    class linepair
    {
	public:
   typedef boost::numeric::ublas::vector< double > Vector;
	mm_pol_line_info_ptr linfo1, linfo2;
	double			dist1,dist2;
	Vector intersection;
				//point of intersection of the two lines

	linepair(): intersection(2) {
     //linfo1=linfo2=0;
   }
	// ~linepair(){ delete linfo1; delete linfo2; }
    };

    typedef boost::shared_ptr< linepair > linepair_ptr;
    typedef std::list< linepair_ptr > mm_linepair_ptr_list;
  typedef boost::shared_array< mm_linepair_ptr_list > mm_linepair_ptr_list_array;

    //internal vars
    mm_linepair_ptr_list_array hash_bin;	///< hashbin[num_abins]. used to group pairs of lines based on angle between them
    mm_line_ptr_list model_lines;	///< the model lines. *only* used for viewing purposes and delete management

    double		loc_thres;	///< threshold used when locating the model. set automatically but depends on num linepairs in model
    double		radius;		///< simple measure of the size of the model
    int			num_lps;	///< number of linepairs forming the model
    bool		scale_inv;	///< will we try to match w no regard to scale?

    //private funcs
    /**
    *
    * Create the model from the given list of straight line segements.
    *
    */
    public:
    void		createFromLines( mm_line_ptr_list &slaps );

public:
  mm_pol_model(): 
    hash_bin( new mm_linepair_ptr_list[num_abins] )
    {
	loc_thres=0;
	num_lps=0;
	radius=0;
	scale_inv=false;
    }
//     ~mm_pol_model()
//     {
//       mm_linepair_ptr_list::iterator	lpi;
    
// 	for(int i=0;i<num_abins;i++)
// 	{
// 	    for(lpi=hash_bin[i].begin();lpi!=hash_bin[i].end();++lpi)
// 	    {
// 		delete *lpi;
// 	    }
// 	    hash_bin[i].clear();
// 	}
// 	delete[] hash_bin;
	
//     }

    /**
    * Create the model (rep) from the given image. Edge strings and straight lines will be extracted from 
    * the image automatically.
    *
    */
    void	createFromImageFile( const char* fn );
    
    /**
    * Compare the two given pairs of lines and if they match, make an entry into the location histogram
    * -locate- of the possible centroid position and angle of the model.
    *
    */
    void	match( mm_pol_locator& locate, const mm_pol_line_info &l1_orig, 
				      	      const mm_pol_line_info &l2_orig );
    /**
    *
    * Compare the two lines l1 and l2 and calculate the angle between them. 
    *
    */
    Vector getIntersectionAngle( const mm_pol_line_info &l1,
				      const mm_pol_line_info &l2,
				      mm_angle *retval);

    ///< inspectors and modifiers...
    double	getThreshold() { return loc_thres; }
    double	getRadius()	{ return radius; }
    void	setRadius(double r) { radius=r; }
    bool	scaleInvariant();
    void	setScaleInvariance( bool r ) {scale_inv=r; }
    mm_line_ptr_list getModelLines();
};

}
#endif
