#ifndef MM_RGBA_H
#define MM_RGBA_H

#include <cmath>

namespace mimas {

  template <typename T>
    class mm_rgba
    {
    protected:
      T b,g,r,a;
		
    public:
		
      mm_rgba(void)
	{
	  r=g=b=a=T();
	}
	       
      template< class U >
		mm_rgba(const mm_rgba< U > &rhs)
		{
        r = (T)rhs.getRed();
        g = (T)rhs.getGreen();
        b = (T)rhs.getBlue();
        a = (T)rhs.getAlpha();
		}

      mm_rgba( T _r, T _g, T _b, T _a = T() ):
      b(_b), g(_g), r(_r), a(_a)
	{
	}
   
   template< typename U >
   explicit mm_rgba(const U &val)
	{
	  r=(T)val;
	  g=(T)val;
	  b=(T)val;
	  a=T();
	}
      
		//inline functions to avoid overhead of the call,
		//especially used for convert_trait
      inline T getRed(void) const
	{
	  return r;
	}

      inline T getGreen(void) const
	{
	  return g;
	}

      inline T getBlue(void) const
	{
	  return b;
	}
      
      inline T getAlpha(void) const
	{
	  return a;
	}
      

      void setRed(T val)
	{
	  r=val;
	}
      
      void setGreen(T val)
	{
	  g=val;
	}
      
      void setBlue(T val)
	{
	  b=val;
	}
      
      void setAlpha(T val)
	{
	  a=val;
	}
      
      void set(T val)
	{
	  r=g=b=a=val;
	}


      //mixes the 2 colors according to the ammount of transparency
      void overlay(T r_,T g_,T b_,T a_)
	{
	  //assumption that alpha take a value between 0 and 255
	  //should be corrected according to the type T
	  r = ((255 - a_)* r + a_ * r_)/255;
	  g = ((255 - a_)* g + a_ * g_)/255;
	  b = ((255 - a_)* b + a_ * b_)/255;
	}
      
      //overload casting to double
		operator double(void) const
      {
        double rwgt=0.3086;
        double gwgt=0.6094;
        double bwgt=0.0820;
        return rwgt * (double)r + gwgt * (double)g + bwgt * (double)b;
      }

      //overload casting to unsigned char
		operator unsigned char(void) const
      {
        double rwgt=0.3086;
        double gwgt=0.6094;
        double bwgt=0.0820;
        return (unsigned char)( rwgt * (double)r + gwgt * (double)g + bwgt * (double)b );
      }

      double getHue(void) const;///< returns number in the range 0-360
      double getSaturation(void) const; ///< returns saturation between 0 and 1.0
      
    };
 
}

#include "mm_rgba.tcc"

#endif
 
