#ifndef MM_SLAP_TOOL_H
#define MM_SLAP_TOOL_H

#include <boost/numeric/ublas/vector.hpp>
#include <boost/shared_ptr.hpp>
#include <cmath>
#include "mm_line.h"
#include "mm_image.h"
#include "mm_edge_string.h"
#include "mm_edge_string_op.h"

namespace mimas {
/**
 *
 * SLAP == Straight Line APproximation tool.
 *
 * use this tool to turn edge strings into lists of straight line segments.
 * Algorithm here is based on the original by Dr xxxx (can fbc provide this 
 * reference please?). Implementation by stu. Sun 13 Oct 2002 14:45:26 CEST
 *
 *
 * Algorithm takes a parameter- anglesigma which is used to determine how the
 * edge strings should be broken into straight line segments. A high value 
 * means that only large changes in the angle of an edge string will be 
 * encorporated into a single straight line, therefore producing a very 
 * approximate set of output lines (blocky). With a lower value the straight
 * lines more closely follow the edge strings and the output will look more
 * like the input. (smoother)
 *
 *
 * @author$
 * @date$
 * $Header: /cvs/mimas/include/mm_slap_tool.h,v 1.9 2004/12/12 23:31:18 engjw Exp $
 */
class	mm_slap_tool	///< A tool. Not derived from anything.
{
    private:
	double		anglesigma;///< what angle differences can be 'detected' by this tool? see above
			//use this to simplify parameter lists

    public:
    typedef		mm_string_el_ptr_list::iterator string_it;
    typedef boost::numeric::ublas::vector< double > Vector;

    /** 
    * Default constructor. Sets a default anglesigma value. 
    * NOTE: this value needs to be small otherwise the algorithm fails
    *     : as this implementation requires that tan(anglesigma) is 
    *     : approx= anglesigma
    */
    mm_slap_tool(void)
    {
	anglesigma = M_PI/32.0;	
    }
    
    mm_slap_tool( double d )
    {
	anglesigma = d;
    }

    /**
    * Return a list of straight lines from a given list of edge strings. 
    * A suitable list of edge strings is produced by, for example, the 
    * canny_tool.
    */
    mm_line_ptr_list findStraightLines( mm_edge_string_ptr_list& es_list );
    
    private:
    //the rest of the functions are internal to the tool, and shouldn't need to be
    //touched i imagine.
    mm_line_ptr_list poly_strings( mm_edge_string_ptr_list& es_list, double thres );
    mm_line_ptr_list poly_string( mm_edge_string& es , double thres );
    void		poly_prox( string_it start_it, string_it end_it, double thres, 
				    mm_line_ptr_list& result );

};
};

#endif
