//
// string element. used primarily for the extraction
// of edge strings by canny.
//
// stumeikle Sun May 12 21:05:26 2002
//

#ifndef MM_STRING_EL_H
#define MM_STRING_EL_H

#include <boost/shared_ptr.hpp>
#include "mm_edgel.h"
#include <list>

#define EDGE_GET_CONN_MASK  0x000000F0
#define EDGE_SET_CONN_MASK  0xFFFFFF0F
#define EDGE_CONN           0x00000020

namespace mimas {

///
typedef enum { unconnected, connection, junction, termination }mm_connect_type ;

class mm_string_el;
class mm_property_image;

///
typedef boost::shared_ptr< mm_string_el > mm_string_el_ptr;

///
typedef std::list< boost::shared_ptr< mm_string_el > > mm_string_el_ptr_list;

/// Private classes used only for string creation
class mm_string_el : public mm_edgel
{
    //this is additional information which is used to create the edge images, but which
    //should have a 1-1 correspondance to image (pixel) positions.
    private:
        int			flags;
        std::list< boost::weak_ptr< mm_string_el > > conn_list;
        boost::weak_ptr< mm_string_el > conn_c1, conn_c2;
				//pointers to string_el's owned elsewhere
	int			spur_len;
	bool			remove_me;
	bool			loop;
	mm_connect_type		connect;


	public:
	    mm_string_el( const mm_edgel &e )
	    {
		setPropertyType(edge_string);
	    
		//reset the string stuff vars
		flags =0;
		flags&=EDGE_SET_CONN_MASK;
		flags|=EDGE_CONN;
		// conn_c1 = conn_c2 = NULL;
		spur_len = 0;
		remove_me = false;
		loop = false;
		connect = unconnected;
	    
		//copy the edgel structure
		// mm_edgel	*me = (mm_edgel*)this;
		// *me = e;
      mm_edgel::operator=( e );
	    }
	    
	    void draw( mm_image< mm_rgba< unsigned char > >& output,
                  int i, int j, bool c)
	    {
		//change the color of the pixel if its connected, or junc
		//or terminal
		int	s = (int)getStrength();
		
		//we can change the colour depending on the connection type
		if (c)
		{
		    if (s<50)
		        s*=5;
		    else
			s=255;
		}
		else
		    s=255;

		switch( connect )
		{
		    case unconnected: output.setPixel( i,j, mm_rgba< unsigned char >(s,0,0) );
				      break;
		    case connection: output.setPixel( i,j, mm_rgba< unsigned char >(s,0,s) );
				      break;
		    case junction: output.setPixel( i,j, mm_rgba< unsigned char >(0,s,s) );
				      break;
		    case termination: output.setPixel( i,j, mm_rgba< unsigned char >(0,0,s) );
				      break;
				      
		}
	    }

       mm_string_el_ptr getConnC1()
       { return !conn_c1.expired() ? mm_string_el_ptr( conn_c1 ) : mm_string_el_ptr(); }
       mm_string_el_ptr getConnC2()
       { return !conn_c2.expired() ? mm_string_el_ptr( conn_c2 ) : mm_string_el_ptr(); }
       const mm_string_el_ptr getConnC1() const
       { return !conn_c1.expired() ? mm_string_el_ptr( conn_c1 ) : mm_string_el_ptr(); }
       const mm_string_el_ptr getConnC2() const
       { return !conn_c2.expired() ? mm_string_el_ptr( conn_c2 ) : mm_string_el_ptr(); }
	    void setConnC1( boost::weak_ptr< mm_string_el > d) { conn_c1 = d; }
	    void setConnC2( boost::weak_ptr< mm_string_el > d) { conn_c2 = d; }
       std::list< boost::weak_ptr< mm_string_el > >& getConnList()
       { return conn_list; }
       const std::list< boost::weak_ptr< mm_string_el > >& getConnList() const
       { return conn_list; }
	    int				getSpurLen(){ return spur_len; }
	    void			setSpurLen(int s) { spur_len=s; }
	    void			delConnList() 
					{ 
                 conn_list.clear();
					  //DONT delete c1,c2 here
					}
	    void			setRemoveMe() { remove_me = true; }
	    void			resetRemoveMe() {remove_me=false;}
	    bool			removeMe() { return remove_me; }
	    mm_connect_type		getConnectType(){ return connect; }
	    void			setConnectType( mm_connect_type c) { connect=c;}
	    bool			isLoop() { return loop; }
	    void			setLoop(bool b) { loop= b; }
            mm_edgel*			getEdgel() { return (mm_edgel*)this; }
					//(! hope the casting is dealt with properly!
					// ! if not change for dynamic_cast )
       void edgeSetConns( mm_property_image& esim, int x, int y, 
                          mm_string_el_ptr_list& junc_list,
                          mm_string_el_ptr_list& term_list );
};

}
#endif
