#ifndef MM_TIMER_H
#define MM_TIMER_H

#include <boost/shared_ptr.hpp>
#include <sys/time.h>
#include "mm_object.h"

namespace mimas {

/// Class for measuring time.    
class mm_timer
{
 public:
  /** Constructor starting timer.
      The constructor will call \c reset() to. */
  mm_timer(void) { reset(); }
  /** Restart timer.
      Reset the time elapsed to zero again. */
  void reset(void) { gettimeofday( &time, NULL ); }
  /** Get elapsed time.
      The time elapsed (in seconds) is returned.
      @return Time elapsed since last call of \c mm_timer or \c reset() . */
  double elapsed(void) const;
 protected:
  /// Time of construction of this object.
  struct timeval time;
};

///
typedef boost::shared_ptr< mm_timer > mm_timer_ptr;

};


#endif
