#ifndef MM_X11_DISPLAY_H
#define MM_X11_DISPLAY_H

#include <boost/shared_ptr.hpp>
#include <string>
#include <X11/Xlib.h>
#include "mm_exception.h"
#include "mm_object.h"

namespace mimas {

/** Wrapper for X11-display.
    This wrapper provides automatic destruction for an X11-display-variable. */
class mm_x11_display: public mm_object
{
 public:
  /** Constructor.
      Open X11-display.
      @param name Name of X11-display to use (default: use content of
      <TT>DISPLAY</TT>-variable. */
  mm_x11_display( const std::string &name = "" ) throw (mm_exception);
  /// Destructor.
  virtual ~mm_x11_display(void) { XCloseDisplay( display ); }
  /// Get pointer to X11-display.
  Display *get(void) { return display; }
 protected:
  /// X11 display-variable.
  Display *display;
};

///
typedef boost::shared_ptr< mm_x11_display > mm_x11_display_ptr;

};

#endif
