#ifndef MM_X11_WINDOW_H
#define MM_X11_WINDOW_H

#include <boost/shared_ptr.hpp>
#define GLX_GLXEXT_PROTOTYPES
#include <GL/glx.h>
#include <map>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include "mm_x11_display.h"

namespace mimas {

/// Class for opening X11 windows.
class mm_x11_window: public mm_object
{
 public:
  ///
   mm_x11_window( mm_x11_display_ptr _display, int _width, int _height,
                  void (*_paint)(void *), void *_arg,
                  bool depthBuffer = false );
  ///
  virtual ~mm_x11_window(void);
  ///
  int get_width(void) const { return width; }
  ///
  int get_height(void) const { return height; }
  ///
  XVisualInfo *get_visual_info(void) const { return visualInfo; }
  ///
  mm_x11_display_ptr get_display(void) { return display; }
  ///
  Window get_window(void) { return window; }
  /// Process pending events for all open windows.
  static void process_events( bool wait = false ) throw (mm_exception);
  /// Escape hasn't been pressed.
  operator bool(void) const { return state; }
  /// Escape was pressed.
  bool operator!(void) const { return !state; }
  /// Resize the window.
  void resize( int _width, int _height );
 protected:
  ///
  static bool state;
  ///
  static Bool waitForNotify( Display *, XEvent *e, char *arg );
  ///
  static mm_x11_display_ptr display;
  ///
  static std::map< Window, std::pair< void (*)(void *), void * > >
    windows;
  ///
  XVisualInfo *visualInfo;
  /// Colormap of X11-visual.
  Colormap colorMap;
  /// X11 window.
  Window window;
  ///
  int width;
  ///
  int height;
};

///
typedef boost::shared_ptr< mm_x11_window > mm_x11_window_ptr;

};

#endif
