#ifndef __MM_XML_NODE_REFERENCE_LIST_H
#define __MM_XML_NODE_REFERENCE_LIST_H

#include "mm_config.h"

#ifdef HAVE_XALAN

#include <boost/shared_ptr.hpp>
#include <XPath/NodeRefList.hpp>
#include <XPath/XObject.hpp>
#include "mm_object.h"
#include "mm_xml_reference.h"

namespace mimas {

class mm_xml_document;

/** @addtogroup xml
    @{ */
/** Wrapper for list of XML-nodes.
    Objects of this type are returned by the search-methods of class
    \c mm_xml_reference .
    @see mm_xml_reference
    @see mm_xml_document
    @author Jan Wedekind (jan@wedesoft.de)
    @date Wed Jan 12 13:53:41 UTC 2004 */
class mm_xml_node_reference_list: public mm_object
{
public:
#ifdef SIMPLEXPATH
  /// Constructor for internal use only.
  mm_xml_node_reference_list( const NodeRefList &_nodeList,
                              XMLDocument *_document ):
    nodeList(_nodeList), document(_document) {}
#else
  /// Constructor for internal use only.
  mm_xml_node_reference_list( boost::shared_ptr< XObjectPtr > _xObject,
                              mm_xml_document *_document ):
    xObject( _xObject ), nodeList( (*_xObject)->nodeset() ),
    document(_document) {}
#endif
  /// Copy-constructor.
  mm_xml_node_reference_list( const mm_xml_node_reference_list &l );
  /** Retrieve wrapper-object for a node.
      @param index Index of node to retrieve. Must be between \c 0 and
      \c getLength()-1 . */
  mm_xml_reference< XalanNode > item( int index ) const;
  /** Number of elements in the list.
      @return Number of elements. */
  int getLength(void) const;
protected:
#ifdef SIMPLEXPATH
  /// List of nodes.
  NodeRefList nodeList;
#else
  /// XObject holding list of nodes.
  boost::shared_ptr< XObjectPtr > xObject;
  /// Reference to list of nodes.
  const NodeRefListBase &nodeList;
#endif
  /// Pointer to parent document.
  mm_xml_document *document;
};

///@}

};

#endif

#endif
