#ifndef __MM_XML_STRX_H
#define __MM_XML_STRX_H

#include "mm_config.h"

#ifdef HAVE_XALAN

#include <iostream>
#include <util/XercesDefs.hpp>
#include <util/XMLString.hpp>
#include <Include/PlatformDefinitions.hpp>

#ifdef XERCES_HAS_CPP_NAMESPACE
using namespace XERCES_CPP_NAMESPACE;
#endif
#ifdef XALAN_HAS_CPP_NAMESPACE
using namespace XALAN_CPP_NAMESPACE;
#endif

namespace mimas {

/** @addtogroup xml
    @{ */
/** Conversion to and from unicode.
    Mainly for internal use.
    @todo Replace mm_xml_strx with XalanDOMString (if possible)
    @author Jan Wedekind (jan@wedesoft.de)
    @date Wed Jan 12 13:53:41 UTC 2004 */
template <class To, class From>
class mm_xml_strx
{
public:
  /** Constructor.
      @param aXMLStr The string to be  converted. */
  mm_xml_strx( const From *_buffer )
     { buffer = XMLString::transcode( _buffer ); }
  /// Copy-constructor.
  mm_xml_strx( const mm_xml_strx<To,From> &_strX ):
    buffer( XMLString::replicate( _strX.buffer ) ) {}
  /** Destructor.
      The destructor calls \c free on the buffer. */
  virtual ~mm_xml_strx(void) { free( buffer ); }
  /** Length of buffer.
      @return Length of string stored in the buffer. */
  int getLength(void) const
  { return XMLString::stringLen( buffer ); }
  /** Size of buffer.
      @return Size of buffer in bytes. */
  int getSize(void) const { return ( getLength() + 1 ) * sizeof(To); }
  /** Get pointer to buffer.
      @return Pointer to string contained in buffer. */
  const To *getBuffer() const { return buffer; }
protected:
  /// Buffer for storing the string.
  To *buffer;
};

/** Stream-operator for printing string.
    @param target Stream to print string to.
    @param toDump Object containing string to print. */
template <class To, class From>
inline std::ostream& operator<<( std::ostream& target,
                                 const mm_xml_strx<To,From> &toDump)
{
  target << toDump.getBuffer();
  return target;
}

///@}

};

#endif

#endif
