#include "mm_config.h"
#include "mm_object.h"
#include "mm_image.h"
#include "mm_canny.h"

#ifdef HAVE_LIBFFTW3

/* the executable in 'registration' takes the two test images
   in parameters and the size of the square area two be used for
   registration.
   Denis Tessier Sep 2004 */

namespace mimas{

class reg_param
{
 public:
  int shift_x;
  int shift_y;
  double angle;
  double scale;
  reg_param(int x = 0, int y = 0, double a = 0, double sc = 0)
    :shift_x(x),shift_y(y),angle(a),scale(sc){}
};


class registration_parameters : public mm_object
{
private :
  double angle;
  double scale;
  int shiftx;
  int shifty;
 
 public:
  void setAngle(double a)     {angle = a;}
  void setScale(double s)     {scale = s;}
  void setShift(int x, int y) {shiftx = x; shifty = y;}
 
  double getAngle(){return angle;}
  double getScale(){return scale;}
  int getShiftx(){return shiftx;}
  int getShifty(){return shifty;}

  reg_param getRegParam(){return reg_param(shiftx, shifty, angle, scale);}
  
  void registering_images(mm_image<int>& im1, mm_image<int>& im2);
  void registering_images_shift_only(mm_image<int>& im1, mm_image<int>& im2);
 
  registration_parameters(void){angle = 0.0; scale = 1.0; shiftx = 0; shifty = 0;}
  virtual ~registration_parameters(void) {}
};


mm_image<int> registration (mm_image<int>& inimage, mm_image<int>& inimage2, reg_param *result_p = NULL, bool with_scale_and_rotation = true);

/*
  When the images have little texture this method may succed better to register both images.
  It adds edge features to both images thus increasing texture and likelyhood of success
  of the registration.
  @date: 1 sept 2005
  @author: Manuel Boissenin */
mm_image<int> registration_with_canny(mm_image<int>& inimage1, mm_image<int>& inimage2,int lwThresh = 40, int highThresh = 110, float stdDev = 1.2
				      ,reg_param *result_p = NULL, bool with_scale_and_rotation = true);

}

#endif

