 //
// cornerwise
// stumeikle Fri May 24 17:07:40 2002
//

#include "mm_edge_string.h"
#include "mm_string_el.h"
#include "mm_cgh.h"

namespace mimas {
using namespace std;

void 	mm_cgh::setup()
{
    //changed the name to differentiate it from the image setup
	init( size,size );
	
    //calculate how many edges we need at each distance
    // forumula is  n = k1 / d + k2
    // n = number of entries, d = distance, k? = constant
    //
    // we want a smooth line on the histogram at min_radius and we
    // want 1 entry per pixel at max distance. so we can calculate k1 and k2
    //
    
    //(1) take the smooth line at distance min_radius first.
    // circumf = 2* M_PI * min_rad
    // degress/pixel = 360 / circumf
    // hist bins = degrees * size / 360
    // hist bins = n = k1 / d + k2
    // k1 = size * ( min_rad + k2 ) / (2 * M_PI * min_rad)
    // 
        
/* dont need this now
    double min_circ = 2 * M_PI * min_radius;
    double s = (double) size;
    
    k2 = ( s - (2*M_PI*max_radius) );
    k2 *= min_radius;
    k2 /= (( 2*M_PI*min_radius ) - s);
    k1 = max_radius + k2;
            
    //temp? - slightly increase the number to smooth the lines a bit
    k1*=2;
*/
    
    //use fast gauss points
    initFastGauss(1.0,0.01);
}

void	mm_cgh::createFromStrings( mm_edge_string_ptr_list strings, 
                                 const Vector &cpos )
{
    //NOTE: strings should be contrast orientated before calling this function!
    //
    //go through the strings and make those entries into the histogram
    mm_edge_string_ptr_list::iterator	string_it;
    mm_string_el_ptr_list::iterator	strel_it,strel_nit;

    setup();

    for(string_it = strings.begin(); string_it!=strings.end(); string_it ++)
    {
      mm_edge_string_ptr estring( *string_it);
	
	for(strel_it = estring->begin(); strel_it != estring->end(); strel_it++)
	{
	    mm_string_el_ptr s1, s2;
	    Vector tempv( 2 );
	    double		d;

	    strel_nit = strel_it;
	    strel_nit ++;
	    	    
	    if (strel_nit == estring->end())
		break;
	    
	    s1 = (*strel_it);
	    s2 = (*strel_nit);
	    
	    tempv = ( s1->getPosition() + s2->getPosition() ) * 0.5;
	    d = norm_2( tempv - cpos );
	        
	    if (d<max_radius && d>min_radius)
	    {
		double 		n,l;
		Vector v( 2 ), deltav( 2 ), news1( 2 ), news2( 2 );
		mm_angle	theta,deltat;

		news1 = s1->getPosition() - cpos;
		news2 = s2->getPosition() - cpos;
		
		n = (max_radius/d);
		//n = number of pixels per unit length
		//but we need to know the distance between the two points too
		//as pixels @ 45' to the image plane have larger distances
		//twix them
		l = norm_2( news1 - news2 );
		n*=l;

      tempv = ( news2 - news1 ) / n;
		v = news1;
		
		theta = s1->getOrientation();
		deltat = s2->getOrientation() - s1->getOrientation();
		deltat = deltat / n; 		
		
		for(int i=0;i<(int)n;i++)
		{
		    mm_angle	phi;
	    
		    phi = mm_angle( v(1), v(0));
		    //theta is set already
		    
		    //plot the point
          IntVector p( 2 );
          p(0) = (int)((double)size * ((double)phi  +M_PI) / (2*M_PI));
          p(1) = (int)((double)size * ((double)theta+M_PI) / (2*M_PI));
		    
		    addGaussPointFast( p );
		    //set up for the next
		    theta += deltat;
		    v += tempv;
		}
	    }
	}
    }

    //finally normalise the hists, threshold the bins
    //and square root. warrafukkabbout

    //Here we need to (1) normalise (2) threshold and (3) root.
    //don't threshold before normalisation.
    double	maxval=0, total=0;

/*
    for(int j=0;j<size;++j)
    {
	for(int i=0;i<size;++i)
	{
	    double p = pixel(i,j);
	
	    if(p>maxval)
		maxval=p;
	}
    }
*/

    for(int j=size-1;j>=0;--j)
    {
	for(int i=size-1;i>=0;--i)
	{
	    double p = pixel(i,j);
	
	    if(p>maxval)
		maxval=p;
	}
    }
//--
    double	thres = maxval * bin_threshold;
    
/*
    for(int j=0;j<size;++j)
    {
	for(int i=0;i<size;++i)
	{
	    double p = pixel(i,j);
	
	    if (p<thres && p>0)
	    {
		p=0;
		pixel(i,j)=0;
	    }
		
	    total+=p;
	}
    }
*/
    for(int j=size-1;j>=0;--j)
    {
	for(int i=size-1;i>=0;--i)
	{
	    double p = pixel(i,j);
	
	    if (p<thres && p>0)
	    {
		p=0;
		pixel(i,j)=0;
	    }
		
	    total+=p;
	}
    }
//--
    
    if (total<0.001)
	return;
/*
    for(int j=0;j<size;++j)
    {
	for(int i=0;i<size;++i)
	{
	    double p = pixel(i,j);

	    p/=total;
	    p = sqrt(p);
	    setPixel(i,j,p);
	}
    }
*/
    for(int j=size-1;j>=0;--j)
    {
	for(int i=size-1;i>=0;--i)
	{
	    double p = pixel(i,j);

	    p/=total;
	    p = sqrt(p);
	    pixel(i,j) = p;
	}
    }

}




// void	mm_cgh::readFromFile( ifstream& ifs )
// {
//     //yes, we could have said stream, but then we need to virtualise the
//     //base class and blah-de-blah
//     if (!ifs.good())
//     {
// 	cout << "arese" << endl;
//     	return;
//     }
    
//     //call the image base class
//     readFromStream(ifs);
    
//     //update size var now, !@ASSUMES square images
//     size=getWidth();
// //    setup();
// }


// void	mm_cgh::writeToFile( ofstream &outfile)
// {
//     writeToStream(outfile);
// }

}
