//
// edge(l) string operations
// stumeikle Mon May  6 18:11:26 2002
//
// :-( 100% pure porting misery
//
// $Header: /cvs/mimas/lib/mm_edge_string_op.cc,v 1.13 2004/12/13 12:46:24 engjw Exp $
//

#include <iostream>
#include "mm_angle.h"
#include "mm_corner.h"
#include "mm_edge_string_op.h"
#include "mm_linalg.h"
#include "mm_string_el.h"

#define MAX_GAP 3
#define SPUR_LENGTH_THRES 5
#define TEE_ORIENTATION_THRES ( M_PI/2.0 )

using namespace boost;
using namespace std;

namespace mimas {

extern long pushedsel;

bool mm_edge_string_op::fast_mode = false;

mm_edge_string_ptr_list
mm_edge_string_op::extractFromImage( mm_property_image& im, int len_thres,
				     double up_edge_thres )
{
    mm_edge_string_ptr_list	ls;

    //(1)first find the strings
    ls = findEdgeStrings( im , len_thres, up_edge_thres);

    //(2)then remove the unconnected string elements
    //(done as part of find strings now)

    //(3)then threshold based on length and finish the hysteresis off
    //(done as part of find strings now)

    //and return
    return  ls;
}



mm_edge_string_ptr_list
mm_edge_string_op::findEdgeStrings( mm_property_image& input, int len_thres, double thres )
{
    if (fast_mode)
    	return findEdgeStrings_Fast(input, len_thres, thres);

    // first set conn field of edge type to allow all possible
    // connections

    //(0) we need to store some additional info at every pixel to
    //    handle the edge string stuff. its not really 'edgel' stuff though
    //    so we'll create a new image_property type and use that.
    int	  x,y,w=input.getWidth(),h=input.getHeight();

    mm_edgestring_image	esim;
    esim.init( w,h );

    //(1) clear up all the edgel flags associated with string creation
    for(y=0;y<h;++y)
    {
	for(x=0;x<w;++x)
	{
	    if (input.pixel(x,y))
	    {
         //ASSUMES that we've only got edgels in the input image
         mm_string_el_ptr
           msd( new mm_string_el
                ( *dynamic_pointer_cast< mm_edgel >( input.pixel(x,y) ) ) );
         esim.pixel( x, y ) = msd;
	    }
	}
    }
    
    //(2) set the connections (can't be part of the prev.)
    mm_string_el_ptr_list junc_list, term_list;
    for(y=0;y<h;++y)
    {
	for(x=0;x<w;++x)
	{
     mm_string_el_ptr msd( dynamic_pointer_cast< mm_string_el >
                           ( esim.pixel(x,y) ) );
	
	    if (msd)
	    {
		//edge_set_conns
		msd->edgeSetConns( esim, x, y, junc_list, term_list );
	    }
	}
    }
    
    //(3) prune the strings using vomit inducing algorithmic madness
    //    I've removed the first two-> short spurs will be deleted anyway...
    //	  using these leads to too sparse strings in practise. 
//    pruneSpurs(junc_list, term_list);
//    pruneTees(junc_list, term_list);
    pruneJuncs(junc_list, term_list);

    //(4) do some more processing of the junctions lists...
    removeJuncs( junc_list, term_list);
    junc_list.clear();

    //(5) bridge those gaps ...
    // but not really. moderately complex images cause this function to take
    // a very long time to complete
//    bridgeSmallGaps( term_list );

    //(6) make sure all the string parts have their heads and tails lined
    //up properly.
    mm_edge_string_ptr_list	retval;
    retval = findStrings( term_list );        
    
    //(7) reset all the string types (connected, junction, termination)
    // (done in the previous step)
    
    //(8) find any loops and add these to the list of strings

    for(y=0;y<h;++y)
    {
	for(x=0;x<w;++x)
	{
     mm_string_el_ptr el( dynamic_pointer_cast< mm_string_el >
                          ( esim.pixel(x,y) ) );
	    
	    if (!el)	continue;
	    if (el->getConnectType()==unconnected)
	    {
		if (el->getConnC1())
		{
		    //normally connected line, but not joined to any terminator 
		    //==> loop
		    mm_edge_string_ptr loop;
		    loop=findLoop( el );
		    if (loop)
			retval.push_back(loop);
		}
	    }
	}
    }

    //(8.5) threshold based on length and contrast
    thresholdStrings( retval, len_thres, thres );
    //(9) remove any bits which are still floating around/ have been thresholded
    //    also set any pixels which are connected to NULL to stop the data being
    // 	  deleted with the edgestring_image destructor.
//     for(y=0; y<h;++y)
//     {
// 	for(x=0;x<w;++x)
// 	{
// 	    mm_string_el_ptr el = (mm_string_el*)esim.getPixel(x,y);

// 	    if (!el)
// 	    	continue;
// 	    if (el->getConnectType()==unconnected)
// 	    {
// 		delete el;
// 		esim.setPixel(x,y,NULL);
// 	    }
	    
// 	    //for both cases... otherwise linked structures will be deleted with
// 	    //the image
// 	    esim.setPixel(x,y,NULL);
// 	}
//     }

    // the end. input image should be unaffected
    return retval;
}



void 	
mm_edge_string_op::pruneSpurs( mm_string_el_ptr_list& junc_list, mm_string_el_ptr_list& term_list )
{
    mm_string_el_ptr_list			short_spurs_list;
					//list of connection elements. 

    for ( mm_string_el_ptr_list::iterator jit = junc_list.begin();
          jit!=junc_list.end(); ++jit )
    {
	mm_string_el_ptr msd= (*jit);
    
	for( list< weak_ptr< mm_string_el > >::iterator cit=
          msd->getConnList().begin(); cit!=msd->getConnList().end(); ++cit )
	{
       mm_string_el_ptr spur( *cit );
	    int		    spur_len;
	    
	    spur_len = shortSpur( *spur, *msd, SPUR_LENGTH_THRES,1 );
	    if(spur_len)
	    {
		//add the spur to the short_spurs list in order from shortest to
		//longest ... but why??
		mm_string_el_ptr_list::iterator spurit;
		spur->setSpurLen( spur_len );
		if (!short_spurs_list.empty())
		{
		    for(spurit=short_spurs_list.begin(); spurit!=short_spurs_list.end(); ++spurit)
		    {
			if (spur_len < (*spurit)->getSpurLen())
			    break;
		    }
		    if (spurit==short_spurs_list.end())
			short_spurs_list.push_back(spur);
		    else
			short_spurs_list.insert( spurit, spur );
		}
		else
		    short_spurs_list.push_back( spur );
	    }
	}

	if (short_spurs_list.empty())
	    continue;

	for( mm_string_el_ptr_list::iterator cit = short_spurs_list.begin();
        cit!= short_spurs_list.end(); ++cit )
	{
       mm_string_el_ptr spur( *cit );
	    
	    list< weak_ptr< mm_string_el > >& s_conn = spur->getConnList();
	    if (s_conn.size()==1) // terminal
	    {
		spur->delConnList();
		term_list.remove( spur );
	    } else
	    {
		//>1 length spur case (?)
		//chop the spur off and re build conn list
		spur->delConnList();
		if (spur->getConnC1()==msd)
		    spur->getConnList().push_back( spur->getConnC2() );
		else
		    spur->getConnList().push_back( spur->getConnC1() );
		    
		spur->setConnC1( mm_string_el_ptr() );
		spur->setConnC2( mm_string_el_ptr() );
		term_list.push_back( spur ); //doesn't make sense to me , but...
	    }
	    
	    spur->setRemoveMe();
	}

	short_spurs_list.clear();

	// remove the remove mes (update the junction itself so that no connections
	// to spurs are stored)
   {
     list< weak_ptr< mm_string_el > >::iterator nextit;
     for( list< weak_ptr< mm_string_el > >::iterator tit =
            msd->getConnList().begin();
          tit != msd->getConnList().end(); tit = nextit ) {
       nextit = tit; nextit++;
       if ( mm_string_el_ptr( *tit )->removeMe()) {
         mm_string_el_ptr( *tit )->resetRemoveMe();
         msd->getConnList().erase( tit );
       }
     }
   }
    }

    for ( mm_string_el_ptr_list::iterator jit = junc_list.begin();
          jit!=junc_list.end(); ++jit )
    {
   mm_string_el_ptr msd( *jit );
	list< weak_ptr< mm_string_el > >::iterator i;
	list< weak_ptr< mm_string_el > > &conn_list = msd->getConnList();	
	
	//for each junction, count the number of connections again
	//and remove junctions which arent jucntions anymore, or 
	//reassign as terminators
	int count;
	if (msd->getConnC1())	count = 2;
	else			count = msd->getConnList().size();
	switch(count)
	{
	    case 0:	msd->setRemoveMe();
			msd->setConnC1( mm_string_el_ptr() );
			msd->setConnC2( mm_string_el_ptr() );
			break;
	    case 1:	term_list.push_back( msd );
			//DEBUGmay msd->setConnC1( (*conn_list.begin()) );
			msd->setConnC1( mm_string_el_ptr() );
			msd->setConnC2( mm_string_el_ptr() );
			msd->setRemoveMe();
			break;
	    case 2:
	    		i = conn_list.begin();
			msd->setConnC1((*i));		i++;
			msd->setConnC2((*i));
			conn_list.clear();
			msd->setRemoveMe();
			break;
	    default:	msd->setRemoveMe();
			msd->setConnC1( mm_string_el_ptr() );
			msd->setConnC2( mm_string_el_ptr() );
			break;
	}
    }

    //remove the remove me's again
    // remove the remove mes (update the junction itself so that no connections
    // to spurs are stored)
    {
      mm_string_el_ptr_list::iterator	nextjit;
      for ( mm_string_el_ptr_list::iterator jit = junc_list.begin();
            jit!=junc_list.end(); jit=nextjit )
        {
          nextjit = jit; 	nextjit++;
          if ((*jit)->removeMe())
            {
              (*jit)->resetRemoveMe();	
              junc_list.erase( jit );
            }
          jit = nextjit;
        }
    }
}


//should be private
int
mm_edge_string_op::shortSpur( const mm_string_el &spur, const mm_string_el &last, int thres, int count)
{
    mm_string_el_ptr next;

    if (count > thres || spur.getConnList().size()>= 2)	// another junction 
	return (0);

    if (spur.getConnList().size() == 1)		// a termination 
	return (count);

    if ( spur.getConnC1().get() == &last )
      next = spur.getConnC2();
    else
      next = spur.getConnC1();

    //aaaaaarrrgggg recursive function! bye bye stack space
    return ( shortSpur( *next, spur, thres, count + 1));
}



void
mm_edge_string_op::pruneTees(mm_string_el_ptr_list& junc_list, mm_string_el_ptr_list& term_list )
{
    mm_string_el_ptr link[3];
    mm_string_el_ptr lconn[3];
				//we need this rather than the conn list 
				//structures are slightly different than those in tina
    mm_string_el_ptr_list::iterator	jit,nextjit;
    int				i,j;

    for (jit = junc_list.begin(); jit!=junc_list.end(); ++jit)
    {
	mm_string_el_ptr junc= (*jit);
        int     c1 = 0, c2 = 0, c3;
	double   mindif = 2.0*M_PI;

	if (junc->getConnList().size() != 3)	// not a three way junction
	    continue;
	    
	list< weak_ptr< mm_string_el > >::iterator ji = junc->getConnList().begin();
	link[0]= mm_string_el_ptr( *ji ); ji++;
	link[1]= mm_string_el_ptr( *ji ); ji++;
	link[2]= mm_string_el_ptr( *ji );
	
	for(i=0;i<3;++i)
	{
	    lconn[i] = link[i];
	    if(lconn[i]->getConnList().size() !=0 ) // not a simple continuation
		break;
	}
	if (i<3)
	    continue;

	for (i = 0; i < 2; ++i)
	    for (j = i + 1; j < 3; ++j)
	    {
		mm_angle a_diff = link[i]->getEdgel()->getOrientation();
		a_diff = a_diff - link[j]->getEdgel()->getOrientation();
			    
		float   absordif = (float)fabs((double)a_diff);

		//shouldnt be necessary this time
		//absordif = (float)MIN(absordif, TWOPI - absordif);
		if (absordif < mindif)
		{
		    c1 = i;
		    c2 = j;
		    mindif = absordif;
		}
	    }
	    
	if (mindif > TEE_ORIENTATION_THRES)
	    continue;

	junc->setConnC1( link[c1] );
	junc->setConnC2( link[c2] );
	list< weak_ptr< mm_string_el > >& conn_list = junc->getConnList();
	conn_list.clear();
	
	junc->setRemoveMe();
	c3 = 3 - c1 - c2;
	conn_list = lconn[c3]->getConnList();
	//(just to be sure)
	conn_list.clear();
	if (lconn[c3]->getConnC1() == junc)
	    conn_list.push_back( lconn[c3]->getConnC2() );
	else
	    conn_list.push_back( lconn[c3]->getConnC1() );

	lconn[c3]->setConnC1( mm_string_el_ptr() );
	lconn[c3]->setConnC2( mm_string_el_ptr() );
	term_list.push_back( link[c3] );	
    }

    //remove the remove me's again
    // remove the remove mes (update the junction itself so that no connections
    // to spurs are stored)
    for (jit = junc_list.begin(); jit!=junc_list.end(); jit=nextjit)
    {
	nextjit = jit;	nextjit++;
        if ((*jit)->removeMe())
	{
	    (*jit)->resetRemoveMe();	
	    junc_list.erase( jit );
	}
	jit = nextjit;
    }

}

void
mm_edge_string_op::pruneJuncs(mm_string_el_ptr_list& junc_list, mm_string_el_ptr_list& term_list )
{
    for ( mm_string_el_ptr_list::iterator jit = junc_list.begin();
          jit!=junc_list.end(); ++jit)
    {
	mm_string_el_ptr junc= (*jit);

   {
   list< weak_ptr< mm_string_el > >::iterator nextcit;
	for( list< weak_ptr< mm_string_el > >::iterator cit =
          junc->getConnList().begin(); cit!=junc->getConnList().end();
        cit=nextcit )
	{
       mm_string_el_ptr csd( *cit );
	    nextcit = cit;
	    nextcit++;
	    
	    if (csd->getConnList().size() >= 2)	//a neighbouring junction
	    {
		//csd->setRemoveMe();
		junc->getConnList().erase(cit);
	    }		
	    
	}
   }
   }
    
    for ( mm_string_el_ptr_list::iterator jit = junc_list.begin();
          jit!=junc_list.end(); ++jit)
    {
   mm_string_el_ptr junc( (*jit) );
   list< weak_ptr< mm_string_el > >::iterator i;
	list< weak_ptr< mm_string_el > >&  conn_list = junc->getConnList();
    
	int count;
	count = conn_list.size();
	switch(count)
	{
	    case 0:	junc->setConnC1( mm_string_el_ptr() );
			junc->setConnC2( mm_string_el_ptr() );
			junc->setRemoveMe();
			break;
	    case 1: 	term_list.push_back( junc );
			junc->setConnC1( mm_string_el_ptr() );
			junc->setConnC2( mm_string_el_ptr() );
			junc->setRemoveMe();
			break;
	    case 2:	
			i = conn_list.begin();
			junc->setConnC1((*i));		i++;
			junc->setConnC2((*i));
			conn_list.clear();
			junc->setRemoveMe();
			break;
	    default:	
			junc->setConnC1( mm_string_el_ptr() );
			junc->setConnC2( mm_string_el_ptr() );
			break;
	}
    }

    //remove the remove me's again
    // remove the remove mes (update the junction itself so that no connections
    // to spurs are stored)
    {
    mm_string_el_ptr_list::iterator nextjit;
    for ( mm_string_el_ptr_list::iterator	jit = junc_list.begin();
          jit!=junc_list.end(); jit=nextjit)
    {
	nextjit = jit; nextjit++;
        if ((*jit)->removeMe())
	{
	    (*jit)->resetRemoveMe();
	    junc_list.erase( jit );
	}
	jit = nextjit;
    }
    }
}


void
mm_edge_string_op::removeJuncs( mm_string_el_ptr_list& junc_list,
				mm_string_el_ptr_list& term_list )
{

    for ( mm_string_el_ptr_list::iterator jit = junc_list.begin();
          jit!=junc_list.end(); ++jit )
    {
   mm_string_el_ptr junc( *jit );

	for( list< weak_ptr< mm_string_el > >::iterator cit =
          junc->getConnList().begin(); cit!=junc->getConnList().end(); ++cit)
	{
	    mm_string_el_ptr csd( *cit );
	    list< weak_ptr< mm_string_el > > &nconn = csd->getConnList();

	    if (!csd)
		continue;
	    
	    int count;
	    if (csd->getConnC1())	count = 2;
	    else			count = csd->getConnList().size();

	    switch(count)
	    {
       case 1: nconn.clear();
			term_list.remove( csd );
			csd->setConnC1( mm_string_el_ptr() );
			csd->setConnC2( mm_string_el_ptr() );
			//junc->getConnList().remove(csd); //debug
			//term_list.push_back(junc); //debug
			break;
    		case 2:
		{
		    mm_string_el_ptr keep;
		
		    if (csd->getConnC1() == junc)
			keep = csd->getConnC2();
		    else
			keep = csd->getConnC1();
		    
		    nconn.clear();
		    nconn.push_back(keep);
		    csd->setConnC1( mm_string_el_ptr() );
		    csd->setConnC2( mm_string_el_ptr() );
		    term_list.push_back( csd );
		    break;
		}
	    default:
		break;		/* junctions take care of themselves */
	    }
	}
	
	junc->getConnList().clear();
	junc->setConnC1( mm_string_el_ptr() );
	junc->setConnC2( mm_string_el_ptr() );
    }
}


void
mm_edge_string_op::bridgeSmallGaps( mm_string_el_ptr_list& term_list )
{
    //calculate the distance between all terminators and join those
    //which are close(!)
    mm_string_el_ptr_list::iterator	lmin;
    mm_string_el_ptr term, term2;
    list< weak_ptr< mm_string_el > > *conn1, *conn2;
    double 				min_diff, diff;

    Vector v1(2),v2(2),tv1(2),tv2(2),v(2);

    min_diff = MAX_GAP;

    for ( mm_string_el_ptr_list::iterator tit = term_list.begin();
          tit!=term_list.end(); ++tit )
    {
	term= (*tit);

	if (term->removeMe())
	    continue;	
	
	conn1 = &term->getConnList();
	tv1 = term->getEdgel()->getPosition();
	tv2 = mm_string_el_ptr( *conn1->begin() )->getEdgel()->getPosition();
   // 'v2 = unit( tv1 - tv2 )' won't work in release-mode for some reason.
	v1  = tv1 - tv2;
   v1  = unit( v1 );
	// v1  = tv1 - tv2;
	// v1.unit(); This was wrong (no effect) !!!
	
	lmin = NULL;
	
	mm_string_el_ptr_list::iterator tit2 = tit;
	tit2++;
	for(; tit2!= term_list.end(); ++tit2)
	{
	    term2= (*tit2);

	    if (term2->removeMe())
		continue;	
	    
	    tv1 = term->getEdgel()->getPosition();
	    v = term2->getEdgel()->getPosition() - tv1;
	    diff = norm_2( v );

	    if (diff>min_diff)
		continue;
		
	    // v.unit(); No effect !!!
       v = unit( v );
	    if ( inner_prod( v, v1 ) < 0.9 )
		continue;

	    conn2 = &term2->getConnList();
	    tv1 = term2->getEdgel()->getPosition();
	    tv2 = mm_string_el_ptr( *conn2->begin() )->getEdgel()->getPosition();
       // 'v2 = unit( tv1 - tv2 )' won't work in release-mode for some reason.
	    v2 = tv1 - tv2;
       v2 = unit( v2 );
	    // v2  = tv1 - tv2;
       // v2.unit(): No effect !!!

	    if ( inner_prod( v2, v ) > -0.9 )
		continue;
	    
	    lmin = tit2;
	    min_diff = diff;
	}
	
	if (lmin == NULL)
	    continue;
	    
	term2 = (*lmin);
	conn2 = &term2->getConnList();
	term->setConnC1( *conn1->begin() );
	term2->setConnC1( *conn2->begin() );
	term->setConnC2( term2 );
	term2->setConnC2( term );
	
	conn1->clear();
	conn2->clear();
	
	(*lmin)->setRemoveMe();
	term->setRemoveMe();
    }

    //remove all the remove mes
    mm_string_el_ptr_list::iterator tit2;
    for ( mm_string_el_ptr_list::iterator tit = term_list.begin();
          tit!=term_list.end(); tit=tit2 )
    {
      tit2 = tit; tit2++;
        if ((*tit)->removeMe())
	{
	    (*tit)->resetRemoveMe();
		term_list.erase( tit );
	}
	tit = tit2;
    }
}



mm_edge_string_ptr_list
mm_edge_string_op::findStrings( mm_string_el_ptr_list& term_list )
{
    //create a new nice and tidy list of strings from the list of
    //terminations. must set the connect type here to stop the
    // string_el's being deleted along with the image
    mm_edge_string_ptr_list		retval;

    mm_string_el_ptr_list::iterator	tit;

    //QUESTION: why does this not produce 2* the real number of strings???
    for(tit=term_list.begin();tit!=term_list.end();++tit)
    {
	//follow the elements and create a new string
	mm_edge_string_ptr s( findEdgeString( *tit ) );

	//if the terminator is the end of a string we've used already then
	//s will be null.
	if (s)
	    retval.push_back(s);
    }

    return retval;
}



mm_edge_string_ptr
mm_edge_string_op::findEdgeString( mm_string_el_ptr el  )
{
    mm_edge_string_ptr s( new mm_edge_string );
    mm_string_el_ptr eptr, from;
    
    if (el->getConnList().size()!=1)
    {
      // delete s;
	return mm_edge_string_ptr();
    }

    if (el->getConnectType()!=unconnected)
    {
      // delete s;
	return mm_edge_string_ptr();
    }

    //do the first one
    eptr = el;
    eptr->setConnectType( termination );
    eptr->setLoop( false );
    s->push_back(eptr);

    from = eptr;
    eptr = mm_string_el_ptr( *(eptr->getConnList().begin()) );
    
    //do the middle
    for( ;; eptr=eptr->getConnC1() )
    {
	if (eptr==NULL)
	{
	    break;
	}
	
	// if we're an endpoint which has already been processed, then return null
	if (eptr->getConnectType()!=unconnected)
	{
	    // first we need to reset things we've set already
	    mm_edge_string::iterator	i;
	    for(i=s->begin();i!=s->end();++i)
	    {
		(*i)->setConnectType(unconnected);
	    }
	    s->clear();
	    // delete s;
	    return mm_edge_string_ptr(); }
    
	eptr->setLoop( false );
	switch(eptr->getConnList().size())
	{
	    case 1: eptr->setConnectType( termination );
		    break;
	    case 0: eptr->setConnectType( connection );
		    break;
	    default:eptr->setConnectType( junction );
		    break;
	}

	if (eptr->getConnC2() != from )
	{
       mm_string_el_ptr temp( eptr->getConnC2() );
	    eptr->setConnC2( eptr->getConnC1() );
	    eptr->setConnC1( temp );
	}

	s->push_back(eptr);	

	if (eptr->getConnList().size()!=0)
	    break;
	from = eptr;
    }
    
    return s;
}


mm_edge_string_ptr
mm_edge_string_op::findLoop( mm_string_el_ptr el )
{
  if (!el)	return mm_edge_string_ptr();
    
    //just follow the loop and set the loop and conn type on
    //the way. 
    mm_edge_string_ptr retval;
    mm_string_el_ptr from, first;
    
    if (!el->getConnC1())
      return mm_edge_string_ptr();
    
    
    first = el;
    from = el;
    
    el = el->getConnC1();
    do
    {
    	if (el->getConnC2() != from )
	{
       mm_string_el_ptr temp( el->getConnC2() );
	    el->setConnC2( el->getConnC1() );
	    el->setConnC1( temp );
	}

	from = el;
	el=el->getConnC1();
    } while (el && !(el==first));

    if (el!=first)
      return mm_edge_string_ptr();
	
    //else
    el=first;
    retval = mm_edge_string_ptr( new mm_edge_string );
    do
    {
    	el->setLoop(true);
	el->setConnectType( connection );
	retval->push_back(el);
	el=el->getConnC1();
    }while (el && !(el==first));
    

    return retval;
}

void
mm_edge_string_op::thresholdStrings( mm_edge_string_ptr_list& strings, int lt, double t )
{
    //remove edges which are shorter than length_thres, or which have contrast
    //values which never rise above the given threshold value
    mm_edge_string_ptr_list::iterator	sit,nextsit;
    
    for(sit=strings.begin();sit!=strings.end();sit=nextsit)
    {
	nextsit =sit;
	nextsit++;
	mm_edge_string_ptr s( *sit );
	
	if (thresholdString( *s, lt, t )==false)
	{
	    //set all the elements to 'unconnected' and remove from the list of strings
	    mm_edge_string::iterator	stringit;
	    for(stringit=s->begin();stringit!=s->end();++stringit)
		(*stringit)->setConnectType(unconnected);
	
	    // s->clear();
	    // delete s;
	    strings.erase( sit );
	}
    }
}


bool
mm_edge_string_op::thresholdString( const mm_edge_string& s, int len, double t )
{
    int				l = 0;
    
    for( mm_edge_string::const_iterator stringit=s.begin(); stringit!=s.end();
         ++stringit)
    {
	mm_string_el_ptr el = (*stringit);
	
	if (el->getEdgel()->getContrast() > t && l>len)
	{
	    return true;
	}
	    
	l++;
    }

    return false;
}


void	mm_edge_string_op::findContrastMaxima(mm_edge_string &s,  mm_corner_image& cim )
    {
	//skip along the string and look for turning points in the contrast
	//mark these as corners on the image
	mm_string_el_ptr_list::iterator	i,endm1,ti, lastup,j;
	
	//smooth the string first
	endm1 = s.end();	endm1--;
for(int z=0;z<5;z++)
{
	list<double>		smoothed;
	double 			last = 0;
	for(i=s.begin();i!=endm1;++i)
	{
	    mm_string_el_ptr here, there;
	    double		dc;

	    ti = i;
	    here=(*ti);	ti++;
	    there=(*ti);
	    
	    dc = here->getContrast()*2 + there->getContrast() + last;
	    dc/=4;
	    
	    last = here->getContrast();
	    smoothed.push_back( dc );
	}
	list<double>::iterator	di;
	for(i=s.begin(), di=smoothed.begin();i!=endm1;++i,++di)
	{
       mm_string_el_ptr here( *i );
	    
	    here->setContrast((*di));
	}
}
	//end smooth
	
	
	lastup = s.end();
	for( i=s.begin(); i!=endm1; ++i )
	{
	    mm_string_el_ptr here, there;
	    double		dc;
	    
	    ti = i;
	    here=(*ti);	ti++;
	    there=(*ti);
	    
	    dc = there->getContrast() - here->getContrast();
	    if (dc>-0.5 && dc<0.5)
		dc=0;
	    
	    if (dc<0.0 && lastup!=s.end())
	    {
		//going down (again)
		//find the turn and mark it
		int	count=0,x,y;
		for(j=lastup;j!=i;j++, count++);
		count /=2;
		for(j=lastup;count>=0;++j, --count);
		
		here = (*j);
		
		x = (int) here->getPosition()(0);
		y = (int) here->getPosition()(1);
		if (cim.getPixel(x,y))
		    continue;
		    // one per pixel!
		
		mm_corner_ptr c( new mm_corner );
		c->setContrast( here->getContrast() );
		c->setPosition( here->getPosition() );
		c->setOrientation( here->getOrientation() );
		cim.setPixel(x,y,c);
		
		//reset lastup
		lastup=s.end();
	    }
	    if (dc>0.0)
	    {
		//going up... update lastup
		lastup = i;
	    }
	}
    }


void	mm_edge_string_op::filterWiggles( mm_edge_string_ptr_list& edge_strings,
					  double wiggle_thres, double len_thres, double contrast_thres )
{
    // A simple algorithm by stumeikle Wed May 22 18:39:45 2002

    // remove any edge strings which are:
    // (1) short
    // (2) which wiggle about and don't go anywhere
    // (3) ... unless they have high contrast    
    mm_edge_string_ptr_list::iterator	sit,nextsit;

    for(sit=edge_strings.begin();sit!=edge_strings.end();sit=nextsit)
    {
	mm_edge_string::iterator	esi;
	
	nextsit = sit;
	nextsit++;
	
	// filter
	double 		max_contrast = 0;
	int    		len=0;
	double 		total_wiggle=0, angle;
	mm_string_el_ptr startel;
	mm_angle	last;
	
        startel = (*(*sit)->begin());
        last = startel->getOrientation();

        for(esi=(*sit)->begin();esi!=(*sit)->end();esi++, len++)
        {
	    mm_string_el_ptr sel = (*esi);
		
	    if (sel->getEdgel()->getContrast() > max_contrast)
	        max_contrast = sel->getEdgel()->getContrast();
		
	    mm_angle a = sel->getOrientation();
	    a = a- last;
	    angle = (double)a;
	    if (angle<0)	angle*=-1;
		
	    total_wiggle+=angle;
		
	    last = sel->getOrientation();
	}
	    
	total_wiggle /= (double) len;
	//drop the string if its too curly or short and also doesn't have big contrast
	if (  total_wiggle> wiggle_thres || len<len_thres) 
	{
	    //if contrast high then let it through?
	    if (max_contrast>contrast_thres)
	        ;	//ok
	    else	//nok remove string
	    {
		edge_strings.erase(sit);
		// delete (*sit);
	    }
	}
    }
}



//------------------ the fast routines. a hack by stu -----------------------------------
mm_edge_string_ptr_list
mm_edge_string_op::findEdgeStrings_Fast( mm_property_image& input, int len_thres, double thres )
{
    int	  x,y,w=input.getWidth(),h=input.getHeight();

    //new algorithm by stumeikle 20030304
    int count;

    //(1) remove edgles which are junctions
    for(y=1;y<h-1;++y)
    {
    	for(x=1;x<w-1;++x)
	{
	   count=0;
	   if (input.pixel(x-1,y)!=0)   count++;
	   if (input.pixel(x+1,y)!=0)   count++;
	   if (input.pixel(x,y-1)!=0)   count++;
	   if (input.pixel(x,y+1)!=0)   count++;
	   if (input.pixel(x-1,y-1)!=0) count++;
	   if (input.pixel(x+1,y-1)!=0) count++;
	   if (input.pixel(x-1,y+1)!=0) count++;
	   if (input.pixel(x+1,y+1)!=0) count++;

	   if (count>2)
	   {
	   	if (input.pixel(x,y))
		{
          // mm_prop	* p = input.getPixel(x,y);
          // delete p;

        input.pixel(x,y).reset();
        // input.pixel(x,y) = mm_prop_ptr();
		}
	   }
	}
    }

//input.display();

    //(2) skip through image until
    // (a) we find an isolated edgel --> just delete it.
    //
    // (b) we find an edgel with 2 neighbours
    //
    //push the initial edgel onto the list and remove from image
    //take the first of the two. push it onto the end. and remove from image
    //follow first end to the end.
    //then take the other edgel , push onto the front of the list and remove
    //and follow it.
    //if the min line length has been exceeded and if the upper threshold
    //was crossed keep the line, else dispose.
    //repeat until the image is empty.
    mm_edge_string_ptr_list	retval;

    for(y=0;y<h;++y)
    {
        for(x=0;x<w;++x)
	{
       mm_edgel_ptr e( dynamic_pointer_cast< mm_edgel >
                       ( input.pixel( x, y ) ) );

	    if (!e) continue;

	    //count the number of neighbours. this should be only 0,1 or 2 now.
	    count=0;
	    if (input.getPixel(x-1,y)!=0)   count++;
	    if (input.getPixel(x+1,y)!=0)   count++;
	    if (input.getPixel(x,y-1)!=0)   count++;
	    if (input.getPixel(x,y+1)!=0)   count++;
	    if (input.getPixel(x-1,y-1)!=0) count++;
	    if (input.getPixel(x+1,y-1)!=0) count++;
	    if (input.getPixel(x-1,y+1)!=0) count++;
	    if (input.getPixel(x+1,y+1)!=0) count++;

	    if (count==0)
	    {
	    	// a lone edgel. just remove it and continue
	    	// delete e;
         // input.setPixel(x,y, mm_edgel_ptr() );
         input.pixel( x, y ).reset();
		continue;
	    } else if (count==1 || count==2)
	    {
      mm_edge_string_ptr s( getString_Fast(input, len_thres, thres,x,y) );
		if (s)
			retval.push_back(s);
	    } else if (count>2)
	    {
	        cout << "ERROR!" << endl;
		continue;
	    }
	}
    }

    return retval;
 }


mm_edge_string_ptr mm_edge_string_op::getString_Fast(mm_property_image& input, int len_thres, double thres,
					      int startx, int starty )
{
    //find the string which has a point at x by y.
    mm_edge_string_ptr es( new mm_edge_string );

    //push the current edgel and clear from the image
    mm_string_el_ptr sl( new mm_string_el
                         ( *dynamic_pointer_cast< mm_edgel >
                           ( input.getPixel( startx,starty ) ) ) );
    es->push_back( sl);
    // delete input.getPixel(startx,starty);
    input.setPixel( startx, starty, mm_edgel_ptr() );

    //do the first half of the string
    long	length1=0,length2=0;
    bool	crossed_thres1=false, crossed_thres2=false;

    //find the first part of the line and process
    //and then the second if it exists
    int ix,iy;
    bool first=true;
    for(iy=-1;iy<2;++iy)
    	for(ix=-1;ix<2;++ix)
	{
	    if (first && input.getPixel(startx+ix, starty+iy))
	    {
	    	firstHalf_Fast( input, thres, startx+ix,starty+iy, *es,
                         &crossed_thres1, &length1 );
         first=false;
         continue;
	    }
	    if (!first && input.getPixel(startx+ix, starty+iy))
	    {
         secondHalf_Fast( input, thres, startx+ix,starty+iy, *es,
                          &crossed_thres2, &length2 );
         break;
	    }
	}

    //return the result.
    if (length1+length2+1 >= len_thres && (crossed_thres1 || crossed_thres2))
    {
        //ok.
	return es;
    }
    else
    {
    	//not ok. delete all the entries in the string and then delete teh string
	//itself
// 	mm_edge_string::iterator	i;
// 	for(i=es->begin();i!=es->end();++i)
// 		delete(*i);

//    es->clear();
// 	delete es;
      return mm_edge_string_ptr();
    }
}


void  mm_edge_string_op::firstHalf_Fast(mm_property_image& input, double thres,
				     int startx, int starty, mm_edge_string& es,
				     bool* crossed_thres, long *len )
{
    //recursive
    mm_string_el_ptr sl( new mm_string_el
                         ( *dynamic_pointer_cast< mm_edgel >
                           ( input.getPixel( startx,starty ) ) ) );
    es.push_back( sl );
    // delete input.getPixel(startx,starty);
    input.setPixel( startx, starty, mm_edgel_ptr() );

    *len = *len +1;
    if (sl->getStrength()>thres)
    	*crossed_thres=true;

    //find the next pixel
    for(int iy=-1;iy<2;++iy)
    	for(int ix=-1;ix<2;++ix)
	{
	    if (input.getPixel(startx+ix,starty+iy))
	    {
	    	firstHalf_Fast(input, thres, startx+ix,starty+iy, es, crossed_thres, len);
	    }
	}
}

void  mm_edge_string_op::secondHalf_Fast(mm_property_image& input, double thres,
				     int startx, int starty, mm_edge_string& es,
				     bool* crossed_thres, long *len )
{
    //recursive
  mm_string_el_ptr sl
    ( new mm_string_el( *dynamic_pointer_cast< mm_edgel >
                        ( input.getPixel(startx,starty) ) ) );
    es.push_front( sl);
    // delete input.getPixel(startx,starty);
    input.setPixel(startx,starty, mm_edgel_ptr() );

    *len = *len +1;
    if (sl->getStrength()>thres)
    	*crossed_thres=true;

    //find the next pixel
    for(int iy=-1;iy<2;++iy)
    	for(int ix=-1;ix<2;++ix)
	{
	    if (input.getPixel(startx+ix,starty+iy))
	    {
	    	secondHalf_Fast(input, thres, startx+ix,starty+iy, es, crossed_thres, len);
	    }
	}
}

}

