#include "mm_gnuplot.h"

using namespace std;

namespace mimas {

mm_gnuplot& mm_gnuplot::operator<<( const string &s )
{
	command(s);
	command("\n");
	return *this;
}


mm_gnuplot::mm_gnuplot(void)
{
	fp=popen("gnuplot","w");
	// linetitlesSet=false;
}

mm_gnuplot::~mm_gnuplot(void)
{
	unsetLineTitle();
	if (fp) pclose(fp);
}

void mm_gnuplot::close(void)
{
	if (fp) pclose(fp);
	fp=NULL;
}

void mm_gnuplot::open(void)
{
	if (fp) return;
	fp=popen("gnuplot","w");

}

void mm_gnuplot::checkOpen(void)
{
	open();
}


void mm_gnuplot::setMultiPlot(bool multiplot)
{
	checkOpen();
	if (multiplot)
		command("set multiplot\n");
	else
		command("set nomultiplot\n");
}

void mm_gnuplot::command(const char *c)
{
	checkOpen();
	fputs(c,fp);
	fflush(fp);
}

void mm_gnuplot::command(const string &s)
{
	command(s.c_str());
}

void mm_gnuplot::unsetLineTitle(void)
{
  lineTitles.clear();
  // if (linetitlesSet)
  //	{
  //		delete[] linetitles;
  //		linetitlesSet=false;
  //	}
}

void mm_gnuplot::setLineTitle(const char *title, ...)
{
	unsetLineTitle();
	
	va_list args;
	
	va_start(args,title);

	// precount parameters
	int paramcount=-1;
	const char *val="dummy";
	while(val!=NULL)
	{
		paramcount++;
		val=va_arg(args,const char *);
	}
	paramcount++;

	// linetitles = new string[paramcount];
	lineTitles.push_back( title );

	va_end(args);


	va_start(args,title);

	for (int i=1; i<paramcount; i++) // start from 1 since first parameter already loaded
	{
		val=va_arg(args,const char *);
		lineTitles.push_back( val );
	} 

	va_end(args);

	// linetitlesSet=true;

}

void mm_gnuplot::startSave(const char *fn, const char *format)
{
	char buf[65535];
	
	if (strcmp(format,"PS")==0)
		command("set term post enh 14\n");
	else if (strcmp(format,"EPS")==0)
		command("set term post eps enh 14\n");
	else if (strcmp(format,"PSCOLOR")==0)
		command("set term post eps enh color 14\n");
	else if (strcmp(format,"EPSCOLOR")==0)
		command("set term post eps enh color 14\n");
	else if (strcmp(format,"PNG")==0)
		command("set term png small color\n");
	else if (strcmp(format,"JPG")==0)
		command("set term jpeg large size 1024,768\n");
	else
	{
#ifndef NDEBUG
      cerr << "mm_gnuplot::save - cannot save, unknown file format. "
           << "Recognised formats are PS, EPS, PSCOLOR, EPSCOLOR, PNG, JPG"
           << endl;
#endif
		return;
	}

	sprintf(buf,"set out \"%s\"\n", fn);
	command(buf);
	//command("replot\n");
	
}


void mm_gnuplot::endSave(void)
{
	command("set term x11\n"); // reset terminal to x11
}

}
