#include <boost/multi_array.hpp>
#include <qimage.h>
#include "mm_image_fileinput.h"
#include "mm_rgba.h"

using namespace boost;
using namespace std;

namespace mimas {

template<>
void mm_image_fileinput< unsigned char >::read
  ( mm_image< unsigned char > &image ) throw (mm_exception)
{
  MMERROR( state, mm_exception, ,
	   "Attempt to read a second image from same stream using "
	   "mm_image_fileinput." );

  streampos pos = stream->tellg();
  stream->seekg( 0, ios::end );
  int filesize = stream->tellg() - pos;
  stream->seekg( pos, ios::beg );

  QByteArray data( filesize );
  stream->read( data.data(), filesize );

  QImage img( data );
  MMERROR( !img.isNull(), mm_exception, , "Error reading image from stream." );

  if ( img.depth() == 8 && img.allGray() ) {
    image.init( img.width(), img.height() );
    // Line-wise copying required, as QImage does 32 bit memory alignment!
    // Direct access to image-memory!
    for ( int y=0; y<img.height(); y++ )
      copy_n( (const unsigned char *)img.scanLine( y ), img.width(),
              image.rawData()[y].begin() );
  } else //if the image from the file is a colour image
    {
    stream->seekg( pos, ios::beg );
    mm_image_fileinput< mm_rgba< unsigned char > > input( *stream );
    mm_image< mm_rgba< unsigned char > > temp;
    input.read( temp );
    image = mm_image< unsigned char >( temp );
  };

  state = false;
}

template<>
void mm_image_fileinput< mm_rgba< unsigned char > >::read
  ( mm_image< mm_rgba< unsigned char > > &image ) throw (mm_exception)
{
  MMERROR( state, mm_exception, ,
	   "Attempt to read a second image from same stream using "
	   "mm_image_fileinput." );

  streampos pos = stream->tellg();
  stream->seekg( 0, ios::end );
  int filesize = stream->tellg() - pos;
  stream->seekg( 0, ios::beg );

  QByteArray data( filesize );
  stream->read( data.data(), filesize );

  QImage img( data );
  MMERROR( !img.isNull(), mm_exception, , "Error reading image from stream." );

  if ( img.depth() != 32 )
    img = img.convertDepth( 32 );
  // img = img.swapRGB();
  image.init( img.width(), img.height() );
  assert( sizeof( mm_rgba< unsigned char > ) == 4 );
  // Line-wise copying required, as QImage does 32 bit memory alignment!
  // Direct access to image-memory!
  for ( int y=0; y<image.getHeight(); y++ )
    copy_n( (const mm_rgba< unsigned char > *)img.scanLine( y ), img.width(),
            image.rawData()[y].begin() );

  state = false;
};

};
