#include "mm_image_gstreamerinput.h"
#include "mm_rgba.h"

#include <iostream>

using namespace std;

namespace mimas {

template<>
void mm_image_gstreamerinput< mm_rgba< unsigned char > >::read
   ( mm_image< mm_rgba< unsigned char > > &image ) throw (mm_exception)
{
  MMERROR( state, mm_exception, ,
	   "Attempt to read image from mm_image_gstreamerinput, after "
	   "gstreamer-pipeline has terminated." );

  // Decrease read semaphore (blocking).
  struct sembuf buf = { 0, -1, SEM_UNDO };
  if ( semop( semid, &buf, 1 ) == 0 ) {

    // Retrieve size of image.
    const int *size = (int *)shmem;
    image.init( size[0], size[1] );
    
    // Copy data from shared memory and convert from RGB to RGBA.
    // Direct access to image-memory!
    assert( sizeof( mm_rgba< unsigned char > ) == 4 );
    unsigned char *p = (unsigned char *)image.rawData().data();
    const unsigned char *q = (unsigned char *)shmem + 8;
    int numpix = image.rawData().num_elements();
    for ( int i=0; i<numpix; i++ ) {
      *p++ = *q++;
      *p++ = *q++;
      *p++ = *q++;
      *p++ = 0;
    };
    
    // Increase write semaphore.
    struct sembuf buf = { 1, +1, 0 };
    MMERROR( semop( semid, &buf, 1 ) == 0, mm_exception, ,
             "Error switching semaphores with key " << semkey << "." );
  } else {
    state = false;
  };
}

template<>
void mm_image_gstreamerinput< unsigned char >::read
   ( mm_image< unsigned char > &image ) throw (mm_exception)
{
  MMERROR( state, mm_exception, ,
	   "Attempt to read image from mm_image_gstreamerinput, after "
	   "gstreamer-pipeline has terminated." );

  // Decrease read semaphore (blocking).
  struct sembuf buf = { 0, -1, SEM_UNDO };
  if ( semop( semid, &buf, 1 ) == 0 ) {

    // Retrieve size of image.
    const int *size = (int *)shmem;
    mm_image< mm_rgba< unsigned char > > colorImage;
    colorImage.init( size[0], size[1] );
    
    // Copy data from shared memory and convert from RGB to RGBA.
    // Direct access to image-memory!
    assert( sizeof( mm_rgba< unsigned char > ) == 4 );
    unsigned char *p = (unsigned char *)colorImage.rawData().data();
    const unsigned char *q = (unsigned char *)shmem + 8;
    int numpix = colorImage.rawData().num_elements();
    for ( int i=0; i<numpix; i++ ) {
      *p++ = *q++;
      *p++ = *q++;
      *p++ = *q++;
      *p++ = 0;
    };

    image = mm_image< unsigned char >( colorImage );
    
    // Increase write semaphore.
    struct sembuf buf = { 1, +1, 0 };
    MMERROR( semop( semid, &buf, 1 ) == 0, mm_exception, ,
             "Error switching semaphores with key " << semkey << "." );
  } else {
    state = false;
  };
} 

};
