#include <boost/shared_array.hpp>
#include "mm_image_mplayeroutput.h"

using namespace boost;

namespace mimas {

#define rgbtoyuv(r, g, b, y, u, v) \
y=(unsigned char)(((int)30*r  +(int)59*g +(int)11*b)/100); \
	u=(unsigned char)(((int)-17*r  -(int)33*g +(int)50*b+12800)/100); \
	v=(unsigned char)(((int)50*r  -(int)42*g -(int)8*b+12800)/100); 

// The following function is from vconvert which is Mozilla MPL licensed. 
// The license looks quite liberal. Can anyone check?
static void rgba_to_yuv420p( const void *in, int width, int height, void *out )
{
  // Direct access to image-memory!
   const unsigned int rgbIncrement=4;

	const unsigned int planeSize = width*height;
	const unsigned int halfWidth = width >> 1;

	// get pointers to the data
	unsigned char * yplane  = (unsigned char *)out;
	unsigned char * uplane  = (unsigned char *)out + planeSize;
	unsigned char * vplane  = (unsigned char *)out + planeSize + (planeSize >> 2);
	const unsigned char * rgbIndex = (unsigned char *)in;

	for (int y = 0; y < height; y++) {
     unsigned char * yline  = yplane + (y * width);
     unsigned char * uline  = uplane + ((y >> 1) * halfWidth);
     unsigned char * vline  = vplane + ((y >> 1) * halfWidth);

		for (int x = 0; x < width; x+=2) {
			rgbtoyuv(rgbIndex[2], rgbIndex[1], rgbIndex[0],*yline, *uline, *vline);
			rgbIndex += rgbIncrement;
			yline++;
			rgbtoyuv(rgbIndex[2], rgbIndex[1], rgbIndex[0],*yline, *uline, *vline);
			rgbIndex += rgbIncrement;
			yline++;
			uline++;
			vline++;
		}
	}
}

template<>
void mm_image_mplayeroutput< mm_rgba< unsigned char > >::write
  ( const mm_image< mm_rgba< unsigned char > > &image )
{
  MMERROR( image.getWidth() == width && image.getHeight() == height,
           mm_exception, ,
           "mm_image_mplayeroutput< mm_rgba< unsigned char > >::write() - "
           "Image must be of size " << width << 'x' << height << ", but the "
           "size was " << image.getWidth() << 'x' << image.getHeight()
           << '!' );

  const int yuvSize = width * height * 3 / 2;
  shared_array< unsigned char > yuv( new unsigned char[ yuvSize ] );
  rgba_to_yuv420p( image.rawData().data(), width, height, yuv.get() );
  ::write( imgpipe, yuv.get(), yuvSize );
}

template<>
void mm_image_mplayeroutput< unsigned char >::write
  ( const mm_image< unsigned char > &image )
{
  MMERROR( image.getWidth() == width && image.getHeight() == height,
           mm_exception, ,
           "mm_image_mplayeroutput< mm_rgba< unsigned char > >::write() - "
           "Image must be of size " << width << 'x' << height << ", but the "
           "size was " << image.getWidth() << 'x' << image.getHeight()
           << '!' );

  mm_image< mm_rgba< unsigned char > > colourImage( image );

  const int yuvSize = width * height * 3 / 2;
  shared_array< unsigned char > yuv( new unsigned char[ yuvSize ] );
  rgba_to_yuv420p( colourImage.rawData().data(), width, height, yuv.get() );
  ::write( imgpipe, yuv.get(), yuvSize );
}

};
