#include <boost/multi_array.hpp>
#include <qapplication.h>
#include <qimage.h>
#include <qpixmap.h>
#include "mm_image_qtoutput.h"

using namespace boost;

namespace mimas {

void quitQtApplication( QApplication *app )
{
  app->quit();
}

static QPixmap mm_image_to_qpixmap
 ( const mm_image< unsigned char > &image )
{
  QImage img( image.getWidth(), image.getHeight(), 8, 256 );
  for ( int i=0; i<256; i++ )
    img.setColor( i, qRgb( i, i, i ) );

  // Line-wise copying required, as QImage does 32 bit memory alignment!
  // Direct access to image-memory!
  for ( int y=0; y<image.getHeight(); y++ )
    copy_n( image.rawData()[y].begin(), image.getWidth(), img.scanLine( y ) );

  QPixmap retVal;
  retVal.convertFromImage(img);
  return retVal;
}

static QPixmap mm_image_to_qpixmap
 ( const mm_image< mm_rgba< unsigned char > > &image )
{
  QImage img( image.getWidth(), image.getHeight(), 32 );
  assert( sizeof( mm_rgba< unsigned char > ) == 4 );

  // Line-wise copying required, as QImage does 32 bit memory alignment!
  // Direct access to image-memory!
  for ( int y=0; y<image.getHeight(); y++ )
    copy_n( image.rawData()[y].begin(), image.getWidth(),
            (mm_rgba< unsigned char > *)img.scanLine( y ) );
  
  QPixmap retVal;
  // Direct access to image-memory!
  // retVal.convertFromImage( img.swapRGB() );
  retVal.convertFromImage( img );
  return retVal;
}

template<>
void mm_image_qtoutput< unsigned char >::write
   ( const mm_image< unsigned char > &image )
{
  MMERROR( state, mm_exception, ,
	   "Attempt to display an image with mm_image_qtoutput, after user "
	   "has requested termination by closing the window." );

  writePixmap( mm_image_to_qpixmap( image ) );
}

template<>
void mm_image_qtoutput< mm_rgba< unsigned char > >::write
   ( const mm_image< mm_rgba< unsigned char > > &image )
{
  MMERROR( state, mm_exception, ,
	   "Attempt to display an image with mm_image_qtoutput, after user "
	   "has requested termination by closing the window." );

  writePixmap( QPixmap( mm_image_to_qpixmap( image ) ) );
}

};
