#include <GL/glut.h>
#include "mm_mesa_base.h"

using namespace std;

namespace mimas {

bool mm_mesa_base::existInstance = false;

mm_mesa_base::mm_mesa_base( mm_x11_display_ptr _display,
                            XVisualInfo *_visualInfo )
  throw (mm_exception):
  display( _display ), visualInfo( _visualInfo ), finishBase( false )
{
  if ( existInstance ) {
    MMERROR( false, mm_exception, ,
             "There must be only one instance of mm_mesa_base at any time." );
  };
  existInstance = true;
}

mm_mesa_base::~mm_mesa_base(void)
{
  finish();
}

void mm_mesa_base::finish(void)
{
  if ( !finishBase ) {
    assert( existInstance );
    visualInfo = NULL;
    existInstance = false;
    display.reset();
    finishBase = true;
  };
}

void mm_mesa_base::rectangle( int x1, int y1, int x2, int y2 ) const
{
  assert( !finishBase );
  glBegin( GL_LINE_LOOP );
  glVertex2i( x1, y1 );
  glVertex2i( x2, y1 );
  glVertex2i( x2, y2 );
  glVertex2i( x1, y2 );
  glEnd();
}

void mm_mesa_base::text( const string &s ) const
{
  assert( !finishBase );
  for ( int i=0; i<(signed)s.length(); i++ )
    glutStrokeCharacter( GLUT_STROKE_MONO_ROMAN, s[i] );
}

void mm_mesa_base::glWrap( bool condition, const char *message )
  throw (mm_exception)
{
  if ( !condition ) {
    mm_exception e;
    e << message;
    bool first = true;
    GLenum error;
    while ( ( error = glGetError() ) != GL_NO_ERROR ) {
      e << ( first ? ": " : "; " ) << gluErrorString( error );
      first = false;
    };
    throw e;
  };
}

};
