#include "mm_mesa_onscreen.h"

namespace mimas {

mm_mesa_onscreen::mm_mesa_onscreen( mm_x11_window_ptr _window )
  throw (mm_exception):
  mm_mesa_base( _window->get_display(), _window->get_visual_info() ),
  window(_window), finishOnScreen( false )
{
  // Create OpenGL rendering context for on-screen rendering.
  // Attention: Direct rendering is disabled (GL_FALSE), because it
  // doesn't work together properly with mm_mesa_offscreen in all
  // cases (f.e. 16-bit display with NVidia).
  context = glXCreateContext( window->get_display()->get(),
                              window->get_visual_info(), 0, GL_FALSE );
  glWrap( context != NULL, "Error creating GLX-context" );

  // Attach context to the window.
  glWrap( glXMakeCurrent( window->get_display()->get(), window->get_window(),
                          context ), "Error switching GLX-context" );
}

mm_mesa_onscreen::~mm_mesa_onscreen(void)
{
  finish();
}

void mm_mesa_onscreen::finish(void)
{
  if ( !finishOnScreen ) {

    glFinish();

    glXDestroyContext( window->get_display()->get(), context );
    context = NULL;
    finishOnScreen = true;
    window.reset();

  };

  mm_mesa_base::finish();

}

void mm_mesa_onscreen::ortho2D(void) const
{
  assert( !finishOnScreen );
  glLoadIdentity();
  glOrtho( 0.0, window->get_width(), window->get_height(), 0.0, -1.0, 1.0 );
}

};
