/*
*
*
* stumeikle Sun 27 Oct 2002 23:55:33 CET
* see header file for comments
* $Header: /cvs/mimas/lib/mm_pol_scene.cc,v 1.13 2004/12/12 23:31:19 engjw Exp $
*
*/

#include <fstream>
#include "mm_linalg.h"
#include "mm_image.h"
#include "mm_image_fileinput.h"
#include "mm_pol_scene.h"
#include "mm_canny_tool.h"
#include "mm_edge_string.h"
#include "mm_image_funcs.h"
#include "mm_slap_tool.h"
#include "mm_pol_locator_hough.h"
#include "mm_pol_locator_gen.h"
#include "mm_pol_locator_scale.h"
#include <list>

namespace mimas {
using namespace std;
using namespace boost;

void	mm_pol_scene::createFromImageFile( const char *fn )
{
    mm_image<int>	im;
    fstream file( fn, ios::binary );
    file >> im;
    createFromImage(im);
}

void	mm_pol_scene::createFromImage( mm_image<int> & im )
{
    mm_canny_tool<int>			canny;
    mm_edge_string_ptr_list		edge_strings;
    mm_slap_tool			slapper(M_PI/num_abins);

    edge_strings=canny.findEdgeStrings( im );
    mm_line_ptr_list			scene_lines;

    mm_edge_string_op::filterWiggles( edge_strings, 0.2, 1, 35.0 );
    scene_lines=slapper.findStraightLines( edge_strings );

//     mm_edge_string_ptr_list::iterator	sit;
//     for(sit=edge_strings.begin();sit!=edge_strings.end();++sit)
//         delete (*sit);
//     edge_strings.clear();

    createFromLines( scene_lines, im.getWidth(), im.getHeight() );

}

void 	mm_pol_scene::createFromLines( mm_line_ptr_list
                                     &scene_lines,
                                     int width, int height )
{
    //width and height refer to the size of the image plane on which the lines reside
    //for all the lines
    // double			slen1,slen2;
    // mm_angle			angle_diff;

    //as of Thu 24 Oct 2002 18:42:34 CEST we're gonna do all the necessary
    //calculations first and store in a new structure.
    //we'll also weed out the short lines here
    double		len;
    mm_line_ptr l;
    for( mm_line_ptr_list::iterator sl_it=scene_lines.begin();
         sl_it!=scene_lines.end();++sl_it)
    {
	l = *sl_it;

	len=l->length();
	if (len<min_len)
	    continue;

	mm_pol_line_info_ptr pli( new mm_pol_line_info );

	pli->len = len;
	pli->unitv= unit( l->getVector() );
	pli->midpoint=l->findMidPoint();
	pli->line= l;

	scene_line_info.push_back(pli);
    }

    //we can clear the original now i think
    //(doesn't delete the line structures)
    //scene_lines.clear();

    im_width=width;
    im_height=height;
}




mm_pol_match_ptr_list	mm_pol_scene::getMatches( mm_pol_model& m )
{
    if (m.scaleInvariant())
    {
        mm_pol_locator_scale	locate(m.getThreshold());
	locate.init(2, im_width, im_height, 1.0, 0.01 );
	makeEntries( m,locate );
	return locate.getMatches();
    }
    else
    {
	mm_pol_locator_hough	locate(m.getThreshold());
	locate.init(2, im_width, im_height, 1.0, 0.01 );
	makeEntries( m,locate );
	return locate.getMatches();
    }
}

mm_pol_match_ptr_list	mm_pol_scene::fastFind( mm_pol_model& m,mm_image<int>& im)
{
	mm_pol_match_ptr_list	l;
	return l;
}
/*
    //scan through the image for the model, assimilating all results in a single
    //hough
    //stumeikle 20030107

    //(1) first create the hough
    mm_pol_locator_hough	locate(m.getThreshold());
    locate.init(2, im.getWidth(), im.getHeight(), 1.0, 0.01 );

    int				xsize;
    int				ysize;
    mm_pol_line_info_ptr_list::iterator	i;
    xsize=ysize=(int)m.getRadius();

    //(2)scan through the image
    for( int y=0; y<im.getHeight(); y+=ysize)
    {
    	for( int x=0;x<im.getWidth();x+=xsize)
	{
	    mm_imageregion<int>	r( im, x,y, x+xsize-1, y+ysize-1 );

	    //creatre the list of lines we need...
            createFromImage(r);
	    locate.setOffset( x,y );
	    makeEntries(m,locate);

	    //delete the lines again
	    for(i=scene_line_info.begin(); i!=scene_line_info.end(); ++i)
	        delete (*i);

	    scene_line_info.clear();
	}
    }

    //(3) find the peaks in the normal way
//locate.display();

    return locate.getMatches();
}
*/

void			mm_pol_scene::makeEntries( mm_pol_model& m,
						   mm_pol_locator& locate)
{
    //we use all conceivable pairs of lines from the scene.
    //now we use all conceivable pairs of line info structs
    mm_pol_line_info_ptr_list::iterator	sl1_it,sl2_it;
    mm_pol_line_info_ptr sl1, sl2;
    int	s2,s1;
    // bool scale_inv = m.scaleInvariant();
    for(sl2_it=scene_line_info.begin(),s2=0;sl2_it!=scene_line_info.end();++sl2_it,++s2)
    {
	sl2=*sl2_it;

	//calculate some properties of sl2 here. we only want to do these
	//calculations once!
	for(sl1_it=scene_line_info.begin(),s1=0;
	    sl1_it!=scene_line_info.end();++sl1_it,++s1)
	{
	    sl1=*sl1_it;

	    if(sl1==sl2) continue;
	    if(s1>=s2)	 break;

	    //(! now we won't compare a and b as well as b and a
	    //check if the two lines are too far apart to be considered
	    //20030223
	    if ( norm_2( sl1->midpoint - sl2->midpoint ) > m.getRadius() )
	  	continue;

	    //ok, we have a good pair here.
	    //check if there are any or many matches and make entries to the
	    //hough
	    m.match(locate, *sl1, *sl2 );
	}
    }
}

}
