#include "mm_config.h"

#ifdef HAVE_XALAN

#include <boost/smart_ptr.hpp>
#include <XalanDOM/XalanDOMString.hpp>
#include <framework/LocalFileInputSource.hpp>
#include <framework/MemBufInputSource.hpp>
#include <util/PlatformUtils.hpp>
#ifdef SIMPLEXPATH
#include <XPath/XPathEvaluator.hpp>
#include <DOMSupport/XalanDocumentPrefixResolver.hpp>
#else
#include <XPath/XPath.hpp>
#include <XPath/ElementPrefixResolverProxy.hpp>
#include <XPath/XPathProcessorImpl.hpp>
#endif
#include "mm_xml_strx.h"
#include "mm_xml_document.h"

using namespace boost;
using namespace std;

// using namespace xercesc;
// using namespace xalanc;

// XALAN_USING_XERCES(InputSource);
// XALAN_USING_XERCES(XMLException);

namespace mimas {

mm_xml_document::mm_xml_document( const std::string &schemaURI ):
  theLiaison( theDOMSupport )
#ifndef SIMPLEXPATH
  , theExecutionContext( theEnvSupport, theDOMSupport, theXObjectFactory )
#endif
{
  theDOMSupport.setParserLiaison( &theLiaison );
  theLiaison.setErrorHandler( &xmlErrorHandler );
  if ( !schemaURI.empty() ) {
#ifndef NDEBUG
    cerr << "Specifying schema \"" << schemaURI.c_str() << "\"." << endl;
#endif
    theLiaison.setExternalSchemaLocation
      ( mm_xml_strx<XMLCh,char>( schemaURI.c_str() ).getBuffer() );
    theLiaison.setUseValidation( true );
  };
  theLiaison.setDoNamespaces( true );
};

void mm_xml_document::load( const char *fileName ) throw (mm_exception)
{
  // XALAN_USING_XERCES(LocalFileInputSource)
  try {
    LocalFileInputSource
      localFileInputSource( mm_xml_strx<XMLCh,char>( fileName ).getBuffer() );
    fromInputSource( localFileInputSource );
  } catch ( XMLException &exc ) {
    MMERROR( false, mm_exception, ,
             "Could not load XML file \"" << fileName << "\": "
             << ( mm_xml_strx<char,XMLCh>( exc.getMessage() ).getBuffer() ) );
  };
};

void mm_xml_document::save( const char *fileName ) throw (mm_exception)
{
  assert( false );
  MMERROR( false, mm_exception, ,
           "Saving of XML-documents not implemented yet." );
}

void mm_xml_document::fromString( const string &string ) throw (mm_exception)
{
  // XALAN_USING_XERCES(MemBufInputSource);
  MemBufInputSource memBufInputSource
    ( (const XMLByte *)string.c_str(), string.size(),
      "XML string in memory" );
  fromInputSource( memBufInputSource );
}

void mm_xml_document::fromInputSource( InputSource &inputSource )
  throw (mm_exception)
{
  document = theLiaison.parseXMLStream( inputSource );
  if ( xmlErrorHandler.getErrorOccurred() ) {
    MMERROR( false, mm_exception, ,
             "Error reading XML file: "
             << xmlErrorHandler.getErrorMessage().c_str() );
  };
  assert( document != NULL );
}

istream &operator>>( istream &stream, mm_xml_document &document )
{
  // No proper solution at the moment.
  streampos p = stream.tellg();
  stream.seekg( 0, ios::end );
  int size = stream.tellg() - p;
  stream.seekg( p );
  shared_array< char > data( new char[ size ] );
  stream.read( data.get(), size );

  MemBufInputSource source( (const XMLByte *)data.get(), size,
                            "std::istream object" );
  document.fromInputSource( source );
  return stream;
}

};

#endif
