#include <iostream>
#include <cmath>
#include <valarray>

#include "mm_image.h"

// Denis Tessier Sep 2004

using namespace std;
using namespace mimas;

mm_image<int> rotate_mm_image(mm_image<int>& in_image, double angle, double scale, int background)
{
  mm_image<int> out_image;
  
  int nx, ny, newheight, newwidth, i,j , halfnewheight, halfnewwidth;
  int halfoldheight,halfoldwidth;
  
  double radians; 
  double cosval,sinval;

  radians =  -(angle) / (180 / M_PI);
  cosval = scale*cos(radians);
  sinval = scale*sin(radians);

  newwidth = (int)abs((int)(in_image.getWidth()*cosval)) + (int)abs((int)(in_image.getHeight()*sinval));
  newheight = (int)abs((int)(-in_image.getWidth()*sinval)) + (int)abs((int)(in_image.getHeight()*cosval));

  halfnewheight = newheight / 2;
  halfnewwidth = newwidth / 2;
  halfoldwidth = in_image.getWidth() /2;
  halfoldheight = in_image.getHeight() /2 ;

  out_image.init(newwidth, newheight);
  out_image.fill( background );

  for(i=0;i < newwidth;i++)
  {
    for(j=0;j < newheight;j++)
    {
       nx =(int)( (i - halfnewwidth)*cosval + (j - halfnewheight)*sinval);
       ny =(int)( (halfnewwidth - i)*sinval + (j - halfnewheight)*cosval);
       nx = nx + halfoldwidth;
       ny = ny + halfoldheight;
       if ((nx < in_image.getWidth()) && (ny < in_image.getHeight()) && (nx > 0) && (ny > 0))
	 out_image.setPixel(i,j, in_image.getPixel(nx,ny)); 
//        else if (nx >= in_image.getWidth())
// 	 out_image.setPixel(i,j,125);
//        else if (ny >= in_image.getHeight())
// 	 out_image.setPixel(i,j,100);
//        else if (nx <= 0)
// 	 out_image.setPixel(i,j,150);
//        else //if (ny <= 0)
// 	 out_image.setPixel(i,j,175);
    }
  }
  return out_image;
}


mm_image<int> rotate_mm_image2(mm_image<int>& in_image, double angle, double scale, int background)
{
  mm_image<int> out_image;
  
  int newheight, newwidth, i,j , halfnewheight, halfnewwidth;
  int halfoldheight,halfoldwidth;
  double x, y;  

  double radians; 
  double cosval,sinval;

  radians =  -(angle) / (180 / M_PI);
  cosval = scale * cos(radians);
  sinval = scale * sin(radians);

  newwidth = (int)abs((int)(in_image.getWidth()*cosval)) + (int)abs((int)(in_image.getHeight()*sinval));
  newheight = (int)abs((int)(-in_image.getWidth()*sinval)) + (int)abs((int)(in_image.getHeight()*cosval));

  halfnewheight = newheight / 2;
  halfnewwidth = newwidth / 2;
  halfoldwidth = in_image.getWidth() /2;
  halfoldheight = in_image.getHeight() /2 ;

  //  out_image.init(newheight, newwidth);
  out_image.init(newwidth, newheight);
  out_image.fill( background );

  for(i=0; i < newwidth; i++)
  {
    for(j=0; j < newheight; j++)
    {
      x = (i - halfnewwidth)*cosval + (j - halfnewheight)*sinval;
      y = (halfnewwidth - i)*sinval + (j - halfnewheight)*cosval;
      x += halfoldwidth;
      y += halfoldheight;
      if ((ceil(x) < in_image.getWidth()) && (ceil(y) < in_image.getHeight()) && (x > 0) && (y > 0))
	{
	  double t, u;
	  int k, l;
	  int val;
	  k = (int) floor(x);
	  l = (int) floor(y);
	  t = x - k;
	  u = y - l;
	  
	  val = (int) ((1-t)*(1-u)*in_image.getPixel(k, l) + t*(1-u)*in_image.getPixel(k+1, l) + (1-t)*u*in_image.getPixel(k, l+1) + t*u*in_image.getPixel(k+1, l+1)); 
	  
	  out_image.setPixel(i, j, val); 
	}
    }
  }
  return out_image;
}
